@@ -135,20 +135,38 @@
 
 #include <Foundation/NSObject.h>
 #include <Foundation/NSGeometry.h>
-#include "AppKit/NSCell.h"
+#include <AppKit/NSCell.h>
 // For gradient types
-#include "AppKit/NSButtonCell.h"
+#include <AppKit/NSButtonCell.h>
 // For image frame style
-#include "AppKit/NSImageCell.h"
+#include <AppKit/NSImageCell.h>
+// For scroller constants
+#include <AppKit/NSScroller.h>
 
 #if	OS_API_VERSION(GS_API_NONE,GS_API_NONE)
 @class NSArray;
 @class NSBundle;
+@class NSButton;
 @class NSColor;
+@class NSColorList;
 @class NSDictionary;
 @class NSImage;
+@class NSMenuItemCell;
+@class NSMenuView;
 @class GSDrawTiles;
 
+/* Names for the component parts of a scroller,
+ * allowing tiles and/or colors to be set for them.
+ */
+APPKIT_EXPORT	NSString	*GSScrollerDownArrow;
+APPKIT_EXPORT	NSString	*GSScrollerHorizontalKnob;
+APPKIT_EXPORT	NSString	*GSScrollerHorizontalSlot;
+APPKIT_EXPORT	NSString	*GSScrollerLeftArrow;
+APPKIT_EXPORT	NSString	*GSScrollerRightArrow;
+APPKIT_EXPORT	NSString	*GSScrollerUpArrow;
+APPKIT_EXPORT	NSString	*GSScrollerVerticalKnob;
+APPKIT_EXPORT	NSString	*GSScrollerVerticalSlot;
+
 /**
  * This defines how the values in a tile array should be used when
  * drawing a rectangle.  Mostly this just effects the center, middle
@@ -169,21 +187,43 @@ typedef enum {
  * This enumeration provides constants for informing drawing methods
  * what state a control is in (and consequently how the display element
  * being drawn should be presented).
+ * NB. GSThemeNormalState must be 0 and GSThemeSelectedState must be the
+ * last state, in order to allow code to iterate through all the states.
  */
 typedef enum {
-  GSThemeNormalState,		/** A control in its normal state */
+  GSThemeNormalState = 0,	/** A control in its normal state */
   GSThemeHighlightedState,	/** A control which is highlighted */
-  GSThemeSelectedState,		/** A control which is selected */
+  GSThemeSelectedState		/** A control which is selected */
 } GSThemeControlState;
 
-/** Notification sent when a theme has just become active.
+/** Notification sent when a theme has just become active.<br />
+ * The notification is posted by the -activate method.<br />
+ * This is primarily for internal use by AppKit controls which
+ * need to readjust how they are displayed when a new theme is in use.
  */
 APPKIT_EXPORT	NSString	*GSThemeDidActivateNotification;
 
-/** Notification sent when a theme has become inactive.
+/** Notification sent when a theme has just become inactive.<br />
+ * The notification is posted by the -deactivate method.<br />
+ * This is primarily for use by subclasses of GSTheme which need to perform
+ * additional cleanup when the theme stops being used.
  */
 APPKIT_EXPORT	NSString	*GSThemeDidDeactivateNotification;
 
+/** Notification sent when a theme is about to become active.<br />
+ * The notification is posted by the -activate method.<br />
+ * This is primarily for use by subclasses of GSTheme which need to perform
+ * additional setup before the theme starts being used by the AppKit controls.
+ */
+APPKIT_EXPORT	NSString	*GSThemeWillActivateNotification;
+
+/** Notification sent when a theme is about to become inactive.<br />
+ * The notification is posted by the -deactivate method.<br />
+ * This allows code to make preparatory changes before the current theme
+ * is deactivated.
+ */
+APPKIT_EXPORT	NSString	*GSThemeWillDeactivateNotification;
+
 
 /**
   <p><em>This interface is <strong>HIGHLY</strong> unstable
@@ -193,7 +233,7 @@ APPKIT_EXPORT	NSString	*GSThemeDidDeactivateNotification;
   This is a class used for 'theming', which is mostly a matter of
   encapsulating common drawing behaviors so that GUI appearance can
   be easily modified, but also includes mechanisms for altering
-  some GUI behavior (such mas orientation and position of menus).
+  some GUI behavior (such as orientation and position of menus).
   </p>
   <p>
   Methods in this class standardize drawing of buttons, borders
@@ -225,10 +265,7 @@ APPKIT_EXPORT	NSString	*GSThemeDidDeactivateNotification;
 @interface GSTheme : NSObject
 {
 @private
-  NSBundle		*_bundle;
-  NSMutableDictionary	*_images;
-  NSMutableDictionary	*_tiles;
-  NSImage		*_icon;
+  void		*_reserved;
 }
 
 /**
@@ -261,10 +298,9 @@ APPKIT_EXPORT	NSString	*GSThemeDidDeactivateNotification;
  * <p>The base implementation handles setup and caching of the system
  * color list, standard image information, tiling information,
  * and user defaults.<br />
- * It then sends a GSThemeDidActivateNotification to allow other
- * parts of the GUI library to update themselves from the new theme.<br />
- * If the theme sets an alternative system color list, the notification
- * userInfo dictionary will contain that list keyed on <em>Colors</em>.
+ * It then sends a GSThemeWillActivateNotification and a
+ * GSThemeDidActivateNotification to allow other
+ * parts of the GUI library to update themselves from the new theme.
  * </p>
  * <p>Finally, this method marks all windows in the application as needing
  * update ... so they will draw themselves with the new theme information.
@@ -282,6 +318,25 @@ APPKIT_EXPORT	NSString	*GSThemeDidDeactivateNotification;
  */
 - (NSBundle*) bundle;
 
+/**
+ * This returns the color for drawing the item whose name is aName in
+ * the specified state.  If aName is nil or if there is no color defined
+ * for the particular combination of item name and state, the method
+ * returns nil.<br />
+ * The useCache argument controls whether the information is retrieved
+ * from cache or regenerated from information in the theme bundle.
+ */
+- (NSColor*) colorNamed: (NSString*)aName
+		  state: (GSThemeControlState)elementState
+	          cache: (BOOL)useCache; 
+
+/**
+ * Returns the system color list defined by the receiver.<br />
+ * The default implementation returns the color list provided in the
+ * theme bundle (if any) or the default system color list.
+ */
+- (NSColorList*) colors;
+
 /**
  * <p>This method is called automatically when the receiver is stopped from
  * being the currently active theme by the use of the +setTheme: method
@@ -349,6 +404,35 @@ APPKIT_EXPORT	NSString	*GSThemeDidDeactivateNotification;
  */
 - (NSString*) name;
 
+/** Returns the name used to locate theming resources for a particular gui
+ * element.  If no name has been set for the particular object this method
+ * returns nil.
+ */
+- (NSString*) nameForElement: (id)anObject;
+
+/** Set the name of this theme ... used for testing by Thematic.app
+ */
+- (void) setName: (NSString*)aString;
+
+/** Set the name that is used to identify theming resources for a particular
+ * control or other gui element.  This is used so that where an element is
+ * part of a control it can be displayed differently from the same class of
+ * element used outside that control.<br />
+ * Supplying a nil value for aString simply removes any name setting for
+ * anObject.<br />
+ * Supplying nil for anObject is illegal (raises an exception) unless
+ * the value of aString is also nil (and the method does nothing).<br />
+ * Any control which uses this method to set names for subsidiary elements
+ * must also make sure to remove the name mapping before that element is
+ * deallocated, unless the takeOwnership option is YES, in which case
+ * anObject is retained, the name mapping lasts only until the receiver
+ * is deactivated, and at that point anObject is released.
+ */
+- (void) setName: (NSString*)aString
+      forElement: (id)anObject
+       temporary: (BOOL)takeOwnership;
+	
+
 /**
  * <p>Provides a standard inspector window used to display information about
  * the receiver.  The default implementation displays the icon, the name,
@@ -363,15 +447,19 @@ APPKIT_EXPORT	NSString	*GSThemeDidDeactivateNotification;
 
 /**
  * Returns the tile image information for a particular image name,
- * or nil if there is no such information.<br />
+ * or nil if there is no such information or the name is nil.<br />
  * The GUI library uses this internally to handling tiling of image
  * information to draw user interface elements.  The tile information
  * returned by this method can be passed to the
  * -fillRect:withTiles:background:fillStyle: method.<br />
+ * The elementState argument specifies the state for which tiles are
+ * requested.<br />
  * The useCache argument controls whether the information is retrieved
  * from cache or regenerated from information in the theme bundle.
  */
-- (GSDrawTiles*) tilesNamed: (NSString*)aName cache: (BOOL)useCache; 
+- (GSDrawTiles*) tilesNamed: (NSString*)aName
+		      state: (GSThemeControlState)elementState
+		      cache: (BOOL)useCache; 
 @end
 
 /**
@@ -392,8 +480,9 @@ APPKIT_EXPORT	NSString	*GSThemeDidDeactivateNotification;
 /**
  * Amount by which the button is inset by the border.
  */
-- (NSSize) buttonBorderForStyle: (int)style 
-                          state: (GSThemeControlState)state;
+- (NSSize) buttonBorderForCell: (NSCell*)cell
+			 style: (int)style 
+                         state: (GSThemeControlState)state;
 
 /** 
  * Draws the indicator (normally a dotted rectangle) to show that
@@ -431,6 +520,21 @@ APPKIT_EXPORT	NSString	*GSThemeDidDeactivateNotification;
  */
 - (NSSize) sizeForImageFrameStyle: (NSImageFrameStyle)frameStyle;
 
+
+/** Methods for scroller theming.
+ */
+- (NSButtonCell*) cellForScrollerArrow: (NSScrollerArrow)part
+			    horizontal: (BOOL)horizontal;
+- (NSCell*) cellForScrollerKnob: (BOOL)horizontal;
+- (NSCell*) cellForScrollerKnobSlot: (BOOL)horizontal;
+- (float) defaultScrollerWidth;
+
+/** Method for toolbar theming.
+ */
+- (void) drawToobarRect: (NSRect)aRect
+                  frame: (NSRect)viewFrame
+             borderMask: (unsigned int)borderMask;
+
 @end
 
 /**
@@ -528,14 +632,5 @@ withRepeatedImage: (NSImage*)image
 		  flipped: (BOOL)flipped;
 @end
 
-//
-// Function which should be somewhere else
-//
-static inline NSSize 
-_sizeForBorderType (NSBorderType aType)
-{
-  return [[GSTheme theme] sizeForBorderType: aType];
-}
-
 #endif /* OS_API_VERSION */
 #endif /* _GNUstep_H_GSTheme */
