@@ -13,19 +13,20 @@
    This file is part of the GNUstep GUI Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
-   
+
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+   Lesser General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; see the file COPYING.LIB.
-   If not, write to the Free Software Foundation,
-   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+   If not, see <http://www.gnu.org/licenses/> or write to the 
+   Free Software Foundation, 51 Franklin Street, Fifth Floor, 
+   Boston, MA 02110-1301, USA.
 */ 
 
 #ifndef _GSToolbarView_h_INCLUDE
@@ -35,11 +36,9 @@
 #include <AppKit/NSView.h>
 #include <AppKit/NSColor.h>
 
-#include "GNUstepGUI/GSToolbar.h"
-// Necessary for NSToolbarDisplayMode and NSToolbarSizeMode
-
 @class NSMutableArray;
 @class NSClipView;
+@class NSToolbar;
 @class NSToolbarItem;
 @class GSToolbarClippedItemsButton;
 
@@ -53,26 +52,20 @@ typedef enum {
 
 @interface GSToolbarView : NSView
 {
-  GSToolbar *_toolbar;
-  NSClipView *_clipView, *_clipViewForEditMode;
+  NSToolbar *_toolbar;
+  NSClipView *_clipView;
   GSToolbarClippedItemsButton *_clippedItemsMark;
   NSMutableArray *_visibleBackViews;
-  BOOL _willBeVisible;
   unsigned int _borderMask;
-  NSToolbarDisplayMode _displayMode;
-  NSToolbarSizeMode _sizeMode;
   NSRect _rectAvailable;
   float _heightFromLayout;
 }
 
 - (id) initWithFrame: (NSRect)frame;
-- (id) initWithFrame: (NSRect)frame 
-         displayMode: (NSToolbarDisplayMode)displayMode 
-	    sizeMode: (NSToolbarSizeMode)sizeMode; 
 
 // Accessors
-- (GSToolbar *) toolbar;
-- (void) setToolbar: (GSToolbar *)toolbar;
+- (NSToolbar *) toolbar;
+- (void) setToolbar: (NSToolbar *)toolbar;
 - (unsigned int) borderMask;
 - (void) setBorderMask: (unsigned int)borderMask;
 
