@@ -156,6 +156,7 @@
 #include <AppKit/NSSearchFieldCell.h>
 #include <AppKit/NSSecureTextField.h>
 #include <AppKit/NSSound.h>
+#include <AppKit/NSSpeechSynthesizer.h>
 #include <AppKit/NSStepper.h>
 #include <AppKit/NSStepperCell.h>
 #include <AppKit/NSTableColumn.h>
@@ -175,7 +176,6 @@
 #include <AppKit/NSUserDefaultsController.h>
 #include <AppKit/NSUserInterfaceValidation.h>
 #include <AppKit/NSWindowController.h>
-#include <AppKit/NSWindow+Toolbar.h>
 #endif
 
 #include <AppKit/PSOperators.h>
