@@ -30,7 +30,7 @@
 
 #include <Foundation/NSObject.h>
 
-#if OS_API_VERSION(100300,GS_API_LATEST)
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_3, GS_API_LATEST)
 
 @class NSMutableArray;
 
@@ -50,6 +50,12 @@
 - (void) objectDidBeginEditing: (id)editor;
 - (void) objectDidEndEditing: (id)editor;
 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_4, GS_API_LATEST)
+- (void) commitEditingWithDelegate: (id)delegate
+                 didCommitSelector: (SEL)didCommitSelector
+                       contextInfo: (void*)contextInfo;
+#endif
+
 @end
 
 #endif // OS_API_VERSION
