@@ -108,7 +108,9 @@ typedef enum _NSSaveOperationType {
     struct __docFlags {
         unsigned int in_close:1;
         unsigned int has_undo_manager:1;
-        unsigned int RESERVED:30;
+        unsigned int permanently_modified:1;
+        unsigned int autosave_permanently_modified:1;
+        unsigned int RESERVED:28;
     } _doc_flags;
     void 		*_reserved1;
 }
@@ -148,7 +150,7 @@ withContentsOfURL:(NSURL *)url
 - (void)showWindows;
 - (void)removeWindowController:(NSWindowController *)windowController;
 - (void)setWindow:(NSWindow *)aWindow;
-#if OS_API_VERSION(MAC_OS_X_VERSION_10_4, GS_API_LATEST)
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_3, GS_API_LATEST)
 - (NSWindow *)windowForSheet;
 #endif
 
@@ -247,7 +249,7 @@ originalContentsURL:(NSURL *)orig
 - (BOOL)shouldRunSavePanelWithAccessoryView;
 #if OS_API_VERSION(GS_API_MACOSX, MAC_OS_X_VERSION_10_4)
 - (NSString *)fileNameFromRunningSavePanelForSaveOperation:(NSSaveOperationType)saveOperation;
-- (int)runModalSavePanel:(NSSavePanel *)savePanel withAccessoryView:(NSView *)accessoryView;
+- (NSInteger)runModalSavePanel:(NSSavePanel *)savePanel withAccessoryView:(NSView *)accessoryView;
 #endif 
 - (NSString *)fileTypeFromLastRunSavePanel;
 - (NSDictionary *)fileAttributesToWriteToFile: (NSString *)fullDocumentPath 
@@ -267,15 +269,21 @@ originalContentsURL:(NSURL *)orig
                             forSaveOperation:(NSSaveOperationType)op
                          originalContentsURL:(NSURL *)original
                                        error:(NSError **)error;
+#endif 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_1, GS_API_LATEST)
 - (BOOL)fileNameExtensionWasHiddenInLastRunSavePanel;
 #endif 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_5, GS_API_LATEST)
+- (NSString *)fileNameExtensionForType:(NSString *)typeName
+                         saveOperation:(NSSaveOperationType)saveOperation;
+#endif 
 
 /*" Printing "*/
 - (NSPrintInfo *)printInfo;
 - (void)setPrintInfo:(NSPrintInfo *)printInfo;
 - (BOOL)shouldChangePrintInfo:(NSPrintInfo *)newPrintInfo;
 - (IBAction)runPageLayout:(id)sender;
-- (int)runModalPageLayoutWithPrintInfo:(NSPrintInfo *)printInfo;
+- (NSInteger)runModalPageLayoutWithPrintInfo:(NSPrintInfo *)printInfo;
 - (IBAction)printDocument:(id)sender;
 - (void)printShowingPrintPanel:(BOOL)flag;
 #if OS_API_VERSION(MAC_OS_X_VERSION_10_4, GS_API_LATEST)
