@@ -1,5 +1,4 @@
-/* 	-*-ObjC-*- */
-/** 
+/** -*-ObjC-*- 
    Copyright (C) 2002 Free Software Foundation, Inc.
 
    Author: Frederic De Jaeger
@@ -8,22 +7,25 @@
    This file is part of the GNU Objective C User interface library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
-   
+
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-   
-   You should have received a copy of the GNU Library General Public
-   License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
-   */
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+   Lesser General Public License for more details.
+
+   You should have received a copy of the GNU Lesser General Public
+   License along with this library; see the file COPYING.LIB.
+   If not, see <http://www.gnu.org/licenses/> or write to the 
+   Free Software Foundation, 51 Franklin Street, Fifth Floor, 
+   Boston, MA 02110-1301, USA.
+*/
 
 #ifndef _NSOpenGL_h_INCLUDE
 #define _NSOpenGL_h_INCLUDE
+#import <GNUstepBase/GSVersionMacros.h>
 
 #include <Foundation/NSObject.h>
 @class NSView;
@@ -42,9 +44,13 @@ typedef enum {
   NSOpenGLPFAMaximumPolicy = 52,
   NSOpenGLPFAOffScreen = 53,
   NSOpenGLPFAFullScreen = 54,
-  NSOpenGLPFASampleBuffers = 55,
-  NSOpenGLPFASamples = 56,
-  NSOpenGLPFAAuxDepthStencil = 57,
+  NSOpenGLPFASampleBuffers = 55,        // 10.2
+  NSOpenGLPFASamples = 56,              // 10.2
+  NSOpenGLPFAAuxDepthStencil = 57,      // 10.2
+  NSOpenGLPFAColorFloat = 58,           // 10.4
+  NSOpenGLPFAMultisample = 59,          // 10.4
+  NSOpenGLPFASupersample = 60,          // 10.4
+  NSOpenGLPFASampleAlpha = 61,          // 10.4
   NSOpenGLPFARendererID = 70,
   NSOpenGLPFASingleRenderer = 71,
   NSOpenGLPFANoRecovery = 72,
@@ -57,7 +63,9 @@ typedef enum {
   NSOpenGLPFAMultiScreen = 81,
   NSOpenGLPFACompliant = 83,
   NSOpenGLPFAScreenMask = 84,
-  NSOpenGLPFAVirtualScreenCount = 128
+  NSOpenGLPFAPixelBuffer = 90,          // 10.3
+  NSOpenGLPFAAllowOfflineRenderers = 96,// 10.5
+  NSOpenGLPFAVirtualScreenCount = 128   // 10.2
 } NSOpenGLPixelFormatAttribute;
 
 typedef enum {
