@@ -15,19 +15,19 @@
    This file is part of the GNUstep GUI Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
-   
+
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+   Lesser General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; see the file COPYING.LIB.
-   If not, write to the Free Software Foundation,
-   51 Franklin Street, Fifth Floor,
+   If not, see <http://www.gnu.org/licenses/> or write to the 
+   Free Software Foundation, 51 Franklin Street, Fifth Floor, 
    Boston, MA 02110-1301, USA.
 */ 
 
@@ -47,7 +47,8 @@
   NSWindowDepth        _depth;
   NSRect               _frame;
   int                  _screenNumber;
-  NSWindowDepth       *_supportedWindowDepths;
+  NSWindowDepth		*_supportedWindowDepths;
+  void			*_reserved;
 }
 
 /*
@@ -56,7 +57,7 @@
 + (NSScreen*) mainScreen;
 + (NSScreen*) deepestScreen;
 + (NSArray*) screens;
-+ (void)resetScreens;
++ (void) resetScreens;
 
 /*
  * Reading Screen Information
@@ -72,6 +73,9 @@
 #if OS_API_VERSION(GS_API_NONE, GS_API_NONE)
 - (int) screenNumber;
 #endif
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_4, GS_API_LATEST)
+- (float) userSpaceScaleFactor;
+#endif
 
 @end
 #endif // _GNUstep_H_NSScreen
