@@ -11,19 +11,19 @@
    This file is part of the GNUstep GUI Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
-   
+
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+   Lesser General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; see the file COPYING.LIB.
-   If not, write to the Free Software Foundation,
-   51 Franklin Street, Fifth Floor,
+   If not, see <http://www.gnu.org/licenses/> or write to the 
+   Free Software Foundation, 51 Franklin Street, Fifth Floor, 
    Boston, MA 02110-1301, USA.
 */ 
 
@@ -37,15 +37,15 @@
 
 enum
 {
-    NSSearchFieldRecentsTitleMenuItemTag,
-    NSSearchFieldRecentsMenuItemTag,
-    NSSearchFieldClearRecentsMenuItemTag,
-    NSSearchFieldNoRecentsMenuItemTag
+    NSSearchFieldRecentsTitleMenuItemTag = 1000,
+    NSSearchFieldRecentsMenuItemTag = 1001,
+    NSSearchFieldClearRecentsMenuItemTag = 1002,
+    NSSearchFieldNoRecentsMenuItemTag = 1003
 };
 
 @interface NSSearchFieldCell : NSTextFieldCell
 {
-    NSArray *_recent_searches;
+    NSMutableArray *_recent_searches;
     NSString *_recents_autosave_name;
     NSButtonCell *_search_button_cell;
     NSButtonCell *_cancel_button_cell;
