@@ -3,7 +3,7 @@
 
    The window class
 
-   Copyright (C) 1996,1999 Free Software Foundation, Inc.
+   Copyright (C) 1996,1999,2004 Free Software Foundation, Inc.
 
    Author:  Scott Christley <scottc@net-community.com>
    Date: 1996
@@ -11,6 +11,8 @@
    Date: June 1998
    Modified:  Richard Frith-Macdonald <richard@brainstorm.co.uk>
    Date:  1998,1999
+   Author:  Quentin Mathe <qmathe@club-internet.fr>
+   Date: January 2004
 
    This file is part of the GNUstep GUI Library.
 
@@ -244,7 +246,7 @@ APPKIT_EXPORT NSSize NSTokenSize;
     unsigned preserves_content_during_live_resize: 1;
   } _f;
  
-  void          *_reserved_s;
+  NSToolbar     *_toolbar;
   void          *_reserved_1;
 }
 
@@ -746,6 +748,15 @@ APPKIT_EXPORT NSSize NSTokenSize;
 
 @end
 
+@class NSToolbar;
+
+@interface NSWindow (Toolbar)
+- (void) runToolbarCustomizationPalette: (id)sender;
+- (void) toggleToolbarShown: (id)sender;
+- (NSToolbar *) toolbar;
+- (void) setToolbar: (NSToolbar*)toolbar;
+@end
+
 #if OS_API_VERSION(GS_API_NONE, GS_API_NONE)
 /*
  * GNUstep backend methods
@@ -803,6 +814,9 @@ APPKIT_EXPORT NSSize NSTokenSize;
 willPositionSheet: (NSWindow *)sheet
         usingRect: (NSRect)rect;
 #endif
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_1, GS_API_LATEST)
+- (NSWindow *) attachedSheet;
+#endif
 - (NSSize) windowWillResize: (NSWindow*)sender
 		     toSize: (NSSize)frameSize;
 - (id) windowWillReturnFieldEditor: (NSWindow*)sender
