@@ -896,6 +896,12 @@ GSCurrentServer(void)
   [self subclassResponsibility: _cmd];
 }
 
+- (void) setParentWindow: (int)partentWin 
+          forChildWindow: (int)childWin
+{
+  [self subclassResponsibility: _cmd];
+}
+
 @end
 
 /* ----------------------------------------------------------------------- */
