@@ -1137,16 +1137,6 @@ Fills in all glyph holes up to last. only looking at levels below level
     }
 #endif
 
-  /* TODO: this case is also dubious, but it makes sense to return like this,
-  so it's mostly the caller's fault */
-  if (charRange.length == 0)
-    {
-      NSLog(@"Warning: %s called with zero-length range", __PRETTY_FUNCTION__);
-      if (actualCharRange)
-        *actualCharRange = NSMakeRange(0, 0);
-      return NSMakeRange(0, 0);
-    }
-
   pos = NSMaxRange(charRange) - 1;
   [self _generateGlyphsUpToCharacter: pos];
   if (glyphs->char_length <= pos)
@@ -2356,14 +2346,24 @@ forStartOfGlyphRange: (NSRange)glyphRange
 
 #undef SETUP_STUFF
 
+- (NSTextContainer *) textContainerForGlyphAtIndex: (NSUInteger)glyphIndex
+                                    effectiveRange: (NSRange *)effectiveRange
+{
+  return [self textContainerForGlyphAtIndex: glyphIndex
+               effectiveRange: effectiveRange
+               withoutAdditionalLayout: NO];
+}
 
-- (NSTextContainer *) textContainerForGlyphAtIndex: (unsigned int)glyphIndex
-				    effectiveRange: (NSRange *)effectiveRange
+- (NSTextContainer *) textContainerForGlyphAtIndex: (NSUInteger)glyphIndex
+                                    effectiveRange: (NSRange *)effectiveRange
+                           withoutAdditionalLayout: (BOOL)flag
 {
   textcontainer_t *tc;
   int i;
 
-  [self _doLayoutToGlyph: glyphIndex];
+  if (!flag)
+    [self _doLayoutToGlyph: glyphIndex];
+
   for (i = 0, tc = textcontainers; i < num_textcontainers; i++, tc++)
     if (tc->pos + tc->length > glyphIndex)
       break;
@@ -2382,14 +2382,25 @@ forStartOfGlyphRange: (NSRange)glyphRange
   return tc->textContainer;
 }
 
-- (NSRect) lineFragmentRectForGlyphAtIndex: (unsigned int)glyphIndex
+- (NSRect) lineFragmentRectForGlyphAtIndex: (NSUInteger)glyphIndex
 			    effectiveRange: (NSRange *)effectiveGlyphRange
+{
+  return [self lineFragmentRectForGlyphAtIndex: glyphIndex
+               effectiveRange: effectiveGlyphRange
+               withoutAdditionalLayout: NO];
+}
+
+- (NSRect) lineFragmentRectForGlyphAtIndex: (NSUInteger)glyphIndex
+                            effectiveRange: (NSRange *)effectiveGlyphRange
+                   withoutAdditionalLayout: (BOOL)flag
 {
   int i;
   textcontainer_t *tc;
   linefrag_t *lf;
 
-  [self _doLayoutToGlyph: glyphIndex];
+  if (!flag)
+    [self _doLayoutToGlyph: glyphIndex];
+
   for (i = 0, tc = textcontainers; i < num_textcontainers; i++, tc++)
     if (tc->pos + tc->length > glyphIndex)
       break;
@@ -2415,14 +2426,25 @@ forStartOfGlyphRange: (NSRange)glyphRange
   return lf->rect;
 }
 
-- (NSRect) lineFragmentUsedRectForGlyphAtIndex: (unsigned int)glyphIndex
+- (NSRect) lineFragmentUsedRectForGlyphAtIndex: (NSUInteger)glyphIndex
 				effectiveRange: (NSRange *)effectiveGlyphRange
+{
+  return [self lineFragmentUsedRectForGlyphAtIndex: glyphIndex
+               effectiveRange: effectiveGlyphRange
+               withoutAdditionalLayout: NO];
+}
+
+- (NSRect) lineFragmentUsedRectForGlyphAtIndex: (NSUInteger)glyphIndex
+                                effectiveRange: (NSRange *)effectiveGlyphRange
+                       withoutAdditionalLayout: (BOOL)flag
 {
   int i;
   textcontainer_t *tc;
   linefrag_t *lf;
 
-  [self _doLayoutToGlyph: glyphIndex];
+  if (!flag)
+    [self _doLayoutToGlyph: glyphIndex];
+
   for (i = 0, tc = textcontainers; i < num_textcontainers; i++, tc++)
     if (tc->pos + tc->length > glyphIndex)
       break;
