@@ -51,6 +51,7 @@
 #include <Foundation/NSKeyedArchiver.h>
 #include "AppKit/AppKit.h"
 #include <GNUstepBase/GSObjCRuntime.h>
+#include "GSGuiPrivate.h"
 
 /*
  *      Setup for inline operation of arrays.
@@ -95,11 +96,14 @@ static BOOL _isInInterfaceBuilder = NO;
 @end
 
 @interface NSMenu (NibCompatibility)
-- (void) _setGeometry;
 - (void) _setMain: (BOOL)isMain;
 @end
+@interface	NSMenu (GNUstepPrivate)
+- (void) _setGeometry;
+@end
 
 @implementation NSMenu (NibCompatibility)
+// FIXME: Why can't this be merged with setMain: ?
 - (void) _setMain: (BOOL)isMain
 {
   if (isMain)
@@ -107,13 +111,11 @@ static BOOL _isInInterfaceBuilder = NO;
       NSMenuView	*oldRep;
       NSInterfaceStyle	oldStyle;
       NSInterfaceStyle	newStyle;
-      NSMenuItem        *appItem;
       NSString          *processName;
 
       if([self numberOfItems] == 0)
-	return;
+        return;
 
-      appItem = [self itemAtIndex: 0]; // Info item.
       oldRep = [self menuRepresentation];
       oldStyle = [oldRep interfaceStyle];
       newStyle = NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", nil);
@@ -127,8 +129,8 @@ static BOOL _isInInterfaceBuilder = NO;
 	  NSMenuView	*newRep;
 
 	  newRep = [[NSMenuView alloc] initWithFrame: NSZeroRect];
-	  if (newStyle == NSMacintoshInterfaceStyle ||
-	      newStyle == NSWindows95InterfaceStyle)
+	  if (newStyle == NSMacintoshInterfaceStyle
+	    || newStyle == NSWindows95InterfaceStyle)
 	    {
 	      [newRep setHorizontal: YES];
 	    }
@@ -146,19 +148,22 @@ static BOOL _isInInterfaceBuilder = NO;
 
       // if it's a standard menu, transform it to be more NeXT'ish/GNUstep-like
       if(_menu.horizontal == NO)
-	{
-	  NSString *infoString = NSLocalizedString (@"Info", @"Info");
-	  NSString *quitString = [NSString stringWithFormat: @"%@ %@", 
-					   NSLocalizedString (@"Quit", @"Quit"), processName];
-	  NSMenuItem *quitItem = [[NSMenuItem alloc] initWithTitle: quitString
-						     action: @selector(terminate:)
-						     keyEquivalent: @"q"];
+        {
+          NSString *infoString = NSLocalizedString (@"Info", @"Info");
+          NSString *quitString = [NSString stringWithFormat: @"%@ %@", 
+                                           NSLocalizedString (@"Quit", @"Quit"), processName];
+          NSMenuItem *quitItem = [[NSMenuItem alloc] initWithTitle: quitString
+                                                     action: @selector(terminate:)
+                                                     keyEquivalent: @"q"];
+          NSMenuItem *appItem;
+          
+          appItem = (NSMenuItem*)[self itemAtIndex: 0]; // Info item.
 
-	  [self addItem: quitItem];
-	  [self setTitle: processName];
-	  [appItem setTitle: infoString];
-	  [[appItem submenu] setTitle: infoString];
-	}
+          [self addItem: quitItem];
+          [self setTitle: processName];
+          [appItem setTitle: infoString];
+          [[appItem submenu] setTitle: infoString];
+        }
 
       [self _setGeometry];
       [self sizeToFit];
@@ -201,7 +206,7 @@ static BOOL _isInInterfaceBuilder = NO;
 @implementation NSView (NibCompatibility)
 - (void) _setWindow: (id) w
 {
-  ASSIGN(_window,w);
+  _window = w;
 }
 
 - (void) _fixSubviews
@@ -224,6 +229,11 @@ static BOOL _isInInterfaceBuilder = NO;
 }
 @end
 
+/**
+ * NSWindowTemplate
+ *
+ * Instances of this class take the place of all windows in the nib file.
+ */
 @implementation NSWindowTemplate
 + (void) initialize
 {
@@ -243,6 +253,9 @@ static BOOL _isInInterfaceBuilder = NO;
   [super dealloc];
 }
 
+/**
+ * Designated initializer for NSWindowTemplate.
+ */
 - (id) initWithWindow: (NSWindow *)window
             className: (NSString *)windowClass
            isDeferred: (BOOL) deferred
@@ -332,7 +345,7 @@ static BOOL _isInInterfaceBuilder = NO;
         }
       if ([coder containsValueForKey: @"NSFrameAutosaveName"])
         {
-          ASSIGN(_autosaveName, [coder decodeObjectForKey: @"NSFrameAutosaveName"]);
+	  ASSIGN(_autosaveName, [coder decodeObjectForKey: @"NSFrameAutosaveName"]);
         }
       if ([coder containsValueForKey: @"NSWindowTitle"])
         {
@@ -374,6 +387,9 @@ static BOOL _isInInterfaceBuilder = NO;
     }
 }
 
+/**
+ * This method is used to get the real object when connections are established.
+ */
 - (id) nibInstantiate
 {
   if (_realObject == nil)
@@ -432,133 +448,214 @@ static BOOL _isInInterfaceBuilder = NO;
 }
 
 // setters and getters
+/**
+ * sets the type of backing store the window uses.
+ */
 - (void) setBackingStoreType: (NSBackingStoreType)type
 {
   _backingStoreType = type;
 }
 
+/**
+ * Returns the type of backing store which is used.
+ */
 - (NSBackingStoreType) backingStoreType
 {
   return _backingStoreType;
 }
 
+/**
+ * Sets whether or not the window is deferred.
+ */
 - (void) setDeferred: (BOOL)flag
 {
   _flags.isDeferred = flag;
 }
 
+/**
+ * Returns YES, if the window is deferred, NO otherwise.
+ */
 - (BOOL) isDeferred
 {
   return _flags.isDeferred;
 }
 
+/**
+ * Sets the maximum size of the window.
+ */
 - (void) setMaxSize: (NSSize)maxSize
 {
   _maxSize = maxSize;
 }
 
+/**
+ * Returns the maximum size of the window.
+ */
 - (NSSize) maxSize
 {
   return _maxSize;
 }
 
+/**
+ * Sets the minimum size of the window.
+ */
 - (void) setMinSize: (NSSize)minSize
 {
   _minSize = minSize;
 }
 
+/**
+ * Returns the maximum size of the window.
+ */
 - (NSSize) minSize
 {
   return _minSize;
 }
 
+/**
+ * Sets the window style.
+ */
 - (void) setWindowStyle: (unsigned)style
 {
   _windowStyle = style;
 }
 
+/** 
+ * Returns the window style.
+ */
 - (unsigned) windowStyle
 {
   return _windowStyle;
 }
 
+/**
+ * Sets the window title.
+ */
 - (void) setTitle: (NSString *) title
 {
   ASSIGN(_title, title);
 }
 
+/**
+ * Returns the window style.
+ */
 - (NSString *)title;
 {
   return _title;
 }
 
+/**
+ * Sets the class used for the content view.
+ */
 - (void) setViewClass: (NSString *)viewClass
 {
   ASSIGN(_viewClass,viewClass);
 }
 
+/**
+ * Returns the name of the class used for the content view.
+ */
 - (NSString *)viewClass
 {
   return _viewClass;
 }
 
+/**
+ * Sets the window rect.
+ */
 - (void) setWindowRect: (NSRect)rect
 {
   _windowRect = rect;
 }
 
+/**
+ * Returns the window rect.
+ */
 - (NSRect)windowRect
 {
   return _windowRect;
 }
 
+/**
+ * Sets the screen rect.
+ */
 - (void) setScreenRect: (NSRect)rect
 {
   _screenRect = rect;
 }
 
+/**
+ * Returns the screen rect.
+ */
 - (NSRect) screenRect
 {
   return _screenRect;
 }
 
+/**
+ * Sets the instantiated object/real object.
+ */
 - (void) setRealObject: (id)o
 {
   ASSIGN(_realObject,o);
 }
 
+/**
+ * Returns the real object represented by this template.
+ */
 - (id) realObject
 {
   return _realObject;
 }
 
+/**
+ * Sets the view instance.
+ */ 
 - (void) setView: (id)view
 {
   ASSIGN(_view,view);
 }
 
+/** 
+ * Gets the view instance.
+ */
 - (id) view
 {
   return _view;
 }
 
+/**
+ * sets the class name to be used when unarchiving the window.
+ */
 - (void) setClassName: (NSString *)name
 {
   ASSIGN(_windowClass, name);
 }
 
+/**
+ * Returns the class instance.
+ */
 - (NSString *)className
 {
   return _windowClass;
 }
 
+/**
+ * Returns the base window class.   This is usually NSWindow, but this method
+ * is overriden in the editor so that a different class may be used to take the
+ * place of the window.   In the case of Gorm, this is GormNSWindow.
+ */
 - (Class) baseWindowClass
 {
   return _baseWindowClass;
 }
 @end
 
-// Template for any classes which derive from NSView
+/*
+ * NSViewTemplate
+ *
+ * Template for any classes which derive from NSView
+ */
 @implementation NSViewTemplate
 + (void) initialize
 {
@@ -568,6 +665,9 @@ static BOOL _isInInterfaceBuilder = NO;
     }
 }
 
+/**
+ * Designated initializer for NSViewTemplate.
+ */
 - (id) initWithObject: (id)o
 	    className: (NSString *)name
 {
@@ -604,6 +704,7 @@ static BOOL _isInInterfaceBuilder = NO;
 	    }
 	}
       
+      // FIXME: Should release self
       return _realObject;
     }
   else
@@ -631,21 +732,33 @@ static BOOL _isInInterfaceBuilder = NO;
 }
 
 // setters and getters
+/**
+ * Set the class name to be used by the NSView subclass.
+ */
 - (void) setClassName: (NSString *)name
 {
   ASSIGN(_className, name);
 }
 
+/**
+ * Returns the classname.
+ */
 - (NSString *)className
 {
   return _className;
 }
 
+/**
+ * Set the real object of the template.
+ */
 - (void) setRealObject: (id)o
 {
   ASSIGN(_realObject, o);
 }
 
+/**
+ * Get the real object represented by the template.
+ */
 - (id) realObject
 {
   return _realObject;
@@ -663,7 +776,11 @@ static BOOL _isInInterfaceBuilder = NO;
 }
 @end
 
-// Template for any classes which derive from NSTextView
+/**
+ * NSTextViewTemplate 
+ *
+ * Template for any classes which derive from NSTextView
+ */
 @implementation NSTextViewTemplate
 + (void) initialize
 {
@@ -686,6 +803,7 @@ static BOOL _isInInterfaceBuilder = NO;
 
 - (id) initWithCoder: (NSCoder *)aCoder
 {
+  RELEASE(self);
   return nil;
 }
 
@@ -1021,6 +1139,9 @@ static BOOL _isInInterfaceBuilder = NO;
 }
 @end
 
+/**
+ * This class represents an image or a sound which is referenced by the nib file.
+ */
 @implementation NSCustomResource
 - (void) setClassName: (NSString *)className
 {
@@ -1086,7 +1207,15 @@ static BOOL _isInInterfaceBuilder = NO;
 }
 @end
 
+/**
+ * Category to add methods to NSKeyedUnarchiver which are needed during
+ * nib reading.
+ */
 @implementation NSKeyedUnarchiver (NSClassSwapperPrivate)
+/**
+ * This method is used to replace oldObj with newObj
+ * in the map that is maintained in NSKeyedUnarchiver.
+ */
 - (BOOL) replaceObject: (id)oldObj withObject: (id)newObj
 {
   unsigned int i = 0;
@@ -1107,11 +1236,18 @@ static BOOL _isInInterfaceBuilder = NO;
   return NO;
 }
 
+/**
+ * This method is private and is purely for debugging purposes.
+ */
 - (NSDictionary *)keyMap
 {
   return _keyMap;
 }
 
+/**
+ * This method returns the class which replaces the class named
+ * by className.   It uses the classes map to do this.
+ */
 - (Class) replacementClassForClassName: (NSString *)className
 {
   Class aClass;
@@ -1128,6 +1264,15 @@ static BOOL _isInInterfaceBuilder = NO;
 }
 @end
 
+/**
+ * NSClassSwapper
+ *
+ * This class is used to stand-in for objects which need to be replaced by another object.  
+ * When this class is loaded in the live application, it unarchives and immediately replaces
+ * itself with the instance of the object requested.   This is necessary since IB/Gorm does 
+ * have objects this is used for in palettes, so there is no "live" or actual instance saved
+ * in the gorm file... only this object as a stand in.
+ */
 @implementation NSClassSwapper
 - (id) initWithObject: (id)object 
         withClassName: (NSString *)className
@@ -1142,46 +1287,75 @@ static BOOL _isInInterfaceBuilder = NO;
   return self;
 }
 
+/**
+ * This class method keeps track of whether or not we are operating within IB/Gorm.   
+ * When unarchiving in IB/Gorm some behavior may need to be surpressed for some objects
+ * or it 
+ */
 + (void) setIsInInterfaceBuilder: (BOOL)flag
 {
   _isInInterfaceBuilder = flag;
 }
 
+/**
+ * returns YES, if we are currently in IB/Gorm.
+ */
 + (BOOL) isInInterfaceBuilder
 {
   return _isInInterfaceBuilder;
 }
 
+/**
+ * Sets the template represented by temp.
+ */
 - (void) setTemplate: (id)temp
 {
   ASSIGN(_template, temp);
 }
 
+/**
+ * Returns the template.
+ */
 - (id) template
 {
   return _template;
 }
 
+/**
+ * Sets the class name.
+ */
 - (void) setClassName: (NSString *)className
 {
   ASSIGNCOPY(_className, className);
 }
 
+/**
+ * Returns the class name.
+ */
 - (NSString *)className
 {
   return _className;
 }
 
+/**
+ * Sets the original class name.
+ */
 - (void) setOriginalClassName: (NSString *)className
 {
   ASSIGNCOPY(_originalClassName, className);
 }
 
+/**
+ * Returns the original class name.
+ */
 - (NSString *)originalClassName
 {
   return _originalClassName;
 }
 
+/** 
+ * Instantiates the real object using className.
+ */
 - (void) instantiateRealObject: (NSCoder *)coder withClassName: (NSString *)className
 {
   Class newClass = nil;
@@ -1209,7 +1383,11 @@ static BOOL _isInInterfaceBuilder = NO;
   [decoder setDelegate: nil]; // unset the delegate...
 }
 
-// Delegate method...
+/**
+ * This delegate method makes the proper substitution for cellClass
+ * when the object needs to have it's own cell.   An example of this
+ * is NSSecureTextField/NSSecureTextFieldCell.
+ */
 - (id) unarchiver: (NSKeyedUnarchiver *)coder
   didDecodeObject: (id)obj
 {
@@ -1233,15 +1411,18 @@ static BOOL _isInInterfaceBuilder = NO;
       [_className isEqualToString: _originalClassName] == NO)
     {
       Class newCellClass = [newClass cellClass];
-      if(newCellClass != [NSCell class])
-	{
-	  result = [[newCellClass alloc] initWithCoder: coder];      
-	}
+      if (newCellClass != [NSCell class])
+        {
+          result = [[newCellClass alloc] initWithCoder: coder];      
+        }
     }
 
   return result;
 }
 
+/**
+ * Decode NSClassSwapper.
+ */
 - (id) initWithCoder: (NSCoder *)coder
 {
   if ([coder allowsKeyedCoding])
@@ -1251,13 +1432,21 @@ static BOOL _isInInterfaceBuilder = NO;
 
       // build the real object...
       if ([NSClassSwapper isInInterfaceBuilder] == YES)
-	{
+        {
           [self instantiateRealObject: coder withClassName: _originalClassName];
         }
       else
         {
           [self instantiateRealObject: coder withClassName: _className];
-	} 
+        }
+      
+      {
+        id object;
+
+        object = RETAIN(_template);
+        RELEASE(self);
+        return AUTORELEASE(object);
+      }
     }
   else
     {
@@ -1266,9 +1455,12 @@ static BOOL _isInInterfaceBuilder = NO;
                    NSStringFromClass([coder class])];
     }
 
-  return _template;
+  return self;
 }
 
+/**
+ * Encode NSClassSwapper.
+ */
 - (void) encodeWithCoder: (NSCoder *)coder
 {
   if ([coder allowsKeyedCoding])
@@ -1285,6 +1477,9 @@ static BOOL _isInInterfaceBuilder = NO;
     }
 }
 
+/**
+ * Deallocate NSClassSwapper instance.
+ */
 - (void) dealloc
 {
   RELEASE(_className);
@@ -1295,14 +1490,26 @@ static BOOL _isInInterfaceBuilder = NO;
 @end
 
 @implementation NSNibConnector (NibCompatibility)
+/**
+ * This method causes the connection to instantiate the objects in it's source
+ * and destination.   The instantiator is the object which holds any custom
+ * class information which might be needed to do the proprer substitution of
+ * objects based on the contents of the maps.
+ */
 - (void) instantiateWithInstantiator: (id<GSInstantiator>)instantiator
 {
-  _src = [instantiator instantiateObject: _src];
-  _dst = [instantiator instantiateObject: _dst];
+  [self setSource: [instantiator instantiateObject: _src]];
+  [self setDestination: [instantiator instantiateObject: _dst]];
 }
 @end
 
 @implementation NSNibControlConnector (NibCompatibility)
+/**
+ * This method overrides the default implementation of instantiate with
+ * instantiator.   It also corrects a common issue in some nib files
+ * by adding a colon to the end if none was given.   It then calls the
+ * superclass with the corrected label.
+ */
 - (void) instantiateWithInstantiator: (id<GSInstantiator>)instantiator
 {
   NSRange colonRange = [_tag rangeOfString: @":"];
@@ -1318,6 +1525,23 @@ static BOOL _isInInterfaceBuilder = NO;
 }
 @end
 
+/**
+ * NSIBObjectData
+ *
+ * This class is the container for all of the nib data.  It contains several maps.
+ * The maps are the following:
+ * 
+ *     name -> object (name table)
+ *     object -> name (name table reverse lookup)
+ *     classes -> object (for custom class storage)
+ *     oids -> object (for relating the oid to each object)
+ *     accessibilityOids -> object 
+ *
+ * The maps are stored in the nib itself as a set of synchronized 
+ * arrays one array containing the keys and the other the values.  This is why, in the
+ * initWithCoder: and encodeWithCoder: methods they are saved as arrays and then 
+ * loaded into NSMapTables.   
+ */
 @implementation NSIBObjectData
 /**
  * Get the values from the map in the same order as the keys.
@@ -1358,7 +1582,7 @@ static BOOL _isInInterfaceBuilder = NO;
 }
 
 /**
- * Encode the NSIBObjectData.
+ * Encode the NSIBObjectData container
  */
 - (void) encodeWithCoder: (NSCoder *)coder
 {
@@ -1404,7 +1628,7 @@ static BOOL _isInInterfaceBuilder = NO;
 }
 
 /**
- * Decode the NSIBObjectData.
+ * Decode the NSIBObjectData container.
  */
 - (id) initWithCoder: (NSCoder *)coder
 {
@@ -1421,15 +1645,38 @@ static BOOL _isInInterfaceBuilder = NO;
       NSArray *accessibilityOidsKeys = nil;
       NSArray *accessibilityOidsValues = nil;
 
+      //
+      // Get root, font, framwork and oid. 
+      // Retain objects since NSKeyedUnarchiver autoreleases unarchived objects.
+      //
       ASSIGN(_root, [coder decodeObjectForKey: @"NSRoot"]);
-      ASSIGN(_visibleWindows,  
-	     (NSMutableArray *)[coder decodeObjectForKey: @"NSVisibleWindows"]);
-      ASSIGN(_accessibilityConnectors, 
-	     (NSMutableArray *)[coder decodeObjectForKey: @"NSAccessibilityConnectors"]);
       ASSIGN(_fontManager, [coder decodeObjectForKey: @"NSFontManager"]);
       ASSIGN(_framework, [coder decodeObjectForKey: @"NSFramework"]);
       _nextOid = [coder decodeIntForKey: @"NSNextOid"];
 
+      // get connections.
+      ASSIGN(_connections, (NSMutableArray *)
+	[coder decodeObjectForKey: @"NSConnections"]);
+      ASSIGN(_accessibilityConnectors, (NSMutableArray *)
+	[coder decodeObjectForKey: @"NSAccessibilityConnectors"]);
+
+      // get visible windows
+      ASSIGN(_visibleWindows, (NSMutableArray *)
+	[coder decodeObjectForKey: @"NSVisibleWindows"]);
+
+      // instantiate the maps..
+      _classes = NSCreateMapTable(NSObjectMapKeyCallBacks,
+				  NSObjectMapValueCallBacks, 2);
+      _names = NSCreateMapTable(NSObjectMapKeyCallBacks,
+				NSObjectMapValueCallBacks, 2);
+      _objects = NSCreateMapTable(NSObjectMapKeyCallBacks,
+				  NSObjectMapValueCallBacks, 2);
+      
+      // 
+      // Get the maps.  There is no need to retain these, 
+      // since they are going to be placed into the NSMapTable
+      // structures anyway.
+      //
       nameKeys = (NSArray *)
 	[coder decodeObjectForKey: @"NSNamesKeys"];
       nameValues = (NSArray *)
@@ -1438,14 +1685,30 @@ static BOOL _isInInterfaceBuilder = NO;
 	[coder decodeObjectForKey: @"NSClassesKeys"];
       classValues = (NSArray *)
 	[coder decodeObjectForKey: @"NSClassesValues"];
+      objectsKeys = (NSArray *)
+	[coder decodeObjectForKey: @"NSObjectsKeys"];
+      objectsValues = (NSArray *)
+	[coder decodeObjectForKey: @"NSObjectsValues"];
 
-      // Only get this when in the editor...
+      // Fill in the maps...
+      [self _buildMap: _classes 
+	    withKeys: classKeys 
+	    andValues: classValues];
+      [self _buildMap: _names 
+	    withKeys: nameKeys 
+	    andValues: nameValues];
+      [self _buildMap: _objects 
+	    withKeys: objectsKeys 
+	    andValues: objectsValues];
+      
+      //
+      // Only get these maps when in the editor.  They
+      // aren't useful outside of it and only waste memory if
+      // unarchived in the live application.
+      //
       if([NSClassSwapper isInInterfaceBuilder])
 	{
-	  objectsKeys = (NSArray *)
-	    [coder decodeObjectForKey: @"NSObjectsKeys"];
-	  objectsValues = (NSArray *)
-	    [coder decodeObjectForKey: @"NSObjectsValues"];
+	  // Only get these when in the editor...
 	  oidsKeys = (NSArray *)
 	    [coder decodeObjectForKey: @"NSOidsKeys"];
 	  oidsValues = (NSArray *)
@@ -1454,46 +1717,19 @@ static BOOL _isInInterfaceBuilder = NO;
 	    [coder decodeObjectForKey: @"NSAccessibilityOidsKeys"];
 	  accessibilityOidsValues = (NSArray *)
 	    [coder decodeObjectForKey: @"NSAccessibilityOidsValues"];      
-	}
 
-      // instantiate the maps..
-      _objects = NSCreateMapTable(NSObjectMapKeyCallBacks,
-                                  NSObjectMapValueCallBacks, 2);
-      _names = NSCreateMapTable(NSObjectMapKeyCallBacks,
-				NSObjectMapValueCallBacks, 2);
-      _oids = NSCreateMapTable(NSObjectMapKeyCallBacks,
-                               NSObjectMapValueCallBacks, 2);
-      _classes = NSCreateMapTable(NSObjectMapKeyCallBacks,
-				  NSObjectMapValueCallBacks, 2);
-      _accessibilityOids = NSCreateMapTable(NSObjectMapKeyCallBacks,
-                                            NSObjectMapValueCallBacks, 2);
-      
-      // Fill in the maps...
-      /*
-      */
-      [self _buildMap: _classes 
-	    withKeys: classKeys 
-	    andValues: classValues];
-      [self _buildMap: _names 
-	    withKeys: nameKeys 
-	    andValues: nameValues];
-
-      // Only get this when in the editor.
-      if([NSClassSwapper isInInterfaceBuilder])
-	{
+	  _accessibilityOids = NSCreateMapTable(NSObjectMapKeyCallBacks,
+						NSObjectMapValueCallBacks, 2);	  
+	  _oids = NSCreateMapTable(NSObjectMapKeyCallBacks,
+				   NSObjectMapValueCallBacks, 2);
 	  [self _buildMap: _accessibilityOids 
 		withKeys: accessibilityOidsKeys 
 		andValues: accessibilityOidsValues];
-	  [self _buildMap: _objects 
-		withKeys: objectsKeys 
-		andValues: objectsValues];
 	  [self _buildMap: _oids 
 		withKeys: oidsKeys 
 		andValues: oidsValues];
 	}
 
-      ASSIGN(_connections,  [[coder decodeObjectForKey: @"NSConnections"] mutableCopy]);
-      
       // instantiate...
       _topLevelObjects = [[NSMutableSet alloc] init];
     }
@@ -1508,7 +1744,7 @@ static BOOL _isInInterfaceBuilder = NO;
 }
 
 /**
- * Instantiate a new one.
+ * Initialize a new NSIBObjectData.
  */
 - (id) init
 {
@@ -1539,16 +1775,20 @@ static BOOL _isInInterfaceBuilder = NO;
 }
 
 /**
- * Deallocate.
+ * Deallocate NSIBObjectData.
  */
 - (void) dealloc
 {
   // free the maps.
   NSFreeMapTable(_objects);
   NSFreeMapTable(_names);
-  NSFreeMapTable(_oids);
   NSFreeMapTable(_classes);
-  NSFreeMapTable(_accessibilityOids);
+  // these are not allocated when not in interface builder.
+  if([NSClassSwapper isInInterfaceBuilder])
+    {
+      NSFreeMapTable(_oids);
+      NSFreeMapTable(_accessibilityOids);
+    }
 
   // free other objects.
   RELEASE(_accessibilityConnectors);
@@ -1698,67 +1938,108 @@ static BOOL _isInInterfaceBuilder = NO;
   return result;
 }
 
+/**
+ * Set the root object.
+ */
 - (void) setRoot: (id) root
 {
   ASSIGN(_root, root);
 }
 
+/**
+ * Return the root object.
+ */
 - (id) root
 {
   return _root;
 }
 
+/**
+ * Set the value of the next available oid.
+ */
 - (void) setNextOid: (int)noid
 {
   _nextOid = noid;
 }
 
+/**
+ * Get the value of the next available oid.
+ */
 - (int) nextOid
 {
   return _nextOid;
 }
 
+/**
+ * Connections between objects.
+ */
 - (NSMutableArray *) connections
 {
   return _connections;
 }
 
+/**
+ * Set of top level objects.
+ */
 - (NSMutableSet *) topLevelObjects
 {
   return _topLevelObjects;
 }
 
+/**
+ * Names to objects
+ */
 - (NSMutableDictionary *) nameTable
 {
   return nil;
 }
 
+/**
+ * Set of all visible windows.
+ */
 - (NSMutableArray *) visibleWindows
 {
   return _visibleWindows;
 }
 
+/**
+ * Objects to names table.
+ */
 - (NSMapTable *) objects
 {
   return _objects;
 }
 
+/**
+ * Names to objects table.
+ */
 - (NSMapTable *) names
 {
   return _names;
 }
 
+/**
+ * Classes to objects table.
+ */
 - (NSMapTable *) classes
 {
   return _classes;
 }
 
+/**
+ * Oids to objects table.
+ */ 
 - (NSMapTable *) oids
 {
   return _oids;
 }
 @end
 
+/**
+ * NSButtonImageSource
+ * 
+ * This class is used by buttons to pull the correct image based on a given state.
+ */
 @implementation NSButtonImageSource
 - (id) initWithCoder: (NSCoder *)coder
 {
@@ -1774,7 +2055,7 @@ static BOOL _isInInterfaceBuilder = NO;
     }
 
   RELEASE(self);
-  return [NSImage imageNamed: imageName];
+  return RETAIN([NSImage imageNamed: imageName]);
 }
 
 - (void) encodeWithCoder: (NSCoder *)coder
@@ -1791,6 +2072,9 @@ static BOOL _isInInterfaceBuilder = NO;
     }
 }
 
+/**
+ * Initializes with image name.
+ */
 - (id) initWithImageNamed: (NSString *)name
 {
   if ((self = [super init]) != nil)
@@ -1800,6 +2084,9 @@ static BOOL _isInInterfaceBuilder = NO;
   return self;
 }
 
+/**
+ * Returns imageName.
+ */
 - (NSString *)imageName
 {
   return imageName;
@@ -1932,18 +2219,32 @@ static BOOL _isInInterfaceBuilder = NO;
                                     exponent: exponent
                                     isNegative: negative];
     }
+  // FIXME: Needs to release self
   return dn;
 }
 
 @end
 
-// ...dummy/placeholder classes...
-// overridden in NSTableView to be GSTableCornerView, 
-// but the class needs to be present to be overridden.
+/**
+ * NSCornerView
+ *
+ * Overridden in NSTableView to be GSTableCornerView, 
+ * but the class needs to be present to be overridden.
+ * 
+ * Currently this is a place-holder class.
+ */
 @implementation _NSCornerView
 @end
 
-// class needed for nib encoding/decoding by
+/**
+ * NSPSMatrix.
+ *
+ * This class is needed for nib encoding/decoding by transforms.  
+ * Currently it's only referenced in the NSProgressIndicator,
+ * as far as I can tell.
+ *
+ * Place holder class.
+ */
 @implementation NSPSMatrix
 - (void) encodeWithCoder: (NSCoder *)coder
 {
@@ -1952,7 +2253,6 @@ static BOOL _isInInterfaceBuilder = NO;
 
 - (id) initWithCoder: (NSCoder *)coder
 {
-  // what's NSPSMatix all about?
   // NSLog(@"NSPSMatrix = %@",[(NSKeyedUnarchiver *)coder keyMap]);
   return self;
 }
