@@ -340,7 +340,7 @@ NSRegisterServicesProvider(id provider, NSString *name)
 
   /*
    * We assume that messages of the form 'application:...' are all
-   * safe and do mnot need to be listed in GSPermittedMessages.
+   * safe and do not need to be listed in GSPermittedMessages.
    * They can be handled either by the application delegate or by
    * the shared GSServicesManager instance.
    */
@@ -618,6 +618,31 @@ static NSString         *disabledName = @".GNUstepDisabled";
   return result;
 }
 
+- (BOOL) application: (NSApplication*)theApp
+	     openURL: (NSURL*)aURL
+{
+  id	del = [NSApp delegate];
+  BOOL	result = NO;
+
+  if ([del respondsToSelector: _cmd])
+    {
+      result = [del application: theApp openURL: aURL];
+    }
+  else if ([[NSDocumentController sharedDocumentController]
+    openDocumentWithContentsOfURL: aURL display: YES] != nil)
+    {
+      [NSApp activateIgnoringOtherApps: YES];
+      result = YES;
+    }
+  else
+    {
+      NSString	*s = [aURL absoluteString];
+
+      result = [self application: theApp openFile: s];
+    }
+  return result;
+}
+
 - (BOOL) application: (NSApplication*)theApp
 	   printFile: (NSString*)file
 {
@@ -1397,18 +1422,6 @@ static NSString         *disabledName = @".GNUstepDisabled";
     {
       NSArray   	*a;
       unsigned  	i;
-      NSMenu		*mainMenu = [_application mainMenu];
-      BOOL		found = NO;
-
-      a = [mainMenu itemArray];
-      for (i = 0; i < [a count]; i++)
-	if ([[a objectAtIndex: i] submenu] == _servicesMenu)
-	  found = YES;
-      if (found == NO)
-	{
-	  NSLog(@"Services menu not in main menu!\n");
-	  return;
-	}
 
       a = [_servicesMenu itemArray];
 
@@ -1449,18 +1462,17 @@ static NSString         *disabledName = @".GNUstepDisabled";
 		  if (subWasEnabled != subShouldBeEnabled)
 		    {
 		      [subitem setEnabled: subShouldBeEnabled];
-//		      [subMenuCells setNeedsDisplayInRect:
-//				[subMenuCells cellFrameAtRow: j]];
 		    }
 		}
 	    }
 	  else
-	    shouldBeEnabled = [self validateMenuItem: item];
+	    {
+	      shouldBeEnabled = [self validateMenuItem: item];
+	    }
 
           if (wasEnabled != shouldBeEnabled)
             {
               [item setEnabled: shouldBeEnabled];
-//	      [menuCells setNeedsDisplayInRect: [menuCells cellFrameAtRow: i]];
             }
         }
     }
