@@ -30,11 +30,11 @@
 #include <Foundation/NSObject.h>
 #include <Foundation/NSArray.h>
 #include <Foundation/NSDictionary.h>
+#include <Foundation/NSEnumerator.h>
 #include <Foundation/NSException.h>
 #include <Foundation/NSNotification.h>
 #include <Foundation/NSUserDefaults.h>
 #include <Foundation/NSString.h>
-#include "AppKit/NSBezierPath.h"
 #include "AppKit/NSButton.h"
 #include "AppKit/NSClipView.h"
 #include "AppKit/NSColor.h"
@@ -44,12 +44,16 @@
 #include "AppKit/NSImage.h"
 #include "AppKit/NSMenu.h"
 #include "AppKit/NSPasteboard.h"
-#include "AppKit/NSToolbarItem.h" // It contains GSMovableToolbarItemPboardType declaration
+// It contains GSMovableToolbarItemPboardType declaration
+#include "AppKit/NSToolbarItem.h"
 #include "AppKit/NSView.h"
 #include "AppKit/NSWindow.h"
-#include "GNUstepGUI/GSToolbar.h"
+
+#include "GNUstepGUI/GSTheme.h"
 #include "GNUstepGUI/GSToolbarView.h"
 
+#include "NSToolbarFrameworkPrivate.h"
+
 typedef enum {
   ToolbarViewDefaultHeight = 62,
   ToolbarViewRegularHeight = 62,
@@ -61,10 +65,6 @@ static const int ClippedItemsViewWidth = 28;
 // Internal
 static const int current_version = 1;
 static NSColorList *SystemExtensionsColors;
-static NSColor *StandardBackgroundColor;
-static NSColor *BackgroundColor;
-static NSColor *BorderColor;
-
 
 // Toolbar color extensions
 
@@ -75,25 +75,19 @@ static void initSystemExtensionsColors(void)
   NSDictionary *colors;
   
   /* Set up a dictionary containing the names of all the system extensions 
-     colors as keys and with colors in string format as values. */
+     colours as keys and with colours as values. */
   toolbarBorderColor = [NSColor colorWithCalibratedRed: 0.5 
                                                  green: 0.5 
-						  blue: 0.5 
-						 alpha: 1.0];
+                                                  blue: 0.5 
+                                                 alpha: 1.0];
   
-  // Window background color by tranparency						 
+  // Window background color by tranparency
   toolbarBackgroundColor = [NSColor clearColor]; 
   
-  // Window backgound color hardcoded
-  /* toolbarBackgroundColor = [NSColor colorWithCalibratedRed: 0.8 
-                                                        green: 0.8 
-						         blue: 0.8 
-						        alpha: 1.0]; */
-							
   colors = [[NSDictionary alloc] initWithObjectsAndKeys: 
-    toolbarBackgroundColor, @"toolbarBackgroundColor",toolbarBorderColor,
-    @"toolbarBorderColor", nil];
-							     
+    toolbarBackgroundColor, @"toolbarBackgroundColor",
+    toolbarBorderColor, @"toolbarBorderColor", nil];
+                                                             
   SystemExtensionsColors = [NSColorList colorListNamed: @"System extensions"];
   if (SystemExtensionsColors == nil)
     {
@@ -111,18 +105,18 @@ static void initSystemExtensionsColors(void)
       e = [colors keyEnumerator];
   
       while ((colorKey = (NSString *)[e nextObject])) 
-	{
-	  if ([SystemExtensionsColors colorWithKey: colorKey])
-	    continue;
+        {
+          if ([SystemExtensionsColors colorWithKey: colorKey])
+            continue;
 
-	  color = [colors objectForKey: colorKey];
-	  [SystemExtensionsColors setColor: color forKey: colorKey];
+          color = [colors objectForKey: colorKey];
+          [SystemExtensionsColors setColor: color forKey: colorKey];
 
-	  changed = YES;
-	}
+          changed = YES;
+        }
 
       if (changed)
-	[SystemExtensionsColors writeToFile: nil];
+        [SystemExtensionsColors writeToFile: nil];
     }
 }
 
@@ -141,30 +135,6 @@ static void initSystemExtensionsColors(void)
 /*
  * Toolbar related code
  */
-@interface GSToolbar (GNUstepPrivate)
-- (void) _build;
-
-- (void) _concludeRemoveItem: (NSToolbarItem *)item atIndex: (int)index broadcast: (BOOL)broadcast;
-- (int) _indexOfItem: (NSToolbarItem *)item;
-- (void) _insertPassivelyItem: (NSToolbarItem *)item atIndex: (int)newIndex;
-- (void) _moveItemFromIndex: (int)index toIndex: (int)newIndex broadcast: (BOOL)broacast;
-
-- (void) _toolbarViewWillMoveToSuperview: (NSView *)newSuperview;
-
-// Accessors
-- (void) _setToolbarView: (GSToolbarView *)toolbarView;
-- (GSToolbarView *) _toolbarView;
-@end
-
-@interface NSToolbarItem (GNUstepPrivate)
-- (void) _layout;
-
-// Accessors
-- (NSView *) _backView;
-- (NSMenuItem *) _defaultMenuFormRepresentation;
-- (BOOL) _isModified;
-- (BOOL) _isFlexibleSpace;
-@end
 
 @interface GSToolbarButton
 - (NSToolbarItem *) toolbarItem;
@@ -174,46 +144,20 @@ static void initSystemExtensionsColors(void)
 - (NSToolbarItem *) toolbarItem;
 @end
 
-@interface GSToolbarView (GNUstepPrivate)
-- (void) _handleBackViewsFrame;
-- (void) _handleViewsVisibility;
-- (void) _reload;
-- (void) _setToolbar: (GSToolbar *)toolbar;
-- (void) _takeInAccountFlexibleSpaces;
-- (int) _insertionIndexAtPoint: (NSPoint)location;
-
-// Accessors
-- (float) _heightFromLayout;
-- (NSArray *) _visibleBackViews;
-- (void) _setSizeMode: (NSToolbarSizeMode)sizeMode;
-- (NSToolbarSizeMode) _sizeMode;
-- (BOOL) _usesStandardBackgroundColor;
-- (void) _setUsesStandardBackgroundColor: (BOOL)standard;
-- (void) _setWillBeVisible: (BOOL)willBeVisible;
-- (BOOL) _willBeVisible;
-@end
-
 @interface GSToolbarClippedItemsButton : NSButton
 {
-  GSToolbar *_toolbar;
+  NSToolbar *_toolbar;
 }
 
 - (id) init;
 
 // Accessors 
-- (NSMenu *) returnMenu; 
+- (NSMenu *) overflowMenu; 
 /* This method cannot be called "menu" otherwise it would override NSResponder
    method with the same name. */
 
 - (void) layout;
-- (void) setToolbar: (GSToolbar *)toolbar; 
-@end
-
-@interface GSToolbarClipView : NSClipView
-{
-
-}
-
+- (void) setToolbar: (NSToolbar *)toolbar; 
 @end
 
 @implementation GSToolbarClippedItemsButton
@@ -252,30 +196,30 @@ static void initSystemExtensionsColors(void)
 
 - (void) mouseDown: (NSEvent *)event 
 {
-   NSMenu *clippedItemsMenu = [self menuForEvent: event];
+  NSMenu *clippedItemsMenu = [self menuForEvent: event];
    
-   [super highlight: YES];
+  [super highlight: YES];
    
-   if (clippedItemsMenu != nil)
+  if (clippedItemsMenu != nil)
     {
       [NSMenu popUpContextMenu: clippedItemsMenu withEvent: event 
-                                                   forView: self];
+              forView: self];
     }
     
-    [super highlight: NO];
+  [super highlight: NO];
 }
 
 - (NSMenu *) menuForEvent: (NSEvent *)event 
 {
   if ([event type] == NSLeftMouseDown)
     {
-      return [self returnMenu];
+      return [self overflowMenu];
     }
     
   return nil;
 }
 
-- (NSMenu *) returnMenu 
+- (NSMenu *) overflowMenu 
 {
   /* This method cannot be called "menu" otherwise it would
      override NSResponder method with the same name. */
@@ -284,8 +228,6 @@ static void initSystemExtensionsColors(void)
   id item;
   NSArray *visibleItems;
   
-  AUTORELEASE(menu);
-  
   visibleItems = [_toolbar visibleItems];
 
   e = [[_toolbar items] objectEnumerator];
@@ -295,21 +237,24 @@ static void initSystemExtensionsColors(void)
         {
           id menuItem;
       
-      	  menuItem = [item menuFormRepresentation];
+          menuItem = [item menuFormRepresentation];
           if (menuItem == nil)
-	    menuItem = [item _defaultMenuFormRepresentation];
-	    
-	  if (menuItem != nil)
-            [menu addItem: menuItem];
+            menuItem = [item _defaultMenuFormRepresentation];
+            
+          if (menuItem != nil)
+            {
+              [item validate];
+              [menu addItem: menuItem];
+            }
         }
     }
 
-  return menu;
+  return AUTORELEASE(menu);
 }
 
 // Accessors
 
-- (void) setToolbar: (GSToolbar *)toolbar
+- (void) setToolbar: (NSToolbar *)toolbar
 {
   // Don't do an ASSIGN here, the toolbar view retains us.
   _toolbar = toolbar;
@@ -318,103 +263,42 @@ static void initSystemExtensionsColors(void)
 
 // ---
 
-@implementation GSToolbarClipView
-
-// Nothing here
-
-@end
-
 // Implementation GSToolbarView
 
 @implementation GSToolbarView
 + (void) initialize
 {
-  if (self != [GSToolbarView class])
-    return;
-    
-  initSystemExtensionsColors();
-  StandardBackgroundColor = 
-    [NSColor colorWithCalibratedRed: 0.8 green: 0.8 blue: 0.8 alpha: 1.0];
-  RETAIN(StandardBackgroundColor);
-  /* Never released, but that's not a problem because the variable is static and 
-     then will be deallocated with the class when the application quits. */
+  if (self == [GSToolbarView class])
+    initSystemExtensionsColors();
 }
 
 - (id) initWithFrame: (NSRect)frame
 {
-  return [self initWithFrame: frame 
-                 displayMode: NSToolbarDisplayModeDefault 
-	            sizeMode: NSToolbarSizeModeDefault];
-}
-
-- (id) initWithFrame: (NSRect)frame 
-         displayMode: (NSToolbarDisplayMode)displayMode 
-	    sizeMode: (NSToolbarSizeMode)sizeMode
-{
-  if ((self = [super initWithFrame: frame]) != nil)
+  if ((self = [super initWithFrame: frame]) == nil)
     {
-      _displayMode = displayMode;
-      _sizeMode = sizeMode;
-      
-      switch (_sizeMode)
-        {
-	  case NSToolbarSizeModeDefault:
-	    _heightFromLayout = ToolbarViewDefaultHeight;
-	    break;
-	  case NSToolbarSizeModeRegular:
-	    _heightFromLayout = ToolbarViewRegularHeight;
-	    break;
-	  case NSToolbarSizeModeSmall:
-	    _heightFromLayout = ToolbarViewSmallHeight;
-	    break;
-	  default:
-	    // Raise exception
-	    _heightFromLayout = 0;
-	}
-    
-      [self setFrame: NSMakeRect(frame.origin.x, frame.origin.y, 
-        frame.size.width, _heightFromLayout)];
-        
-      // ---
-                 
-      _clipView = 
-        [[GSToolbarClipView alloc] initWithFrame: NSMakeRect(0, 0, 100, 100)];
-
-      [_clipView setAutoresizingMask: (NSViewWidthSizable |
-        NSViewHeightSizable)];
-	
-      [self setBorderMask: GSToolbarViewTopBorder | GSToolbarViewBottomBorder 
-        | GSToolbarViewRightBorder | GSToolbarViewLeftBorder]; 
-      // Adjust the clip view frame
-      
-      [self addSubview: _clipView];
-      
-      _clippedItemsMark = [[GSToolbarClippedItemsButton alloc] init];
-	
-      BackgroundColor = 
-        [SystemExtensionsColors colorWithKey: @"toolbarBackgroundColor"];
-      BorderColor = 
-        [SystemExtensionsColors colorWithKey: @"toolbarBorderColor"];
-      RETAIN(BackgroundColor);
-      RETAIN(BorderColor);
-      /* Never released, but that's not a problem because the variables are 
-         static and then will be deallocated with the class when the application
-	 quits. */
-      
-      // ---
-      
-     [self registerForDraggedTypes: 
-       [NSArray arrayWithObject: GSMovableToolbarItemPboardType]];
-
-      return self;
+      return nil;
     }
     
-  return nil;
-}
-
-- (id) retain
-{
-  return [super retain];
+  _heightFromLayout = ToolbarViewDefaultHeight;
+  [self setFrame: NSMakeRect(frame.origin.x, frame.origin.y, 
+                             frame.size.width, _heightFromLayout)];
+        
+  _clipView = [[NSClipView alloc] initWithFrame: 
+                                      NSMakeRect(0, 0, frame.size.width, 
+                                                 _heightFromLayout)];
+  [_clipView setAutoresizingMask: (NSViewWidthSizable | NSViewHeightSizable)];
+  [_clipView setDrawsBackground: NO];
+  [self addSubview: _clipView];
+  // Adjust the clip view frame
+  [self setBorderMask: GSToolbarViewTopBorder | GSToolbarViewBottomBorder 
+        | GSToolbarViewRightBorder | GSToolbarViewLeftBorder]; 
+  
+  _clippedItemsMark = [[GSToolbarClippedItemsButton alloc] init];
+  
+  [self registerForDraggedTypes: 
+            [NSArray arrayWithObject: GSMovableToolbarItemPboardType]];
+  
+  return self;
 }
 
 - (void) dealloc
@@ -423,7 +307,6 @@ static void initSystemExtensionsColors(void)
   
   [[NSNotificationCenter defaultCenter] removeObserver: self];
   
-  RELEASE(_toolbar);
   RELEASE(_clippedItemsMark);
   RELEASE(_clipView);
 
@@ -432,9 +315,28 @@ static void initSystemExtensionsColors(void)
 
 // Dragging related methods
 
+- (int) _insertionIndexAtPoint: (NSPoint)location
+{
+  id hitView = [self hitTest: location];
+  NSRect hitViewFrame = [hitView frame];
+  int index;
+  
+  if ((hitView != nil)
+    && ([hitView isKindOfClass: NSClassFromString(@"GSToolbarButton")] 
+    || [hitView isKindOfClass: NSClassFromString(@"GSToolbarBackView")]))
+    {
+      index = [_toolbar _indexOfItem: [hitView toolbarItem]];
+      if (location.x - hitViewFrame.origin.x > hitViewFrame.size.width / 2)
+        index++;
+      
+      return index; 
+    }
+  return NSNotFound;
+}
+
 - (NSDragOperation) draggingEntered: (id <NSDraggingInfo>)info
 {
-  GSToolbar *toolbar = [self toolbar];
+  NSToolbar *toolbar = [self toolbar];
   NSArray *allowedItemIdentifiers = 
     [[toolbar delegate] toolbarAllowedItemIdentifiers: toolbar];
   NSString *itemIdentifier = 
@@ -443,15 +345,15 @@ static void initSystemExtensionsColors(void)
   if ([self _insertionIndexAtPoint: [info draggingLocation]] != NSNotFound
     && [allowedItemIdentifiers containsObject: itemIdentifier]);
     {
-	  return NSDragOperationGeneric;
+      return NSDragOperationGeneric;
     }
-	
+        
   return NSDragOperationNone;
 }
 
 - (NSDragOperation) draggingUpdated: (id <NSDraggingInfo>)info
 {
-  GSToolbar *toolbar = [self toolbar];
+  NSToolbar *toolbar = [self toolbar];
   NSArray *allowedItemIdentifiers = 
     [[toolbar delegate] toolbarAllowedItemIdentifiers: toolbar];
   NSString *itemIdentifier = 
@@ -460,9 +362,9 @@ static void initSystemExtensionsColors(void)
   if ([self _insertionIndexAtPoint: [info draggingLocation]] != NSNotFound
     && [allowedItemIdentifiers containsObject: itemIdentifier]);
     {
-	  return NSDragOperationGeneric;
+      return NSDragOperationGeneric;
     }
-	
+        
   return NSDragOperationNone;
 }
 
@@ -471,7 +373,7 @@ static void initSystemExtensionsColors(void)
   NSPasteboard *pboard = [info draggingPasteboard];
   NSString *str = [pboard stringForType: [[pboard types] objectAtIndex: 0]];
   int index = [str intValue];
-  GSToolbar *toolbar = [self toolbar];
+  NSToolbar *toolbar = [self toolbar];
   
   [toolbar _concludeRemoveItem: 
     [[info draggingSource] toolbarItem] atIndex: index broadcast: YES];
@@ -492,15 +394,31 @@ static void initSystemExtensionsColors(void)
   NSPasteboard *pboard = [info draggingPasteboard];
   NSString *str = [pboard stringForType: [[pboard types] objectAtIndex: 0]];
   int index = [str intValue];
-  GSToolbar *toolbar = [self toolbar];
+  NSToolbar *toolbar = [self toolbar];
   NSToolbarItem *item = [[info draggingSource] toolbarItem];
   int newIndex = [self _insertionIndexAtPoint: [info draggingLocation]]; 
-  // Calculate the index
-      
-  [toolbar _insertPassivelyItem:item atIndex: index];
-  RELEASE(item);
-  [toolbar _moveItemFromIndex: index toIndex: newIndex broadcast: YES]; 
+
+  if(index == -1)
+    {
+      NSString *identifier = [item itemIdentifier];
+      if([_toolbar _containsItemWithIdentifier: identifier] == NO)
+	{
+	  [toolbar _insertItemWithItemIdentifier: identifier 
+		   atIndex: newIndex
+		   broadcast: YES];
+	}
+      RELEASE(item);
+    }
+  else
+    {
+      [toolbar _insertPassivelyItem:item atIndex: index];
+      RELEASE(item);
+      [toolbar _moveItemFromIndex: index toIndex: newIndex broadcast: YES]; 
+    }
   
+  // save the configuration...
+  [toolbar _saveConfig];
+
   return YES;
 }
 
@@ -513,48 +431,14 @@ static void initSystemExtensionsColors(void)
 
 - (void) drawRect: (NSRect)aRect
 {
-  NSBezierPath *rect = [NSBezierPath bezierPathWithRect: aRect];
-  NSRect viewFrame = [self frame];
-  
-  // We draw the background
-  if (![BackgroundColor isEqual: [NSColor clearColor]])
-    {
-      [BackgroundColor set];
-      [rect fill];
-    }
-  
-  // We draw the border
-  [BorderColor set];
-  if (_borderMask & GSToolbarViewBottomBorder)
-  {
-    [NSBezierPath strokeLineFromPoint: NSMakePoint(0, 0.5) 
-                              toPoint: NSMakePoint(viewFrame.size.width, 0.5)];
-  }
-  if (_borderMask & GSToolbarViewTopBorder)
-  {
-    [NSBezierPath strokeLineFromPoint: NSMakePoint(0, 
-                                         viewFrame.size.height - 0.5) 
-                              toPoint: NSMakePoint(viewFrame.size.width, 
-                                         viewFrame.size.height -  0.5)];
-  }
-  if (_borderMask & GSToolbarViewLeftBorder)
-  {
-    [NSBezierPath strokeLineFromPoint: NSMakePoint(0.5, 0) 
-                              toPoint: NSMakePoint(0.5, viewFrame.size.height)];
-  }
-  if (_borderMask & GSToolbarViewRightBorder)
-  {
-    [NSBezierPath strokeLineFromPoint: NSMakePoint(viewFrame.size.width - 0.5,0)
-                              toPoint: NSMakePoint(viewFrame.size.width - 0.5, 
-                                         viewFrame.size.height)];
-  }
-  
-  [super drawRect: aRect];
+  [[GSTheme theme] drawToobarRect: aRect
+                   frame: [self frame]
+                   borderMask: _borderMask];
 }
 
 - (BOOL) isOpaque
 {
-  if ([BackgroundColor isEqual: [NSColor clearColor]])
+  if ([[NSColor toolbarBackgroundColor] alphaComponent] < 1.0)
     {
       return NO;
     }
@@ -591,17 +475,17 @@ static void initSystemExtensionsColors(void)
   [nc removeObserver: self name: NSWindowDidResizeNotification object: nil];
   [nc addObserver: self selector: @selector(windowDidResize:) 
                             name: NSWindowDidResizeNotification 
-			  object: _window];
+                          object: _window];
 }
 
-// More methods... Accessors
+// Accessors
 
 - (unsigned int) borderMask
 {
   return _borderMask;
 }
 
-- (GSToolbar *) toolbar
+- (NSToolbar *) toolbar
 {
   return _toolbar;
 }
@@ -609,70 +493,49 @@ static void initSystemExtensionsColors(void)
 - (void) setBorderMask: (unsigned int)borderMask
 {
   NSRect toolbarViewFrame = [self frame];
-  NSRect rect = NSMakeRect(0, 0, toolbarViewFrame.size.width, 100);
+  NSRect rect = NSMakeRect(0, 0, toolbarViewFrame.size.width, 
+                           toolbarViewFrame.size.height);
   
   _borderMask = borderMask;
   
-  // Take in account the size mode
-  
-  switch (_sizeMode)
-    {
-      case NSToolbarSizeModeDefault:
-        rect.size.height = ToolbarViewDefaultHeight;
-	break;
-      case NSToolbarSizeModeRegular:
-        rect.size.height = ToolbarViewRegularHeight;
-	break;
-      case NSToolbarSizeModeSmall:
-        rect.size.height = ToolbarViewSmallHeight;
-	break;
-      default:
-	; // Invalid
-    }
-  
   // Take in account the border
-  
   if (_borderMask & GSToolbarViewBottomBorder)
     {
       rect = NSMakeRect(rect.origin.x, ++rect.origin.y, rect.size.width, 
-        --rect.size.height);
+                        --rect.size.height);
     }
 
   if (_borderMask & GSToolbarViewTopBorder)
     {
       rect = NSMakeRect(rect.origin.x, rect.origin.y, rect.size.width, 
-        --rect.size.height); 
+                        --rect.size.height); 
     }
     
   if (_borderMask & GSToolbarViewLeftBorder)
     {
       rect = NSMakeRect(++rect.origin.x, rect.origin.y, --rect.size.width, 
-        rect.size.height);
+                        rect.size.height);
     }
     
   if (_borderMask & GSToolbarViewRightBorder)
     {
       rect = NSMakeRect(rect.origin.x, rect.origin.y, --rect.size.width, 
-        rect.size.height);
+                        rect.size.height);
     }
     
   [_clipView setFrame: rect];
 }
 
-- (void) setToolbar: (GSToolbar *)toolbar 
+- (void) setToolbar: (NSToolbar *)toolbar 
 {
-  if ([toolbar isKindOfClass: NSClassFromString(@"NSToolbar")])
-    [NSException raise: NSInvalidArgumentException
-                format: @"NSToolbar instance can't be attached directly to a \
-                          toolbar view, setToolbar: from the NSWindow toolbar \
-                          category must be used."];
-  
-  [self _setToolbar: toolbar];
-}
+  if (_toolbar == toolbar)
+    return;
 
-- (NSColor *) standardBackgroundColor
-{
-  return StandardBackgroundColor;
+  _toolbar = toolbar;
+
+  [_clippedItemsMark setToolbar: _toolbar];
+  // Load the toolbar in the toolbar view
+  [self _reload];
 }
 
 // Private methods
@@ -697,16 +560,16 @@ static void initSystemExtensionsColors(void)
         || [item _isModified] 
         || [item _isFlexibleSpace])
         {
-	  // When a label is changed, _isModified returns YES to let us known we
-	  // must recalculate the text length and then the size for the edited
-	  // item back view
+          // When a label is changed, _isModified returns YES to let us known we
+          // must recalculate the text length and then the size for the edited
+          // item back view
           [item _layout];
         }
       
       itemBackViewFrame = [itemBackView frame];
       [itemBackView setFrame: NSMakeRect(x, itemBackViewFrame.origin.y, 
         itemBackViewFrame.size.width, itemBackViewFrame.size.height)];
-	
+        
       x += [itemBackView frame].size.width;
       
       if (itemBackViewFrame.size.height > newHeight)
@@ -717,6 +580,63 @@ static void initSystemExtensionsColors(void)
     _heightFromLayout = newHeight;
 }
 
+- (void) _takeInAccountFlexibleSpaces
+{
+  NSArray *items = [_toolbar items];
+  NSEnumerator *e = [items objectEnumerator];
+  NSToolbarItem *item;
+  NSView *backView; 
+  NSRect lastBackViewFrame;
+  float lengthAvailable;
+  unsigned int flexibleSpaceItemsNumber = 0;
+  BOOL mustAdjustNext = NO;
+  float x = 0;
+  
+  if ([items count] == 0)
+    return; 
+  
+  lastBackViewFrame = [[[items lastObject] _backView] frame];
+  lengthAvailable = [self frame].size.width - NSMaxX(lastBackViewFrame);
+
+  if (lengthAvailable < 1)
+    return;
+  
+  while ((item = [e nextObject]) != nil) 
+  {
+    if ([item _isFlexibleSpace])
+    {
+      flexibleSpaceItemsNumber++;
+    }
+  }
+  
+  if (lengthAvailable < flexibleSpaceItemsNumber)
+    return; 
+  
+  e = [items objectEnumerator];
+  while ((item = [e nextObject]) != nil)
+  {
+    backView = [item _backView];
+    if ([item _isFlexibleSpace])
+    {
+      NSRect backViewFrame = [backView frame];
+      
+      [backView setFrame: NSMakeRect(x, backViewFrame.origin.y,
+        lengthAvailable / flexibleSpaceItemsNumber, 
+        backViewFrame.size.height)];
+      mustAdjustNext = YES;
+    }
+    else if (mustAdjustNext)
+    {
+      NSRect backViewFrame = [backView frame];
+      
+      [backView setFrame: NSMakeRect(x, backViewFrame.origin.y,
+        backViewFrame.size.width, backViewFrame.size.height)];
+    }
+    x += [backView frame].size.width;
+  }
+  
+}
+
 - (void) _handleViewsVisibility
 {
   NSArray *items = [_toolbar items];
@@ -790,7 +710,7 @@ static void initSystemExtensionsColors(void)
                                       clipViewFrame.origin.y, 
                                       width,
                                       clipViewFrame.size.height)]; 
-	
+        
       // Adjust the clipped items mark frame handling   
       
       [_clippedItemsMark layout];
@@ -798,9 +718,9 @@ static void initSystemExtensionsColors(void)
       clipViewFrame = [_clipView frame]; // We get the new _clipView frame
       [_clippedItemsMark setFrameOrigin: NSMakePoint(
         [self frame].size.width - ClippedItemsViewWidth, clipViewFrame.origin.y)];
-	
+        
       // ---
-	
+        
       if ([_clippedItemsMark superview] == nil)       
         [self addSubview: _clippedItemsMark];  
       
@@ -811,10 +731,10 @@ static void initSystemExtensionsColors(void)
       [_clippedItemsMark removeFromSuperview];
       
       [_clipView setFrame: NSMakeRect(clipViewFrame.origin.x, 
-        clipViewFrame.origin.y, [self frame].size.width, 
-	clipViewFrame.size.height)]; 
+                                      clipViewFrame.origin.y, 
+                                      [self frame].size.width, 
+                                      clipViewFrame.size.height)]; 
     }
- 
 }
 
 - (void) _reload 
@@ -823,97 +743,6 @@ static void initSystemExtensionsColors(void)
   [self setNeedsDisplay: YES];
 }
 
-- (void) _setToolbar: (GSToolbar *)toolbar
-{
-  if ([toolbar sizeMode] != _sizeMode)
-    ; // FIXME: Raise exception here
-  
-  [toolbar _setToolbarView: self]; // We set the toolbar view on the new toolbar
-  [_toolbar _setToolbarView: nil]; // We unset the toolbar view from the previous toolbar    
-  
-  ASSIGN(_toolbar, toolbar);
-  
-  [_clippedItemsMark setToolbar: _toolbar];
-  
-  [self _reload]; // Load the toolbar in the toolbar view
-}
-
-- (void) _takeInAccountFlexibleSpaces
-{
-  NSArray *items = [_toolbar items];
-  NSEnumerator *e = [items objectEnumerator];
-  NSToolbarItem *item;
-  NSView *backView; 
-  NSRect lastBackViewFrame;
-  float lengthAvailable;
-  unsigned int flexibleSpaceItemsNumber = 0;
-  BOOL mustAdjustNext = NO;
-  float x = 0;
-  
-  if ([items count] == 0)
-    return; 
-  
-  lastBackViewFrame = [[[items lastObject] _backView] frame];
-  lengthAvailable = [self frame].size.width - NSMaxX(lastBackViewFrame);
-
-  if (lengthAvailable < 1)
-    return;
-  
-  while ((item = [e nextObject]) != nil) 
-  {
-    if ([item _isFlexibleSpace])
-    {
-      flexibleSpaceItemsNumber++;
-    }
-  }
-  
-  if (lengthAvailable < flexibleSpaceItemsNumber)
-    return; 
-  
-  e = [items objectEnumerator];
-  while ((item = [e nextObject]) != nil)
-  {
-    backView = [item _backView];
-    if ([item _isFlexibleSpace])
-    {
-      NSRect backViewFrame = [backView frame];
-      
-      [backView setFrame: NSMakeRect(x, backViewFrame.origin.y,
-        lengthAvailable / flexibleSpaceItemsNumber, 
-	backViewFrame.size.height)];
-      mustAdjustNext = YES;
-    }
-    else if (mustAdjustNext)
-    {
-      NSRect backViewFrame = [backView frame];
-      
-      [backView setFrame: NSMakeRect(x, backViewFrame.origin.y,
-        backViewFrame.size.width, backViewFrame.size.height)];
-    }
-    x += [backView frame].size.width;
-  }
-  
-}
-
-- (int) _insertionIndexAtPoint: (NSPoint)location
-{
-  id hitView = [self hitTest: location];
-  NSRect hitViewFrame = [hitView frame];
-  int index;
-  
-  if ((hitView != nil)
-    && ([hitView isKindOfClass: NSClassFromString(@"GSToolbarButton")] 
-    || [hitView isKindOfClass: NSClassFromString(@"GSToolbarBackView")]))
-    {
-      index = [_toolbar _indexOfItem: [hitView toolbarItem]];
-      if (location.x - hitViewFrame.origin.x > hitViewFrame.size.width / 2)
-        index++;
-      
-      return index; 
-    }
-  return NSNotFound;
-}
-
 // Accessors private methods
 
 - (float) _heightFromLayout
@@ -965,47 +794,21 @@ static void initSystemExtensionsColors(void)
   
 }
 
-- (void) _setSizeMode: (NSToolbarSizeMode)sizeMode
+- (NSColor *) standardBackgroundColor
 {
-  _sizeMode = sizeMode;
-}
-
-- (NSToolbarSizeMode) _sizeMode
-{
-  return _sizeMode;
-}
-
-/*
- * _willBeVisible indicates that the toolbar view previously hidden is in
- * process to become visible again before the end of current the event loop.
- */
-- (BOOL) _willBeVisible
-{
-  return _willBeVisible;
-}
-
-- (void) _setWillBeVisible: (BOOL)willBeVisible
-{
-  _willBeVisible = willBeVisible;
+  NSLog(@"Use of deprecated method %@", NSStringFromSelector(_cmd));
+  return nil;
 }
 
 - (BOOL) _usesStandardBackgroundColor
 {
-  return [BackgroundColor isEqual: [self standardBackgroundColor]];
+  NSLog(@"Use of deprecated method %@", NSStringFromSelector(_cmd));
+  return NO;
 }
 
 - (void) _setUsesStandardBackgroundColor: (BOOL)standard
 {
-  if (standard)
-    {
-      RELEASE(BackgroundColor);
-      BackgroundColor = [self standardBackgroundColor];
-    }
-  else
-    {
-      BackgroundColor = [NSColor clearColor];
-      RETAIN(BackgroundColor);
-    }
+  NSLog(@"Use of deprecated method %@", NSStringFromSelector(_cmd));
 }
 
 @end
