@@ -55,6 +55,7 @@
 #include "GNUstepGUI/IMLoading.h"
 #include "GNUstepGUI/GMAppKit.h"
 #include "GNUstepGUI/GMArchiver.h"
+#include "GSGuiPrivate.h"
 
 extern NSThread *GSAppKitThread;
 
@@ -350,6 +351,7 @@ setControl(NSView* content, id control, NSString *title)
     return nil;
 
   [self setTitle: @" "];
+  [self setLevel: NSModalPanelWindowLevel];
   content = [self contentView];
   
   // we're an ATTENTION panel, therefore:
@@ -1292,14 +1294,21 @@ void NSBeginAlertSheet(NSString *title,
 
   panel = getSomePanel(&standardAlertPanel, defaultTitle, title, message,
     defaultButton, alternateButton, otherButton);
-  
   // FIXME: We should also change the button action to call endSheet:
   [NSApp beginSheet: panel
 	 modalForWindow: docWindow
 	 modalDelegate: modalDelegate
-	 didEndSelector: didEndSelector
+	 didEndSelector: willEndSelector
 	 contextInfo: contextInfo];
   [panel close];
+  if (modalDelegate && [modalDelegate respondsToSelector: didEndSelector])
+    {
+      void (*didEnd)(id, SEL, id, int, void*);
+
+      didEnd = (void (*)(id, SEL, id, int, void*))[modalDelegate methodForSelector: 
+								 didEndSelector];
+      didEnd(modalDelegate, didEndSelector, panel, [panel result], contextInfo);
+    }
   NSReleaseAlertPanel(panel);
 }
 
@@ -1324,14 +1333,21 @@ void NSBeginCriticalAlertSheet(NSString *title,
 
   panel = getSomePanel(&criticalAlertPanel, @"Critical", title, message,
     defaultButton, alternateButton, otherButton);
-
   // FIXME: We should also change the button action to call endSheet:
   [NSApp beginSheet: panel
 	 modalForWindow: docWindow
 	 modalDelegate: modalDelegate
-	 didEndSelector: didEndSelector
+	 didEndSelector: willEndSelector
 	 contextInfo: contextInfo];
   [panel close];
+  if (modalDelegate && [modalDelegate respondsToSelector: didEndSelector])
+    {
+      void (*didEnd)(id, SEL, id, int, void*);
+
+      didEnd = (void (*)(id, SEL, id, int, void*))[modalDelegate methodForSelector: 
+								 didEndSelector];
+      didEnd(modalDelegate, didEndSelector, panel, [panel result], contextInfo);
+    }
   NSReleaseAlertPanel(panel);
 }
 
@@ -1358,14 +1374,21 @@ void NSBeginInformationalAlertSheet(NSString *title,
 		      @"Information",
 		      title, message,
 		      defaultButton, alternateButton, otherButton);
-
   // FIXME: We should also change the button action to call endSheet:
   [NSApp beginSheet: panel
 	 modalForWindow: docWindow
 	 modalDelegate: modalDelegate
-	 didEndSelector: didEndSelector
+	 didEndSelector: willEndSelector
 	 contextInfo: contextInfo];
   [panel close];
+  if (modalDelegate && [modalDelegate respondsToSelector: didEndSelector])
+    {
+      void (*didEnd)(id, SEL, id, int, void*);
+
+      didEnd = (void (*)(id, SEL, id, int, void*))[modalDelegate methodForSelector: 
+								 didEndSelector];
+      didEnd(modalDelegate, didEndSelector, panel, [panel result], contextInfo);
+    }
   NSReleaseAlertPanel(panel);
 }
 
@@ -1634,23 +1657,14 @@ void NSBeginInformationalAlertSheet(NSString *title,
 		      contextInfo: (void *)contextInfo
 {
   [self _setupPanel];
-  if (GSCurrentThread() != GSAppKitThread)
-    {
-      [self performSelectorOnMainThread: _cmd
-			     withObject: nil
-			  waitUntilDone: YES];
-    }
-  else
-    {
-      _modalDelegate = delegate;
-      _didEndSelector = didEndSelector;
-      [NSApp beginSheet: _window
-	 modalForWindow: window
-	  modalDelegate: self
+  _modalDelegate = delegate;
+  _didEndSelector = didEndSelector;
+  [NSApp beginSheet: _window
+         modalForWindow: window
+         modalDelegate: self
          didEndSelector: @selector(_alertDidEnd:returnCode:contextInfo:)
-	    contextInfo: contextInfo];
-      DESTROY(_window);
-    }
+         contextInfo: contextInfo];
+  DESTROY(_window);
 }
 
 - (void) _alertDidEnd: (NSWindow *)sheet
