@@ -195,8 +195,8 @@ nsanimation_progressMarkSorter ( NSAnimationProgress first,NSAnimationProgress s
   if (_isThreaded)                  \
   {                                 \
     NSAssert(__gs_isLocked == NO, NSInternalInconsistencyException); \
-    NSDebugFLLog(@"NSAnimationLock",\
-                 @"%@ LOCK %@",self,[NSThread currentThread]);\
+    NSDebugMLLog(@"NSAnimationLock",\
+                 @"LOCK %@", [NSThread currentThread]);\
     [_isAnimatingLock lock];        \
     __gs_isLocked = YES;            \
    }
@@ -205,8 +205,8 @@ nsanimation_progressMarkSorter ( NSAnimationProgress first,NSAnimationProgress s
   if (__gs_isLocked)                \
   {                                 \
     /* NSAssert(__gs_isLocked == YES, NSInternalInconsistencyException); */ \
-    NSDebugFLLog(@"NSAnimationLock",\
-                 @"%@ UNLOCK %@",self,[NSThread currentThread]);\
+    NSDebugMLLog(@"NSAnimationLock",\
+                 @"UNLOCK %@", [NSThread currentThread]);\
     __gs_isLocked = NO;             \
     [_isAnimatingLock unlock];      \
   }
@@ -238,9 +238,9 @@ nsanimation_progressMarkSorter ( NSAnimationProgress first,NSAnimationProgress s
   if (GSIArrayCount(_progressMarks) == 0)
     { // First mark
       GSIArrayAddItem (_progressMarks,progress);
-      NSDebugFLLog (@"NSAnimationMark",
-                    @"%@ Insert 1st mark for %f (next:#%d)",
-                    self,progress,_nextMark);
+      NSDebugMLLog (@"NSAnimationMark",
+                    @"Insert 1st mark for %f (next:#%d)",
+                    progress, _nextMark);
       _nextMark = (progress >= [self currentProgress])? 0 : 1;
     }
   else 
@@ -254,9 +254,9 @@ nsanimation_progressMarkSorter ( NSAnimationProgress first,NSAnimationProgress s
             && progress < GSIArrayItemAtIndex(_progressMarks,_nextMark))
           _nextMark++;
       GSIArrayInsertItem (_progressMarks,progress,index);
-      NSDebugFLLog (@"NSAnimationMark",
-                    @"%@ Insert mark #%d/%d for %f (next:#%d)",
-                    self,index,GSIArrayCount(_progressMarks),progress,_nextMark);
+      NSDebugMLLog (@"NSAnimationMark",
+                    @"Insert mark #%d/%d for %f (next:#%d)",
+                    index,GSIArrayCount(_progressMarks),progress,_nextMark);
     }
   _isCachedProgressMarkNumbersValid = NO;
 
@@ -333,8 +333,8 @@ nsanimation_progressMarkSorter ( NSAnimationProgress first,NSAnimationProgress s
 
   if (_delegate_animationValueForProgress)
     { // method is cached (the animation is running)
-      NSDebugFLLog (@"NSAnimationDelegate",
-                    @"%@ [delegate animationValueForProgress] (cached)",self);
+      NSDebugMLLog (@"NSAnimationDelegate",
+                    @"[delegate animationValueForProgress] (cached)");
       value = (*_delegate_animationValueForProgress)
                 (GS_GC_UNHIDE (_currentDelegate),
                  @selector (animation:valueForProgress:),
@@ -345,8 +345,8 @@ nsanimation_progressMarkSorter ( NSAnimationProgress first,NSAnimationProgress s
          && [GS_GC_UNHIDE (_delegate) respondsToSelector:
                @selector (animation:valueForProgress:)] )
       {
-        NSDebugFLLog (@"NSAnimationDelegate",
-                      @"%@ [delegate animationValueForProgress]",self);
+        NSDebugMLLog (@"NSAnimationDelegate",
+                      @"[delegate animationValueForProgress]");
         value = [GS_GC_UNHIDE (_delegate) animation: self
                                    valueForProgress: _currentProgress];
       }
@@ -459,21 +459,21 @@ nsanimation_progressMarkSorter ( NSAnimationProgress first,NSAnimationProgress s
 {
   [self stopAnimation];
 
-  GSIArrayEmpty (_progressMarks);
-  NSZoneFree ([self zone], _progressMarks);
+  GSIArrayEmpty(_progressMarks);
+  NSZoneFree([self zone], _progressMarks);
   if (_cachedProgressMarkNumbers != NULL)
     {
       unsigned i;
-      for ( i=0; i<_cachedProgressMarkNumberCount; i++)
-        RELEASE (_cachedProgressMarkNumbers[i]);
-      NSZoneFree ([self zone], _cachedProgressMarkNumbers);
+
+      for (i = 0; i < _cachedProgressMarkNumberCount; i++)
+        RELEASE(_cachedProgressMarkNumbers[i]);
+      NSZoneFree([self zone], _cachedProgressMarkNumbers);
     }
 
   [self clearStartAnimation];
   [self clearStopAnimation];
 
-  TEST_RELEASE (_animator);
-
+  TEST_RELEASE(_animator);
   RELEASE(_isAnimatingLock);
 
   [super dealloc];
@@ -485,7 +485,7 @@ nsanimation_progressMarkSorter ( NSAnimationProgress first,NSAnimationProgress s
   _NSANIMATION_LOCKING_SETUP;
 
   _NSANIMATION_LOCK;
-  f = (_animator != nil)? [_animator isAnimationRunning] : NO;
+  f = (_animator != nil) ? [_animator isAnimationRunning] : NO;
   _NSANIMATION_UNLOCK;
   return f;
 }
@@ -498,7 +498,7 @@ nsanimation_progressMarkSorter ( NSAnimationProgress first,NSAnimationProgress s
 
   _NSANIMATION_LOCK;
 
-  count = GSIArrayCount (_progressMarks);
+  count = GSIArrayCount(_progressMarks);
 
   if (!_isCachedProgressMarkNumbersValid)
     {
@@ -506,17 +506,19 @@ nsanimation_progressMarkSorter ( NSAnimationProgress first,NSAnimationProgress s
 
       if (_cachedProgressMarkNumbers != NULL)
         {   
-          for ( i=0; i<_cachedProgressMarkNumberCount; i++)
-            RELEASE (_cachedProgressMarkNumbers[i]);
+          for (i = 0; i < _cachedProgressMarkNumberCount; i++)
+            RELEASE(_cachedProgressMarkNumbers[i]);
           _cachedProgressMarkNumbers =
-           (NSNumber**)NSZoneRealloc ([self zone], _cachedProgressMarkNumbers,count*sizeof(NSNumber*));
+           (NSNumber**)NSZoneRealloc([self zone], _cachedProgressMarkNumbers,
+                                     count * sizeof(NSNumber*));
         }
       else
         {
           _cachedProgressMarkNumbers =
-           (NSNumber**)NSZoneMalloc ([self zone], count*sizeof(NSNumber*));
+           (NSNumber**)NSZoneMalloc([self zone], count * sizeof(NSNumber*));
         }
-      for ( i=0; i<count; i++)
+
+      for (i = 0; i < count; i++)
         {
           _cachedProgressMarkNumbers[i] =
            [NSNumber numberWithFloat: GSIArrayItemAtIndex (_progressMarks,i)];
@@ -538,17 +540,19 @@ nsanimation_progressMarkSorter ( NSAnimationProgress first,NSAnimationProgress s
 
   _NSANIMATION_LOCK;
 
-  index = GSIArraySearch (_progressMarks,progress,nsanimation_progressMarkSorter);
-  if ( index < GSIArrayCount(_progressMarks)
-      && progress == GSIArrayItemAtIndex (_progressMarks,index) )
+  index = GSIArraySearch(_progressMarks, progress, 
+                         nsanimation_progressMarkSorter);
+  if (index < GSIArrayCount(_progressMarks)
+      && progress == GSIArrayItemAtIndex (_progressMarks,index))
     {
-      GSIArrayRemoveItemAtIndex (_progressMarks,index);
+      GSIArrayRemoveItemAtIndex(_progressMarks,index);
       _isCachedProgressMarkNumbersValid = NO;
       if (_nextMark > index) _nextMark--;
-      NSDebugFLLog (@"NSAnimationMark",@"%@ Remove mark #%d for (next:#%d)",self,index,progress,_nextMark);
+      NSDebugMLLog(@"NSAnimationMark",@"Remove mark #%d for (next:#%d)",
+                   index, progress, _nextMark);
     }
   else
-    NSWarnFLog (@"%@ Unexistent progress mark",self);
+    NSWarnMLog(@"Unexistent progress mark");
 
   _NSANIMATION_UNLOCK;
 }
@@ -668,11 +672,11 @@ nsanimation_progressMarkSorter ( NSAnimationProgress first,NSAnimationProgress s
       _nextMark = GSIArrayInsertionPosition (_progressMarks,progress,&nsanimation_progressMarkSorter);
 
       if (_nextMark < GSIArrayCount(_progressMarks))
-        NSDebugFLLog (@"NSAnimationMark",@"%@ Next mark #%d for %f",
-            self,_nextMark, GSIArrayItemAtIndex (_progressMarks,_nextMark));
+        NSDebugMLLog(@"NSAnimationMark",@"Next mark #%d for %f",
+                     _nextMark, GSIArrayItemAtIndex(_progressMarks,_nextMark));
     }
 
-  NSDebugFLLog (@"NSAnimation",@"%@ Progress = %f",self,progress);
+  NSDebugMLLog(@"NSAnimation",@"Progress = %f", progress);
   _currentProgress = progress;
 
   if (progress >= 1.0 && _animator != nil)
@@ -726,13 +730,14 @@ nsanimation_progressMarkSorter ( NSAnimationProgress first,NSAnimationProgress s
   _NSANIMATION_LOCKING_SETUP;
 
   _NSANIMATION_LOCK;
-  GSIArrayEmpty (_progressMarks);
+  GSIArrayEmpty(_progressMarks);
   _nextMark = 0;
   if (marks != nil)
     {
-      unsigned i, count=[marks count];
-      for (i=0;i<count;i++)
-        [self addProgressMark:[(NSNumber*)[marks objectAtIndex:i] floatValue]];
+      unsigned i, count = [marks count];
+
+      for (i = 0; i < count; i++)
+        [self addProgressMark: [(NSNumber*)[marks objectAtIndex:i] floatValue]];
     }
   _isCachedProgressMarkNumbersValid = NO;
   _NSANIMATION_UNLOCK;
@@ -745,12 +750,11 @@ nsanimation_progressMarkSorter ( NSAnimationProgress first,NSAnimationProgress s
   if ([self isAnimating])
     return;
 
-  NSDebugFLLog (@"NSAnimationStart",@"%@",self);
+  NSDebugMLLog(@"NSAnimationStart",@"");
 
-  for (i=0; i<GSIArrayCount(_progressMarks); i++)
-    NSDebugFLLog (@"NSAnimationMark",
-                  @"%@ Mark #%d : %f",
-                  self,i,GSIArrayItemAtIndex(_progressMarks,i));
+  for (i = 0; i < GSIArrayCount(_progressMarks); i++)
+    NSDebugMLLog(@"NSAnimationMark", @"Mark #%d : %f",
+                 i, GSIArrayItemAtIndex(_progressMarks,i));
 
   if ([self currentProgress] >= 1.0) 
     {
@@ -765,10 +769,9 @@ nsanimation_progressMarkSorter ( NSAnimationProgress first,NSAnimationProgress s
     {
       id delegate;
 
-      NSDebugFLLog (@"NSAnimationDelegate",
-                    @"%@ Cache delegation methods",self);
+      NSDebugMLLog(@"NSAnimationDelegate", @"Cache delegation methods");
       // delegation methods are cached while the animation is running
-      delegate = GS_GC_UNHIDE (_delegate);
+      delegate = GS_GC_UNHIDE(_delegate);
       _delegate_animationDidReachProgressMark =
         ([delegate respondsToSelector: @selector (animation:didReachProgressMark:)]) ?
         (void (*)(id,SEL,NSAnimation*,NSAnimationProgress))
@@ -794,19 +797,19 @@ nsanimation_progressMarkSorter ( NSAnimationProgress first,NSAnimationProgress s
         (BOOL (*)(id,SEL,NSAnimation*))
         [delegate methodForSelector: @selector (animationShouldStart:)]
         : NULL;
-      NSDebugFLLog (@"NSAnimationDelegate",
-                    @"%@ Delegation methods : %x %x %x %x %x", self,
-          _delegate_animationDidReachProgressMark,
-          _delegate_animationValueForProgress,
-          _delegate_animationDidEnd,
-          _delegate_animationDidStop,
-          _delegate_animationShouldStart);
+      NSDebugMLLog(@"NSAnimationDelegate",
+                   @"Delegation methods : %x %x %x %x %x",
+                   _delegate_animationDidReachProgressMark,
+                   _delegate_animationValueForProgress,
+                   _delegate_animationDidEnd,
+                   _delegate_animationDidStop,
+                   _delegate_animationShouldStart);
       _currentDelegate = _delegate;
     }
   else
     {
-      NSDebugFLLog (@"NSAnimationDelegate",
-                    @"%@ No delegate : clear delegation methods",self);
+      NSDebugMLLog(@"NSAnimationDelegate",
+                   @" No delegate : clear delegation methods");
       _delegate_animationDidReachProgressMark =
         (void (*)(id,SEL,NSAnimation*,NSAnimationProgress)) NULL;
       _delegate_animationValueForProgress =
@@ -820,17 +823,15 @@ nsanimation_progressMarkSorter ( NSAnimationProgress first,NSAnimationProgress s
       _currentDelegate = nil;
     }
   
-  if (_animator==nil || _isANewAnimatorNeeded)
+  if (_animator == nil || _isANewAnimatorNeeded)
     {
-      TEST_RELEASE (_animator);
+      TEST_RELEASE(_animator);
 
-      _animator = [GSAnimator
-        animatorWithAnimation: self
-                    frameRate: _frameRate
-                         zone: [self zone]];
-      NSAssert (_animator,@"Can not create a GSAnimator");
-      RETAIN (_animator);
-      NSDebugFLLog (@"NSAnimationAnimator",@"%@ New GSAnimator: %@", self,[_animator class]);
+      _animator = [[GSAnimator allocWithZone: [self zone]]
+                      initWithAnimation: self
+                      frameRate: _frameRate];
+      NSAssert(_animator,@"Can not create a GSAnimator");
+      NSDebugMLLog(@"NSAnimationAnimator", @"New GSAnimator: %@", _animator);
       _isANewAnimatorNeeded = NO;
     }
 
@@ -844,7 +845,8 @@ nsanimation_progressMarkSorter ( NSAnimationProgress first,NSAnimationProgress s
         break;
       case NSAnimationNonblocking:
         {
-          NSArray* runLoopModes;
+          NSArray *runLoopModes;
+
           runLoopModes = [self runLoopModesForAnimating];
           if (runLoopModes == nil)
             runLoopModes = _NSAnimationDefaultRunLoopModes;
@@ -872,7 +874,7 @@ nsanimation_progressMarkSorter ( NSAnimationProgress first,NSAnimationProgress s
   _startMark = start;
 
   [_startAnimation addProgressMark: _startMark];
-  NSDebugFLLog (@"NSAnimationMark",@"%@ register for progress %f",self,start);
+  NSDebugMLLog (@"NSAnimationMark",@"register for progress %f", start);
   [[NSNotificationCenter defaultCenter]
     addObserver: self
        selector: @selector (_gs_startAnimationReachesProgressMark:)
@@ -905,7 +907,7 @@ nsanimation_progressMarkSorter ( NSAnimationProgress first,NSAnimationProgress s
   _stopMark = stop;
 
   [_stopAnimation addProgressMark: _stopMark];
-  NSDebugFLLog (@"NSAnimationMark",@"%@ register for progress %f",self,stop);
+  NSDebugMLLog (@"NSAnimationMark",@"register for progress %f", stop);
   [[NSNotificationCenter defaultCenter]
     addObserver: self
        selector: @selector (_gs_stopAnimationReachesProgressMark:)
@@ -934,7 +936,7 @@ nsanimation_progressMarkSorter ( NSAnimationProgress first,NSAnimationProgress s
   id delegate;
   _NSANIMATION_LOCKING_SETUP;
 
-  NSDebugFLLog (@"NSAnimationAnimator",@"%@",self);
+  NSDebugMLLog(@"NSAnimationAnimator",@"");
 
   _NSANIMATION_LOCK;
 
@@ -942,7 +944,7 @@ nsanimation_progressMarkSorter ( NSAnimationProgress first,NSAnimationProgress s
 
   if (_delegate_animationShouldStart) // method is cached (the animation is running)
     {
-      NSDebugFLLog (@"NSAnimationDelegate",@"%@ [delegate animationShouldStart] (cached)",self);
+      NSDebugMLLog(@"NSAnimationDelegate",@"[delegate animationShouldStart] (cached)");
       _delegate_animationShouldStart (delegate,@selector(animationShouldStart:),self);
     }
   RETAIN (self);
@@ -955,7 +957,7 @@ nsanimation_progressMarkSorter ( NSAnimationProgress first,NSAnimationProgress s
   id delegate;
   _NSANIMATION_LOCKING_SETUP;
 
-  NSDebugFLLog (@"NSAnimationAnimator",@"%@ Progress = %f",self,_currentProgress);
+  NSDebugMLLog(@"NSAnimationAnimator",@"Progress = %f", _currentProgress);
 
   _NSANIMATION_LOCK;
 
@@ -964,7 +966,7 @@ nsanimation_progressMarkSorter ( NSAnimationProgress first,NSAnimationProgress s
     {
       if (_delegate_animationDidStop) // method is cached (the animation is running)
         {
-          NSDebugFLLog (@"NSAnimationDelegate",@"%@ [delegate animationDidStop] (cached)",self);
+          NSDebugMLLog(@"NSAnimationDelegate",@"[delegate animationDidStop] (cached)");
           _delegate_animationDidStop (delegate,@selector(animationDidStop:),self);
         }
     }
@@ -972,7 +974,7 @@ nsanimation_progressMarkSorter ( NSAnimationProgress first,NSAnimationProgress s
     {
       if (_delegate_animationDidEnd) // method is cached (the animation is running)
         {
-          NSDebugFLLog (@"NSAnimationDelegate",@"%@ [delegate animationDidEnd] (cached)",self);
+          NSDebugMLLog(@"NSAnimationDelegate",@"[delegate animationDidEnd] (cached)");
           _delegate_animationDidEnd (delegate,@selector(animationDidEnd:),self);
         }
     }
@@ -986,7 +988,7 @@ nsanimation_progressMarkSorter ( NSAnimationProgress first,NSAnimationProgress s
   NSAnimationProgress progress;
   _NSANIMATION_LOCKING_SETUP;
 
-  NSDebugFLLog (@"NSAnimationAnimator",@"%@ Elapsed time : %f",self,elapsedTime);
+  NSDebugMLLog(@"NSAnimationAnimator", @"Elapsed time : %f", elapsedTime);
 
   _NSANIMATION_LOCK;
 
@@ -1023,8 +1025,8 @@ nsanimation_progressMarkSorter ( NSAnimationProgress first,NSAnimationProgress s
   animation = [notification object];
   mark = [[[notification userInfo] objectForKey: NSAnimationProgressMark] floatValue];
 
-  NSDebugFLLog (@"NSAnimationMark",
-                @"%@ Start Animation %@ reaches %f",self,animation,mark);
+  NSDebugMLLog(@"NSAnimationMark",
+               @"Start Animation %@ reaches %f", animation, mark);
 
   if ( animation == _startAnimation && mark == _startMark)
     {
@@ -1046,8 +1048,8 @@ nsanimation_progressMarkSorter ( NSAnimationProgress first,NSAnimationProgress s
   animation = [notification object];
   mark = [[[notification userInfo] objectForKey: NSAnimationProgressMark] floatValue];
 
-  NSDebugFLLog (@"NSAnimationMark",
-                @"%@ Stop Animation %@ reaches %f",self,animation,mark);
+  NSDebugMLLog(@"NSAnimationMark",
+               @"Stop Animation %@ reaches %f",animation, mark);
 
 
   if ( animation == _stopAnimation && mark == _stopMark)
@@ -1067,15 +1069,15 @@ nsanimation_progressMarkSorter ( NSAnimationProgress first,NSAnimationProgress s
 {
   _NSANIMATION_LOCKING_SETUP;
 
-  NSDebugFLLog (@"NSAnimationMark",@"%@ progress %f",self, progress);
+  NSDebugMLLog(@"NSAnimationMark", @"progress %f", progress);
 
   _NSANIMATION_LOCK;
 
   // calls delegate's method
   if (_delegate_animationDidReachProgressMark) // method is cached (the animation is running)
     {
-      NSDebugFLLog (@"NSAnimationDelegate",
-                    @"%@ [delegate animationdidReachProgressMark] (cached)",self);
+      NSDebugMLLog(@"NSAnimationDelegate",
+                   @"[delegate animationdidReachProgressMark] (cached)");
       _delegate_animationDidReachProgressMark (GS_GC_UNHIDE(_currentDelegate),
                                                @selector(animation:didReachProgressMark:),
                                                self,progress);
@@ -1085,14 +1087,14 @@ nsanimation_progressMarkSorter ( NSAnimationProgress first,NSAnimationProgress s
          && [GS_GC_UNHIDE (_delegate)
               respondsToSelector: @selector(animation:didReachProgressMark:)] )
       {
-        NSDebugFLLog (@"NSAnimationDelegate",
-                      @"%@ [delegate animationdidReachProgressMark]",self);
+        NSDebugMLLog(@"NSAnimationDelegate",
+                     @"[delegate animationdidReachProgressMark]");
         [GS_GC_UNHIDE (_delegate) animation: self didReachProgressMark: progress];
       }
 
   // posts a notification
-  NSDebugFLLog (@"NSAnimationNotification",
-                @"%@ Post NSAnimationProgressMarkNotification : %f",self,progress);
+  NSDebugMLLog(@"NSAnimationNotification",
+               @"Post NSAnimationProgressMarkNotification : %f", progress);
   [[NSNotificationCenter defaultCenter]
     postNotificationName: NSAnimationProgressMarkNotification
 		  object: self
@@ -1104,41 +1106,59 @@ nsanimation_progressMarkSorter ( NSAnimationProgress first,NSAnimationProgress s
 
   // skips marks with the same progress value
   while (
-    (++_nextMark) < GSIArrayCount (_progressMarks)
-    && GSIArrayItemAtIndex (_progressMarks,_nextMark) == progress
+    (++_nextMark) < GSIArrayCount(_progressMarks)
+    && GSIArrayItemAtIndex(_progressMarks, _nextMark) == progress
     )
   ;
 
   _NSANIMATION_UNLOCK;
 
-  NSDebugFLLog (@"NSAnimationMark",
-                @"%@ Next mark #%d for %f",
-                self,_nextMark,GSIArrayItemAtIndex(_progressMarks,_nextMark));
+  NSDebugMLLog(@"NSAnimationMark",
+               @"Next mark #%d for %f",
+               _nextMark, GSIArrayItemAtIndex(_progressMarks, _nextMark - 1));
 }
 
 - (void) _gs_startThreadedAnimation
 {
   // NSAssert(_isThreaded);
-  CREATE_AUTORELEASE_POOL (pool);
-  NSDebugFLLog (@"NSAnimationThread",
-                @"%@ Start of %@",self,[NSThread currentThread]);
+  CREATE_AUTORELEASE_POOL(pool);
+  NSDebugMLLog(@"NSAnimationThread",
+               @"Start of %@", [NSThread currentThread]);
   [self _gs_startAnimationInOwnLoop];
-  NSDebugFLLog (@"NSAnimationThread",
-                @"%@ End of %@",self,[NSThread currentThread]);
-  RELEASE (pool);
+  NSDebugMLLog(@"NSAnimationThread",
+               @"End of %@", [NSThread currentThread]);
+  RELEASE(pool);
   _isThreaded = NO;
 }
 
 
 - (void) _gs_startAnimationInOwnLoop
 {
+  NSRunLoop	*loop;
+  NSDate *end;
+
   [_animator setRunLoopModesForAnimating:
     [NSArray arrayWithObject: NSAnimationBlockingRunLoopMode]];
   [_animator startAnimation];
-  while ( [[NSRunLoop currentRunLoop]
-          runMode: NSAnimationBlockingRunLoopMode
-       beforeDate: [NSDate distantFuture]] )
-    /* do nothing */;
+  loop = [NSRunLoop currentRunLoop];
+  end = [NSDate distantFuture];
+  for (;;)
+    {
+      if ([loop runMode: NSAnimationBlockingRunLoopMode beforeDate: end] == NO)
+        {
+          NSDate	*d;
+          CREATE_AUTORELEASE_POOL(pool);
+
+          d = [loop limitDateForMode: NSAnimationBlockingRunLoopMode];
+          if (d == nil)
+            {
+              RELEASE(pool);
+              break;	// No inputs and no timers.
+            }
+          [NSThread sleepUntilDate: d];
+          RELEASE(pool);
+        }
+    }
 }
 
 - (_NSAnimationCurveDesc*) _gs_curveDesc
@@ -1385,16 +1405,15 @@ NSString *NSViewAnimationFadeOutEffect = @"NSViewAnimationFadeOutEffect";
   if (self)
     {
       [self setAnimationBlockingMode: NSAnimationNonblocking];
-      _viewAnimations = [animations retain];
-      _viewAnimationDesc = nil;
+      [self setViewAnimations: animations];
     }
   return self;
 }
 
 - (void) dealloc
 {
-  RELEASE (_viewAnimations);
-  RELEASE (_viewAnimationDesc);
+  RELEASE(_viewAnimations);
+  RELEASE(_viewAnimationDesc);
   [super dealloc];
 }
 
@@ -1404,8 +1423,8 @@ NSString *NSViewAnimationFadeOutEffect = @"NSViewAnimationFadeOutEffect";
 
   _NSANIMATION_LOCK;
   if (_viewAnimations != animations)
-    DESTROY (_viewAnimationDesc);
-  ASSIGN (_viewAnimations, animations) ;
+    DESTROY(_viewAnimationDesc);
+  ASSIGN(_viewAnimations, animations) ;
   _NSANIMATION_UNLOCK;
 }
 
@@ -1426,17 +1445,21 @@ NSString *NSViewAnimationFadeOutEffect = @"NSViewAnimationFadeOutEffect";
 
   _NSANIMATION_LOCK;
   if (_viewAnimationDesc == nil)
-  {
-    unsigned i,c;
-    c = [_viewAnimations count];
-    _viewAnimationDesc = [NSMutableArray arrayWithCapacity: c];
-    RETAIN (_viewAnimationDesc);
-    for (i=0;i<c;i++)
-      [_viewAnimationDesc
-	addObject: [[_GSViewAnimationBaseDesc alloc]
-                      initWithProperties: [_viewAnimations objectAtIndex:i]]
-      ];
-  }
+    {
+      unsigned int i, c;
+
+      c = [_viewAnimations count];
+      _viewAnimationDesc = [[NSMutableArray alloc] initWithCapacity: c];
+      for (i = 0; i < c; i++)
+        {
+          _GSViewAnimationBaseDesc *vabd;
+
+          vabd = [[_GSViewAnimationBaseDesc alloc]
+                     initWithProperties: [_viewAnimations objectAtIndex:i]];
+          [_viewAnimationDesc addObject: vabd];
+          RELEASE(vabd);
+        }
+    }
   [super startAnimation];
   _NSANIMATION_UNLOCK;
 }
@@ -1454,11 +1477,12 @@ NSString *NSViewAnimationFadeOutEffect = @"NSViewAnimationFadeOutEffect";
 - (void) _gs_updateViewsWithValue: (NSNumber*) value
 {
   // Runs in main thread : must not call any NSAnimation method to avoid a deadlock
-  unsigned i,c;
+  unsigned int i, c;
   float v;
+
   v = [value floatValue];
   if (_viewAnimationDesc != nil)
-    for (i=0, c=[_viewAnimationDesc count];i<c;i++)
+    for (i = 0, c = [_viewAnimationDesc count]; i < c; i++)
       [[_viewAnimationDesc objectAtIndex: i] setCurrentProgress: v];
 }
 
