@@ -33,6 +33,7 @@
 #include <Foundation/NSBundle.h>
 #include <Foundation/NSCharacterSet.h>
 #include <Foundation/NSDebug.h>
+#include <Foundation/NSError.h>
 #include <Foundation/NSException.h>
 #include <Foundation/NSFileManager.h>
 #include <Foundation/NSPathUtilities.h>
@@ -53,6 +54,7 @@
 #include "AppKit/NSGraphics.h"
 
 #include "GNUstepGUI/GSTextConverter.h"
+#include "GSGuiPrivate.h"
 
 /* Cache class pointers to avoid the expensive lookup by string. */ 
 static Class dictionaryClass = nil;
@@ -139,9 +141,9 @@ Class converter_bundles(NSString *format, BOOL producer)
       dpath = [dpath stringByAppendingPathComponent: @"Bundles"];
       dpath = [dpath stringByAppendingPathComponent: @"TextConverters"];
       if ([[NSFileManager defaultManager] fileExistsAtPath: dpath])
-	direnum = [[NSFileManager defaultManager] enumeratorAtPath: dpath];
+        direnum = [[NSFileManager defaultManager] enumeratorAtPath: dpath];
       else
-	direnum = nil;
+        direnum = nil;
       while (direnum && (path = [direnum nextObject]))
 	{
 	  Class bclass;
@@ -231,6 +233,16 @@ static Class converter_class(NSString *format, BOOL producer)
   return Nil;
 }
 
+static inline NSError*
+create_error(int code, NSString* desc)
+{
+  return [NSError errorWithDomain: @"NSAttributedString"
+                  code: code 
+                  userInfo: [NSDictionary 
+                                dictionaryWithObjectsAndKeys: desc,
+                                NSLocalizedDescriptionKey, nil]];
+}
+
 @implementation NSAttributedString (AppKit)
 
 + (NSArray *) textFileTypes
@@ -256,6 +268,30 @@ static Class converter_class(NSString *format, BOOL producer)
 		  NSRTFDPboardType, NSHTMLPboardType, nil];
 }
 
++ (NSArray *) textTypes
+{
+  // FIXME
+  return [self textUnfilteredTypes];
+}
+
++ (NSArray *) textUnfilteredTypes
+{
+  return [NSArray arrayWithObjects: @"public.plain-text",
+		  @"public.rtf",
+		  @"com.apple.rtfd",
+		  @"public.html",
+		  /*
+		  @"public.xml",
+		  @"com.apple.webarchive",
+		  @"com.microsoft.word.doc",
+		  @"com.microsoft.word.wordml",
+		  @"org.openxmlformats.wordprocessingml.document",
+		  @"org.oasis-open.opendocument.text",
+		  @"com.apple.traditional-mac-plain-text",
+		  */
+		  nil];
+}
+
 + (NSAttributedString *) attributedStringWithAttachment: 
                                             (NSTextAttachment *)attachment
 {
@@ -359,8 +395,16 @@ static Class converter_class(NSString *format, BOOL producer)
   return [dictionaryClass dictionary];
 }
 
-- (unsigned) lineBreakBeforeIndex: (unsigned)location
-                      withinRange: (NSRange)aRange
+- (NSUInteger) lineBreakByHyphenatingBeforeIndex: (NSUInteger)location
+				     withinRange: (NSRange)aRange
+{
+  // FIXME
+  return [self lineBreakBeforeIndex: location
+                      withinRange: aRange];
+}
+
+- (NSUInteger) lineBreakBeforeIndex: (NSUInteger)location
+			withinRange: (NSRange)aRange
 {
   NSString *str = [self string];
   unsigned length = [str length];
@@ -604,42 +648,6 @@ static Class converter_class(NSString *format, BOOL producer)
     }
 }
 
-- (id) initWithPath: (NSString *)path
- documentAttributes: (NSDictionary **)dict
-{
-  // FIXME: This expects the file to be RTFD
-  NSFileWrapper *fw;
-
-  if (path == nil)
-    {
-      RELEASE (self);
-      return nil;
-    }
-
-  fw = [[NSFileWrapper alloc] initWithPath: path];
-  AUTORELEASE (fw);
-  
-  return [self initWithRTFDFileWrapper: fw  documentAttributes: dict];
-}
-
-- (id) initWithURL: (NSURL *)url 
-documentAttributes: (NSDictionary **)dict
-{
-  NSData *data = [url resourceDataUsingCache: YES];
-  
-  if (data == nil)
-    {
-      RELEASE (self);
-      return nil;
-    }
-
-
-  // FIXME: This expects the URL to point to a HTML page
-  return [self initWithHTML: data
-	       baseURL: [url baseURL]
-	       documentAttributes: dict];
-}
-
 - (id) initWithRTFDFileWrapper: (NSFileWrapper *)wrapper
             documentAttributes: (NSDictionary **)dict
 {
@@ -711,6 +719,21 @@ documentAttributes: (NSDictionary **)dict
 - (id) initWithHTML: (NSData *)data
             baseURL: (NSURL *)base
  documentAttributes: (NSDictionary **)dict
+{
+  NSDictionary *options = nil;
+
+  if (base != nil)
+    options = [NSDictionary dictionaryWithObject: base
+					  forKey: NSBaseURLDocumentOption];
+
+  return [self initWithHTML: data
+		    options: options
+               documentAttributes: dict];
+}
+
+- (id) initWithHTML: (NSData *)data
+            options: (NSDictionary *)options
+ documentAttributes: (NSDictionary **)dict
 {
   if (data == nil)
     {
@@ -722,6 +745,173 @@ documentAttributes: (NSDictionary **)dict
   return self;
 }
 
+- (id) initWithDocFormat: (NSData *)data
+      documentAttributes: (NSDictionary **)dict
+{
+  NSAttributedString *new;
+
+  if (data == nil)
+    {
+      RELEASE (self);
+      return nil;
+    }
+
+  new = [converter_class(@"DOC", NO)
+			parseData: data
+			documentAttributes: dict
+			class: [self class]];
+  // We do not return self but the newly created object
+  RELEASE (self);
+  return RETAIN (new); 
+}
+
+- (id) initWithData: (NSData *)data
+            options: (NSDictionary *)options
+ documentAttributes: (NSDictionary **)dict
+              error: (NSError **)error
+{
+  NSString *type = [options objectForKey: NSDocumentTypeDocumentOption];
+
+  if (data == nil)
+    {
+      *error = create_error(0, NSLocalizedString(@"No data specified for data loading.", 
+                                                 @"Error description"));
+      RELEASE(self);
+      return nil;
+    }
+
+  if (type == nil)
+    {
+      // FIXME: try to determine type
+      *error = create_error(0, NSLocalizedString(@"No type specified for data.", 
+                                                 @"Error description"));
+      RELEASE(self);
+      return nil;
+    }
+
+  if ([type isEqualToString: NSDocFormatTextDocumentType])
+    {
+      return [self initWithDocFormat: data
+                   documentAttributes: dict];
+    }
+  else if ([type isEqualToString: NSHTMLTextDocumentType]
+           || [type isEqualToString: @"public.html"]
+           || [type isEqualToString: @"html"])
+    {
+      return [self initWithHTML: data
+                   options: options
+                   documentAttributes: dict];
+    }
+  else if ([type isEqualToString: NSRTFDTextDocumentType]
+           || [type isEqualToString: @"com.apple.rtfd"]
+           || [type isEqualToString: @"rtfd"])
+    {
+      return [self initWithRTFD: data
+                   documentAttributes: dict];
+    }
+  else if ([type isEqualToString: NSRTFTextDocumentType]
+           || [type isEqualToString: @"public.rtf"]
+           || [type isEqualToString: @"rtf"])
+    {
+      return [self initWithRTF: data
+                   documentAttributes: dict];
+    }
+  else if ([type isEqualToString: NSPlainTextDocumentType]
+           || [type isEqualToString: @"public.plain-text"]
+           || [type isEqualToString: @"text"])
+    {
+      NSStringEncoding encoding = [[options objectForKey: @"CharacterEncoding"] 
+				      intValue];
+      NSDictionary *defaultAttrs = [options objectForKey: @"DefaultAttributes"];
+      NSString *str = [[NSString alloc] initWithData: data 
+                                        encoding: encoding];
+
+      self = [self initWithString: str
+                   attributes: defaultAttrs];
+      RELEASE(str);
+      return self;
+    }
+
+  *error = create_error(0, NSLocalizedString(@"Could not load data.", 
+                                             @"Error description"));
+  RELEASE(self);
+  return nil;
+}
+
+- (id) initWithPath: (NSString *)path
+ documentAttributes: (NSDictionary **)dict
+{
+  BOOL isDir = NO;
+
+  if (path == nil)
+    {
+      RELEASE (self);
+      return nil;
+    }
+
+  if ([[NSFileManager defaultManager]
+          fileExistsAtPath: path isDirectory: &isDir] && isDir)
+    {
+      // FIXME: This expects the file to be RTFD
+      NSFileWrapper *fw;
+
+      fw = [[NSFileWrapper alloc] initWithPath: path];
+      AUTORELEASE (fw);
+  
+      return [self initWithRTFDFileWrapper: fw documentAttributes: dict];
+    }
+  else
+   {
+     return [self initWithURL:  [NSURL fileURLWithPath: path]
+		  documentAttributes: dict];
+   }
+}
+
+- (id) initWithURL: (NSURL *)url 
+documentAttributes: (NSDictionary **)dict
+{
+  NSError *error = nil;
+  NSDictionary *options = [NSDictionary dictionaryWithObject: [url baseURL]
+                                        forKey: NSBaseURLDocumentOption];
+
+  return [self initWithURL: url
+               options: options
+               documentAttributes: dict
+               error: &error];
+}
+
+- (id) initWithURL: (NSURL *)url
+           options: (NSDictionary *)options
+documentAttributes: (NSDictionary **)dict
+             error: (NSError **)error
+{
+  NSData *data = [url resourceDataUsingCache: YES];
+
+  if (data == nil)
+    {
+      *error = create_error(0, NSLocalizedString(@"Could not load data from URL.", 
+                                                 @"Error description"));
+      RELEASE(self);
+      return nil;
+    }
+
+  // Pass on baseURL
+  if (options == nil)
+    options = [NSDictionary dictionaryWithObject: [url baseURL]
+					  forKey: NSBaseURLDocumentOption];
+  else if ([options objectForKey: NSBaseURLDocumentOption] == nil)
+    {
+      options = AUTORELEASE([options mutableCopy]);
+      [(NSMutableDictionary*)options setObject: [url baseURL]
+                             forKey: NSBaseURLDocumentOption];
+    }
+
+  return [self initWithData: data
+               options: options
+               documentAttributes: dict
+               error: error];
+}
+
 - (NSData *) RTFFromRange: (NSRange)range
        documentAttributes: (NSDictionary *)dict
 {
@@ -749,43 +939,6 @@ documentAttributes: (NSDictionary **)dict
 			 documentAttributes: dict];
 }
 
-- (id) initWithDocFormat: (NSData *)data
-      documentAttributes: (NSDictionary **)dict
-{
-  // FIXME
-  RELEASE(self);
-  return nil;
-}
-
-- (id) initWithHTML: (NSData *)data
-            options: (NSDictionary *)options
- documentAttributes: (NSDictionary **)dict
-{
-  // FIXME
-  RELEASE(self);
-  return nil;
-}
-
-- (id) initWithData: (NSData *)data
-            options: (NSDictionary *)options
- documentAttributes: (NSDictionary **)dict
-              error: (NSError **)error
-{
-  // FIXME
-  RELEASE(self);
-  return nil;
-}
-
-- (id) initWithURL: (NSURL *)url
-           options: (NSDictionary *)options
-documentAttributes: (NSDictionary **)dict
-             error: (NSError **)error
-{
-  // FIXME
-  RELEASE(self);
-  return nil;
-}
-
 - (NSData *) docFormatFromRange: (NSRange)range
              documentAttributes: (NSDictionary *)dict
 {
@@ -797,7 +950,46 @@ documentAttributes: (NSDictionary **)dict
         documentAttributes: (NSDictionary *)dict
                      error: (NSError **)error
 {
-  // FIXME
+  NSString *type = [dict objectForKey: NSDocumentTypeDocumentOption];
+
+  if (type == nil)
+    {
+      *error = create_error(0, NSLocalizedString(@"No type specified for data.", 
+                                                 @"Error description"));
+      return nil;
+    }
+
+  if ([type isEqualToString: NSDocFormatTextDocumentType])
+    {
+      return [self docFormatFromRange: range
+                   documentAttributes: dict];
+    }
+  else if ([type isEqualToString: NSHTMLTextDocumentType])
+    {
+      // FIXME
+    }
+  else if ([type isEqualToString: NSRTFDTextDocumentType])
+    {
+      return [self RTFDFromRange: range
+                   documentAttributes: dict];
+    }
+  else if ([type isEqualToString: NSRTFTextDocumentType])
+    {
+      return [self RTFFromRange: range
+                   documentAttributes: dict];
+    }
+  else if ([type isEqualToString: NSPlainTextDocumentType])
+    {
+      NSStringEncoding encoding = [[dict objectForKey: @"CharacterEncoding"] 
+                                      intValue];
+      
+      if (!encoding)
+        encoding = [NSString defaultCStringEncoding];
+      return [[self string] dataUsingEncoding: encoding];
+    }
+
+  *error = create_error(0, NSLocalizedString(@"Could not create data for type.", 
+                                             @"Error description"));
   return nil;
 }
 
@@ -805,15 +997,24 @@ documentAttributes: (NSDictionary **)dict
                       documentAttributes: (NSDictionary *)dict
                                    error: (NSError **)error
 {
-  // FIXME
-  return nil;
-}
+  NSFileWrapper *wrapper;
+  NSData *data;
 
-- (unsigned) lineBreakByHyphenatingBeforeIndex: (unsigned)location
-                                   withinRange: (NSRange)aRange
-{
-  // FIXME
-  return NSNotFound;
+  // FIXME: This wont work for directory bundles.
+  data = [self dataFromRange: range
+	  documentAttributes: dict
+		       error: error];
+  if (data != nil)
+    {
+      wrapper = [[NSFileWrapper alloc] initRegularFileWithContents: data];
+      return AUTORELEASE(wrapper);
+    }
+
+  if (*error == nil)
+      *error = create_error(0, NSLocalizedString(@"Could not create data for type.",
+                                                 @"Error description"));
+
+  return nil;
 }
 
 - (NSRange) itemNumberInTextList: (NSTextList *)list
@@ -1302,7 +1503,7 @@ static NSMutableDictionary *cachedCSets = nil;
       NSTextAttachment *attachment;
 
       if (!eRange.length)
-	break;
+        break;
 
       attachment = [self attribute: NSAttachmentAttributeName
 			 atIndex: eRange.location
@@ -1310,10 +1511,10 @@ static NSMutableDictionary *cachedCSets = nil;
 
       if (attachment == nil)
         {
-	  [self deleteCharactersInRange: NSMakeRange (eRange.location, 1)];
-	  eRange.length--;
-	  end--;
-	}
+          [self deleteCharactersInRange: NSMakeRange (eRange.location, 1)];
+          eRange.length--;
+          end--;
+        }
 
       location = NSMaxRange (eRange);
     }
@@ -1351,7 +1552,7 @@ static NSMutableDictionary *cachedCSets = nil;
 }
 
 - (BOOL) readFromURL: (NSURL *)url
-	     options: (NSDictionary *)options
+             options: (NSDictionary *)options
   documentAttributes: (NSDictionary**)documentAttributes
 {
   return [self readFromURL: url
@@ -1365,79 +1566,19 @@ static NSMutableDictionary *cachedCSets = nil;
   documentAttributes: (NSDictionary **)documentAttributes
                error: (NSError **)error
 {
-  NSString *extension;
-  NSString *type;
+  NSAttributedString *attr;
 
-  if (![url isFileURL])
-    return NO;
-
-  extension = [[url path] pathExtension];
-  type = [[NSDocumentController sharedDocumentController] 
-	     typeFromFileExtension: extension];
-  if (type == nil)
-    return NO;
-  
-  if ([type isEqualToString: @"html"])
+  attr = [[NSAttributedString alloc] 
+                 initWithURL: url
+		     options: options
+	  documentAttributes: documentAttributes
+		       error: error];
+  if (attr != nil)
     {
-      NSData *data = [url resourceDataUsingCache: YES];
-      NSURL *baseURL = [options objectForKey: @"BaseURL"];
-      NSAttributedString *attr;
-      
-      attr = [[NSAttributedString alloc] 
-		 initWithHTML: data
-		 baseURL: baseURL
-		 documentAttributes: documentAttributes];
       [self setAttributedString: attr];
       RELEASE(attr);
-
-      return YES;
-    }
-  else if ([type isEqualToString: @"rtfd"])
-    {
-      NSData *data = [url resourceDataUsingCache: YES];
-      NSAttributedString *attr;
-      
-      attr = [[NSAttributedString alloc] 
-		 initWithRTFD: data
-		 documentAttributes: documentAttributes];
-      [self setAttributedString: attr];
-      RELEASE(attr);
-
-      return YES;
-    }
-  else if ([type isEqualToString: @"rtf"])
-    {
-      NSData *data = [url resourceDataUsingCache: YES];
-      NSAttributedString *attr;
-      
-      attr = [[NSAttributedString alloc] 
-		 initWithRTF: data
-		 documentAttributes: documentAttributes];
-      [self setAttributedString: attr];
-      RELEASE(attr);
-
-      return YES;
-    }
-  else if ([type isEqualToString: @"text"])
-    {
-      NSData *data = [url resourceDataUsingCache: YES];
-      NSStringEncoding encoding = [[options objectForKey: @"CharacterEncoding"] 
-				      intValue];
-      NSDictionary *defaultAttrs = [options objectForKey: @"DefaultAttributes"];
-      NSString *str = [[NSString alloc] initWithData: data 
-					encoding: encoding];
-      NSAttributedString *attr;
-
-      attr = [[NSAttributedString alloc] 
-		 initWithString: str
-		    attributes: defaultAttrs];
-      RELEASE(str);
-      [self setAttributedString: attr];
-      RELEASE(attr);
-
       return YES; 
     }
-  // FIXME This should also support all converter bundles
 
   return NO;
 }
@@ -1457,7 +1598,20 @@ static NSMutableDictionary *cachedCSets = nil;
    documentAttributes: (NSDictionary **)documentAttributes
                 error: (NSError **)error
 {
-  // FIXME
+  NSAttributedString *attr;
+
+  attr = [[NSAttributedString alloc] 
+             initWithData: data
+             options: options
+             documentAttributes: documentAttributes
+             error: error];
+  if (attr)
+    {
+      [self setAttributedString: attr];
+      RELEASE(attr);
+      return YES;
+    }
+
   return NO;
 }
 
