@@ -141,7 +141,7 @@ typedef struct _GSButtonCellFlags
 
 - (id) init
 {
-  [self initTextCell: @"Button"];
+  self = [self initTextCell: @"Button"];
 
   return self;
 }
@@ -199,7 +199,7 @@ typedef struct _GSButtonCellFlags
  */
 - (NSString*) title
 {
-  return [self stringValue];
+  return [super stringValue];
 }
 
 /** <p>Returns the NSButtonCell's alternate title ( used when highlighted ).
@@ -314,8 +314,8 @@ typedef struct _GSButtonCellFlags
 
   [super setFont: fontObject];
 
-  if ((_keyEquivalentFont != nil) && (fontObject != nil) && 
-      ((size = [fontObject pointSize]) != [_keyEquivalentFont pointSize]))
+  if ((_keyEquivalentFont != nil) && (fontObject != nil)
+    && ((size = [fontObject pointSize]) != [_keyEquivalentFont pointSize]))
     {
       [self setKeyEquivalentFont: [_keyEquivalentFont fontName] 
             size: size];       
@@ -327,7 +327,7 @@ typedef struct _GSButtonCellFlags
  */
 - (void) setTitle: (NSString*)aString
 {
-  [self setStringValue: aString];
+  [super setStringValue: aString];
 }
 
 /**<p>Sets the NSButtonCell's alternate title ( used when highlighted ) 
@@ -364,21 +364,17 @@ typedef struct _GSButtonCellFlags
 - (void)setAttributedAlternateTitle:(NSAttributedString *)aString
 {
   // TODO
-  NSString *alternateTitle;
-  
-  alternateTitle = AUTORELEASE ([[aString string] copy]);
-  
-  [self setAlternateTitle: alternateTitle];
+  [self setAlternateTitle: [aString string]];
 }
 
 - (NSAttributedString *)attributedTitle
 {
-  return [self attributedStringValue];
+  return [super attributedStringValue];
 }
 
 - (void)setAttributedTitle:(NSAttributedString *)aString
 {
-  [self setAttributedStringValue: aString];
+  [super setAttributedStringValue: aString];
 }
 
 - (void)setTitleWithMnemonic:(NSString *)aString
@@ -809,7 +805,28 @@ typedef struct _GSButtonCellFlags
   else // NSMixedState
     {        
       return [NSNumber numberWithInt: -1];
-    }
+   }
+}
+
+- (void) setStringValue: (NSString *)aString
+{
+  [self setState: ([aString length] != 0)];
+}
+
+- (NSString *) stringValue
+{
+  return _cell.state ? @"1" : @"";
+}
+
+- (void) setAttributedStringValue: (NSAttributedString *)attrString
+{
+  [self setState: ([attrString length] != 0)];
+}
+
+- (NSAttributedString *) attributedStringValue
+{
+  return AUTORELEASE([[NSAttributedString alloc] 
+                         initWithString: [self stringValue]]); 
 }
 
 /*
@@ -923,6 +940,21 @@ typedef struct _GSButtonCellFlags
 - (void) _drawBorderAndBackgroundWithFrame: (NSRect)cellFrame 
                                     inView: (NSView*)controlView
 {
+  /* The background color is used for borderless cells (the MacOS-X
+   * documentation of the NSButtonCell -backgroundColor method says
+   * it's only used for borderless cells).
+   */
+  if (!_cell.is_bordered)
+    {
+      NSColor	*c = [self backgroundColor];
+
+      if (c != nil)
+	{
+	  [c set];
+	  NSRectFill(cellFrame);
+	}
+    }
+
   // Draw gradient
   if (!_cell.is_highlighted)
     {
@@ -1296,8 +1328,9 @@ typedef struct _GSButtonCellFlags
           buttonState = GSThemeSelectedState;
         }
 
-      borderSize = [[GSTheme theme] buttonBorderForStyle: _bezel_style 
-                                                   state: buttonState];
+      borderSize = [[GSTheme theme] buttonBorderForCell: self
+						  style: _bezel_style 
+                                                  state: buttonState];
     }
   else
     borderSize = NSZeroSize;
@@ -1359,15 +1392,16 @@ typedef struct _GSButtonCellFlags
           buttonState = GSThemeSelectedState;
         }
 
-      borderSize = [[GSTheme theme] buttonBorderForStyle: _bezel_style 
-                                                   state: buttonState];
+      borderSize = [[GSTheme theme] buttonBorderForCell: self
+						  style: _bezel_style 
+                                                  state: buttonState];
       interiorFrame = NSInsetRect(theRect, borderSize.width, borderSize.height);
 
       /* Pushed in buttons contents are displaced to the bottom right 1px.  */
       if (mask & NSPushInCellMask)
         {
-          interiorFrame
-            = NSOffsetRect(interiorFrame, 1.0, [_control_view isFlipped] ? 1.0 : -1.0);
+          interiorFrame = NSOffsetRect(interiorFrame, 1.0,
+	    [_control_view isFlipped] ? 1.0 : -1.0);
         }
       return interiorFrame;
     }
@@ -1747,6 +1781,13 @@ typedef struct _GSButtonCellFlags
           _shows_border_only_while_mouse_inside = tmp;
         }
     }
+
+  // Hack to correct a Gorm problem, there "\n" is used instead of "\r".
+  if ([_keyEquivalent isEqualToString: @"\n" ])
+    {
+      [self setKeyEquivalent: @"\r"];
+    }
+
   return self;
 }
 
