@@ -321,19 +321,33 @@ static NSColor *dtxtCol;
           _cell.contents_is_attributed_string = NO;
           _cell.has_valid_object_value = YES;
         }
-      if ([object isKindOfClass: [NSAttributedString class]] == YES)
+      else if ([object isKindOfClass: [NSAttributedString class]] == YES)
         {
           newContents = object;
           _cell.contents_is_attributed_string = YES;
           _cell.has_valid_object_value = YES;
         }
+      else if([_object_value respondsToSelector: @selector(attributedStringValue)])
+        {
+          newContents = [_object_value attributedStringValue];
+          _cell.contents_is_attributed_string = YES;
+          _cell.has_valid_object_value = YES;
+        }
+      else if([_object_value respondsToSelector: @selector(stringValue)])
+        {
+          // If the thing that was assigned is not a string, but 
+          // responds to stringValue then get that.
+          newContents = [_object_value stringValue];
+          _cell.contents_is_attributed_string = NO;
+          _cell.has_valid_object_value = YES;
+        }
       else
         {
           newContents = [_object_value description];
           _cell.contents_is_attributed_string = NO;
           _cell.has_valid_object_value = YES;
         }
-  }
+    }
   else
     {
       newContents = [_formatter stringForObjectValue: _object_value];
@@ -402,11 +416,9 @@ static NSColor *dtxtCol;
  */
 - (void) setStringValue: (NSString*)aString
 {
-  NSString *string = aString;
-
   /* We warn about nil for compatibiliy with MacOS X, which refuses
      nil.  */
-  if (string == nil)
+  if (aString == nil)
     {
       NSDebugMLLog (@"MacOSXCompatibility", 
                     @"Attempt to use nil as string value");
@@ -416,28 +428,37 @@ static NSColor *dtxtCol;
     {
       [self setType: NSTextCellType];
     }
-  _cell.contents_is_attributed_string = NO;
 
   if (_formatter == nil)
     {
-      ASSIGN (_contents, string);
-      ASSIGN (_object_value, string);
-      _cell.has_valid_object_value = YES;
+      /* if we are a string, we do an optimization and set the value here instead of using setObjectValue
+       * also, we check for nil, since isKindOfClass fails for nil objects */
+      if([aString isKindOfClass: [NSString class]] == YES || aString == nil)
+        {
+          ASSIGN (_contents, aString);
+          ASSIGN (_object_value, aString);
+          _cell.has_valid_object_value = YES;
+        }
+      else
+        {
+          [self setObjectValue: aString];
+        }
     }
   else
     {
       id newObjectValue;
       
       if ([_formatter getObjectValue: &newObjectValue 
-                      forString: string 
+                      forString: aString 
                       errorDescription: NULL])
         {
           [self setObjectValue: newObjectValue];
         }
       else
         {
+          ASSIGN (_contents, aString);
+          _cell.contents_is_attributed_string = NO;
           _cell.has_valid_object_value = NO;
-          ASSIGN (_contents, string);
         }
     }
 }
@@ -1367,6 +1388,31 @@ static NSColor *dtxtCol;
     [self performClickWithFrame: [cv bounds] inView: cv];
 }
 
+/*
+ * Helper method used to send actions. Sender normally is [self controlView].
+ */
+- (BOOL) _sendActionFrom: (id)sender
+{
+  SEL action = [self action];
+
+  if ([sender respondsTo: @selector(sendAction:to:)])
+    {
+      return [sender sendAction: action to: [self target]];
+    }
+  else
+    {
+      if (sender == nil)
+        sender = self;
+
+      if (action)
+        {
+          return [NSApp sendAction: action to: [self target] from: sender];
+        }
+    }
+
+  return NO;
+}
+
 /**
  * Simulates a single click in the cell.
  * The display of the cell with this event
@@ -1375,8 +1421,6 @@ static NSColor *dtxtCol;
  */
 - (void) performClickWithFrame: (NSRect)cellFrame inView: (NSView *)controlView
 {
-  SEL action = [self action];
-
   if (_cell.is_disabled == YES)
     {
       return;
@@ -1399,34 +1443,9 @@ static NSColor *dtxtCol;
       [self highlight: NO withFrame: cellFrame inView: controlView];
       [cvWin flushWindow];
       [controlView unlockFocus];
+    }
 
-      NS_DURING
-        {
-          [(NSControl*)controlView sendAction: action to: [self target]];
-        }
-      NS_HANDLER
-        {
-          [localException raise];
-        }
-      NS_ENDHANDLER
-    }
-  else  // We have no control view.  The best we can do is the following. 
-    {
-      if (action)
-        {
-          NS_DURING
-            {
-              [[NSApplication sharedApplication] sendAction: action
-                                                 to: [self target]
-                                                 from: self];
-            }
-          NS_HANDLER
-            {
-              [localException raise];
-            }
-          NS_ENDHANDLER
-        }
-    }
+  [self _sendActionFrom: controlView];
 }
 
 /*
@@ -1556,8 +1575,6 @@ static NSColor *dtxtCol;
   NSPoint point = [controlView convertPoint: location fromView: nil];
   float delay;
   float interval;
-  id target = [self target];
-  SEL action = [self action];
   NSPoint last_point = point;
   BOOL done;
   BOOL mouseWentUp;
@@ -1575,7 +1592,7 @@ static NSColor *dtxtCol;
 
   if ((_action_mask & NSLeftMouseDownMask) 
       && [theEvent type] == NSLeftMouseDown)
-    [(NSControl*)controlView sendAction: action to: target];
+    [self _sendActionFrom: controlView];
 
   if (_action_mask & NSPeriodicMask)
     {
@@ -1591,7 +1608,7 @@ static NSColor *dtxtCol;
     theEvent = [NSApp currentEvent];
   else
     theEvent = [theApp nextEventMatchingMask: event_mask
-                       untilDate: nil
+                       untilDate: [NSDate distantFuture]
                        inMode: NSEventTrackingRunLoopMode
                        dequeue: YES];
                  
@@ -1667,12 +1684,12 @@ static NSColor *dtxtCol;
                           && (_action_mask & NSLeftMouseDraggedMask))
                           || ((eventType == NSPeriodic)
                           && (_action_mask & NSPeriodicMask))))
-            [(NSControl*)controlView sendAction: action to: target];
+            [self _sendActionFrom: controlView];
         }
       
       if (!done)
         theEvent = [theApp nextEventMatchingMask: event_mask
-                                       untilDate: nil
+                                       untilDate: [NSDate distantFuture]
                                           inMode: NSEventTrackingRunLoopMode
                                          dequeue: YES];
     }
@@ -1690,7 +1707,7 @@ static NSColor *dtxtCol;
     {
       [self setNextState];
       if ((_action_mask & NSLeftMouseUpMask))
-        [(NSControl*)controlView sendAction: action to: target];
+        [self _sendActionFrom: controlView];
     }
 
   // Return YES only if the mouse went up within the cell
