@@ -68,6 +68,7 @@ static NSNotificationCenter *nc = nil;
 - (NSDrawer *) drawer;
 
 // handle notifications...
+- (void) handleWindowDidBecomeKey: (NSNotification *)notification;
 - (void) handleWindowClose: (NSNotification *)notification;
 - (void) handleWindowMiniaturize: (NSNotification *)notification;
 - (void) handleWindowMove: (NSNotification *)notification;
@@ -145,39 +146,42 @@ static NSNotificationCenter *nc = nil;
   NSRectEdge edge = [_drawer preferredEdge];
   int state = [_drawer state];
   BOOL opened = (state == NSDrawerOpenState);
-  NSSize size = [_drawer maxContentSize];
+  NSSize size = [_parentWindow frame].size; // [_drawer maxContentSize];
+  newFrame.size.width = [_drawer minContentSize].width;
   
   if (edge == NSMinXEdge) // left
     {
       newFrame.size.height -= total;
-      newFrame.origin.y += [_drawer trailingOffset];
+      newFrame.origin.y += [_drawer trailingOffset] - (total/2);
       newFrame.origin.x -= (opened)?size.width:0;
     }
   else if (edge == NSMinYEdge) // bottom
     {
       newFrame.size.width -= total;
-      newFrame.origin.x += [_drawer leadingOffset];
+      newFrame.origin.x += [_drawer leadingOffset] + (total/2);
       newFrame.origin.y -= (opened)?size.height:0;
     }
   else if (edge == NSMaxXEdge) // right
     {
       newFrame.size.height -= total;
-      newFrame.origin.y += [_drawer trailingOffset];
+      newFrame.origin.y += [_drawer trailingOffset] - (total/2);
       newFrame.origin.x += (opened)?size.width:0;
     }
   else if (edge == NSMaxYEdge) // top
     {
       newFrame.size.width -= total;
-      newFrame.origin.x += [_drawer leadingOffset];
+      newFrame.origin.x += [_drawer leadingOffset] + (total/2);
       newFrame.origin.y += (opened)?size.height:0;
     }
 
   return newFrame;
 }
+
+
  
 - (BOOL) canBecomeKeyWindow
 {
-  return NO;
+  return YES;
 }
 
 - (BOOL) canBecomeMainWindow
@@ -185,6 +189,7 @@ static NSNotificationCenter *nc = nil;
   return NO;
 }
 
+/*
 - (void) orderFront: (id)sender
 {
   NSPoint holdOrigin = [self frame].origin;
@@ -195,14 +200,15 @@ static NSNotificationCenter *nc = nil;
   tempFrame.origin = newOrigin;
   [self setFrame: tempFrame display: NO];
   [super orderFront: sender];
-  [_parentWindow orderWindow: NSWindowAbove relativeTo: [self windowNumber]];  
+  // [_parentWindow orderWindow: NSWindowAbove relativeTo: [self windowNumber]];  
   tempFrame.origin = holdOrigin;
-  [self setFrame: tempFrame display: YES];
+  // [self setFrame: tempFrame display: YES];
 }
+*/
 
 - (void) startTimer
 {
-  NSTimeInterval time = 3.0;
+  NSTimeInterval time = 1.0;
   _timer = [NSTimer scheduledTimerWithTimeInterval: time
 		    target: self
 		    selector: @selector(_timedWindowReset)
@@ -235,16 +241,19 @@ static NSNotificationCenter *nc = nil;
 
 - (void) openOnEdge
 {
-  [self orderFront: self];
+  // NSRect frame = [self frameFromParentWindowFrame];
+
+  // [self setFrame: frame display: YES];
   [self slide];
-  // [self startTimer];
+  [self orderFront: self];
+  [self startTimer];
 }
 
 - (void) closeOnEdge
 {
   NSRect frame = [self frameFromParentWindowFrame];
 
-  //  [self stopTimer];
+  [self stopTimer];
   [self slide];
   [self setFrame: frame display: YES];
   [self orderOut: self];
@@ -260,57 +269,31 @@ static NSNotificationCenter *nc = nil;
 - (void) slide
 {
   NSRect frame = [self frame];
-  float i;
   NSRectEdge edge = [_drawer preferredEdge];
-  NSSize size = [_drawer maxContentSize];
-  float factor = 1.0;
-
-  // if it's open, then slide it closed.
-  if ([_drawer state] == NSDrawerClosingState)
-    {
-      factor = -factor;
-    }
-  else if ([_drawer state] == NSDrawerOpeningState)
-    {
-      factor = 1.0;
-    }
+  NSSize size = [_parentWindow frame].size;
 
+  [super setParentWindow: nil];
   if (edge == NSMinXEdge) // left
     {
-      // slide left...
-      for (i = 0; i < size.width; i++)
-	{
-	  frame.origin.x -= factor;
-	  [self setFrame: frame display: YES];
-	}
+      frame.origin.x -= size.width;
+      [self setFrame: frame display: YES];
     }
   else if (edge == NSMinYEdge) // bottom
     {
-      // slide down...
-      for (i = 0; i < size.height; i++)
-	{
-	  frame.origin.y -= factor;
-	  [self setFrame: frame display: YES];
-	}
+      frame.origin.y -= size.height;
+      [self setFrame: frame display: YES];      
     }
   else if (edge == NSMaxXEdge) // right
     {
-      // slide right...
-      for (i = 0; i < size.width; i++)
-	{
-	  frame.origin.x += factor;
-	  [self setFrame: frame display: YES];
-	}
+      frame.origin.x += size.width;
+      [self setFrame: frame display: YES];
     }
   else if (edge == NSMaxYEdge) // top
     {
-      // slide up...
-      for (i = 0; i < size.height; i++)
-	{
-	  frame.origin.y += factor;
-	  [self setFrame: frame display: YES];
-	}
+      frame.origin.y += size.height;
+      [self setFrame: frame display: YES];
     }
+  [super setParentWindow: _parentWindow];
 }
 
 - (void) _resetWindowPosition
@@ -321,9 +304,9 @@ static NSNotificationCenter *nc = nil;
 
 - (void) _timedWindowReset
 {
-  NSRect frame = [_parentWindow frame];
+  // NSRect frame = [_parentWindow frame];
   [self _resetWindowPosition];
-  [_parentWindow setFrame: frame display: YES];
+  // [_parentWindow setFrame: frame display: YES];
 }
 
 - (void) handleWindowClose: (NSNotification *)notification
@@ -343,10 +326,20 @@ static NSNotificationCenter *nc = nil;
   [self _resetWindowPosition];
 }
 
+- (void) handleWindowDidBecomeKey: (NSNotification *)notification
+{
+  if([_drawer state] == NSDrawerOpenState)
+    {
+      [self _resetWindowPosition];
+      [self orderFront: self];
+    }
+}
+
 - (void) setParentWindow: (NSWindow *)window
 {
   if (_parentWindow != window)
     {
+      [super setParentWindow: window];
       ASSIGN(_parentWindow, window);
       [nc removeObserver: self];
 
@@ -373,7 +366,12 @@ static NSNotificationCenter *nc = nil;
 	  [nc addObserver: self
 	      selector: @selector(handleWindowMove:)
 	      name: NSWindowDidResizeNotification
-	      object: _parentWindow];	  
+	      object: _parentWindow];
+	  
+	  [nc addObserver: self
+	      selector: @selector(handleWindowDidBecomeKey:)
+	      name: NSWindowDidBecomeKeyNotification
+	      object: _parentWindow];
 	}
     }
 }
