@@ -52,6 +52,7 @@
 #include "AppKit/NSWindow.h"
 #include "AppKit/PSOperators.h"
 #include "GNUstepGUI/GSDisplayServer.h"
+#include "GSThemePrivate.h"
 
 
 /* Helpers.  Would be nicer to use the C99 fmin/fmax functions, but that
@@ -71,7 +72,6 @@ static double gs_max(double x, double y)
     return x;
 }
 
-
 BOOL NSImageForceCaching = NO; /* use on missmatch */
 
 @implementation NSBundle (NSImageAdditions)
@@ -471,8 +471,8 @@ repd_for_rep(NSArray *_reps, NSImageRep *rep)
   RELEASE(_reps);
   /* Make sure we don't remove name from the nameDict if we are just a copy
      of the named image, not the original image */
-  if (_name && self == [nameDict objectForKey: _name]) 
-    [nameDict removeObjectForKey: _name];
+  if (_name && self == [[nameDict objectForKey: _name] _resource]) 
+    [[nameDict objectForKey: _name] _setResource: nil];
   RELEASE(_name);
   TEST_RELEASE(_fileName);
   RELEASE(_color);
@@ -510,27 +510,28 @@ repd_for_rep(NSArray *_reps, NSImageRep *rep)
 
 - (BOOL) setName: (NSString *)aName
 {
-  BOOL retained = NO;
+  GSThemeProxy	*proxy = nil;
   
-  if (!aName || [nameDict objectForKey: aName])
-    return NO;
-
-  if (_name && self == [nameDict objectForKey: _name])
+  if (!aName || [[nameDict objectForKey: aName] _resource] != nil)
     {
-      /* We retain self in case removing from the dictionary releases
-         us */
-      RETAIN (self);
-      retained = YES;
-      [nameDict removeObjectForKey: _name];
+      return NO;
+    }
+  if (_name && self == [(proxy = [nameDict objectForKey: _name]) _resource])
+    {
+      /* We retain self in case removing from the dictionary releases us */
+      IF_NO_GC([[self retain] autorelease]);
+      [proxy _setResource: nil];
     }
   
   ASSIGN(_name, aName);
   
-  [nameDict setObject: self forKey: _name];
-  if (retained)
+  if ((proxy = [nameDict objectForKey: _name]) == nil)
     {
-      RELEASE (self);
+      proxy = [GSThemeProxy alloc];
+      [nameDict setObject: proxy forKey: _name];
+      [proxy release]; 
     }
+  [proxy _setResource: self];
   
   return YES;
 }
@@ -752,7 +753,7 @@ repd_for_rep(NSArray *_reps, NSImageRep *rep)
       NSCachedImageRep *cache = nil;
 
       if (rep == nil)
-        return;
+        NS_VOIDRETURN;
 
       if (([GSCurrentContext() isDrawingToScreen] == YES)
           && _cacheMode != NSImageCacheNever)
@@ -823,7 +824,7 @@ repd_for_rep(NSArray *_reps, NSImageRep *rep)
       NSCachedImageRep *cache = nil;
 
       if (rep == nil)
-        return;
+        NS_VOIDRETURN;
 
       if (([GSCurrentContext() isDrawingToScreen] == YES)
           && _cacheMode != NSImageCacheNever)
@@ -900,7 +901,7 @@ repd_for_rep(NSArray *_reps, NSImageRep *rep)
       NSCachedImageRep *cache = nil;
 
       if (rep == nil)
-        return;
+        NS_VOIDRETURN;
 
       if (([GSCurrentContext() isDrawingToScreen] == YES)
           && _cacheMode != NSImageCacheNever)
