@@ -56,6 +56,7 @@
 #include "AppKit/NSScreen.h"
 #include "AppKit/NSAttributedString.h"
 
+#include "GSGuiPrivate.h"
 
 /*
   Drawing related:
@@ -146,8 +147,8 @@ static BOOL menuBarVisible = YES;
 - (NSString*) _locationKey
 {
   NSInterfaceStyle style = NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", nil);
-  if (style == NSMacintoshInterfaceStyle || 
-      style == NSWindows95InterfaceStyle)
+  if (style == NSMacintoshInterfaceStyle
+    || style == NSWindows95InterfaceStyle)
     {
       return nil;
     }
@@ -208,132 +209,202 @@ static BOOL menuBarVisible = YES;
 
 - (void) _organizeMenu
 {
-  int		i;
+  NSString *infoString = _(@"Info");
+  NSString *servicesString = _(@"Services");
+  int i;
 
   if ([self isEqual: [NSApp mainMenu]] == YES)
     {
-      NSString		*appTitle;
-      NSMenu		*appMenu;
-      id <NSMenuItem>	appItem;
+      NSString *appTitle;
+      NSMenu *appMenu;
+      id <NSMenuItem> appItem;
 
       appTitle = [[NSProcessInfo processInfo] processName];
       appItem = [self itemWithTitle: appTitle];
       appMenu = [appItem submenu];
 
       if (_menu.horizontal == YES)
-	{
-	  NSMutableArray	*itemsToMove;
-	  NSImage		*ti;
-	  float			bar;
+        {
+          NSMutableArray *itemsToMove;
+          NSImage *ti;
+          float bar;
 
-	  ti = [[NSApp applicationIconImage] copy];
-	  if (ti == nil)
-	    {
-	      ti = [[NSImage imageNamed: @"GNUstep"] copy];
-	    }
-	  [ti setScalesWhenResized: YES];
-	  bar = [NSMenuView menuBarHeight] - 4;
-	  [ti setSize: NSMakeSize(bar, bar)];
+          ti = [[NSApp applicationIconImage] copy];
+          if (ti == nil)
+            {
+              ti = [[NSImage imageNamed: @"GNUstep"] copy];
+            }
+          [ti setScalesWhenResized: YES];
+          bar = [NSMenuView menuBarHeight] - 4;
+          [ti setSize: NSMakeSize(bar, bar)];
+          
+          itemsToMove = [NSMutableArray new];
+          
+          if (appMenu == nil)
+            {
+              [self insertItemWithTitle: appTitle
+                    action: NULL
+                    keyEquivalent: @"" 
+                    atIndex: 0];
+              appItem = [self itemAtIndex: 0];
+              appMenu = [NSMenu new];
+              [self setSubmenu: appMenu forItem: appItem];
+              RELEASE(appMenu);
+            }
+          else
+            {
+              int index = [self indexOfItem: appItem];
+              
+              if (index != 0)
+                {
+                  RETAIN (appItem);
+                  [self removeItemAtIndex: index];
+                  [self insertItem: appItem atIndex: 0];
+                  RELEASE (appItem);
+                }
+            }
+          [appItem setImage: ti];
+          RELEASE(ti);
+          
+          // Collect all simple items plus "Info" and "Services"
+          for (i = 1; i < [_items count]; i++)
+            {
+              NSMenuItem *anItem = [_items objectAtIndex: i];
+              NSString *title = [anItem title];
+              NSMenu *submenu = [anItem submenu];
 
-	  itemsToMove = [NSMutableArray new];
+              if (submenu == nil)
+                {
+                  [itemsToMove addObject: anItem];
+                }
+              else
+                {
+                  // The menu may not be localized, so we have to 
+                  // check both the English and the local version.
+                  if ([title isEqual: @"Info"] ||
+                      [title isEqual: @"Services"] ||
+                      [title isEqual: infoString] ||
+                      [title isEqual: servicesString])
+                    {
+                      [itemsToMove addObject: anItem];
+                    }
+                }
+            }
+          
+          for (i = 0; i < [itemsToMove count]; i++)
+            {
+              NSMenuItem *anItem = [itemsToMove objectAtIndex: i];
 
-	  if (appMenu == nil)
-	    {
-	      [self insertItemWithTitle: appTitle
-				 action: NULL
-			  keyEquivalent: @"" 
-				atIndex: 0];
-	      appItem = [self itemAtIndex: 0];
-	      appMenu = [NSMenu new];
-	      [self setSubmenu: appMenu forItem: appItem];
-	      RELEASE(appMenu);
-	    }
-	  else
-	    {
-	      int index = [self indexOfItem: appItem];
-
-	      if (index != 0)
-		{
-		  RETAIN (appItem);
-		  [self removeItemAtIndex: index];
-		  [self insertItem: appItem atIndex: 0];
-		  RELEASE (appItem);
-		}
-	    }
-	  [appItem setImage: ti];
-	  RELEASE(ti);
-
-	  for (i = 1; i < [_items count]; i++)
-	    {
-	      NSMenuItem	*anItem = [_items objectAtIndex: i];
-	      NSString	*title = [anItem title];
-	      NSMenu	*submenu = [anItem submenu];
-
-	      if (submenu == nil)
-		{
-		  [itemsToMove addObject: anItem];
-		}
-	      else
-		{
-		  if ([title isEqual: NSLocalizedString (@"Info", @"Info")])
-		    {
-		      [itemsToMove addObject: anItem];
-		    }
-		  [[submenu menuRepresentation] update];
-		}
-	    }
-
-	  for (i = 0; i < [itemsToMove count]; i++)
-	    {
-	      [self removeItem: [itemsToMove objectAtIndex: i]];
-	      [appMenu addItem: [itemsToMove objectAtIndex: i]];
-	    }
-	  [[appMenu menuRepresentation] update];
-
-	  RELEASE(itemsToMove);
-	}      
+              [self removeItem: anItem];
+              [appMenu addItem: anItem];
+            }
+          
+          RELEASE(itemsToMove);
+        }      
       else 
-	{
-	  [appItem setImage: nil];
-	  if (appMenu != nil)
-	    {
-	      NSArray	*array = [NSArray arrayWithArray: [appMenu itemArray]];
+        {
+          [appItem setImage: nil];
+          if (appMenu != nil)
+            {
+              NSArray	*array = [NSArray arrayWithArray: [appMenu itemArray]];
+              /* 
+               * Everything above the Serives menu goes into the info submenu,
+               * the rest into the main menu.
+               */
+              int k = [appMenu indexOfItemWithTitle: servicesString];
 
-	      for (i = 0; i < [array count]; i++)
-	        {
-		  NSMenuItem	*anItem = [array objectAtIndex: i];
-		  NSMenu	*submenu = [anItem submenu];
+              // The menu may not be localized, so we have to 
+              // check both the English and the local version.
+              if (k == -1)
+                k = [appMenu indexOfItemWithTitle: @"Services"];
 
-		  [appMenu removeItem: anItem];
-		  if (submenu == nil)
-		    {
-		      [self addItem: anItem];
-		    }
-		  else
-		    {
-		      [self insertItem: anItem atIndex: 0];	// Info menu
-		    }
-		}
-	      [self removeItem: appItem];
-	    }
-	}  
+              if ((k > 0) && ([[array objectAtIndex: k - 1] isSeparatorItem]))
+                k--;
+
+              if (k == 1)
+                {
+                  // Exactly one info item
+                  NSMenuItem *anItem = [array objectAtIndex: 0];
+
+                  [appMenu removeItem: anItem];
+                  [self insertItem: anItem atIndex: 0];
+                }
+              else if (k > 1)
+                {
+                  id <NSMenuItem> infoItem;
+                  NSMenu *infoMenu;
+
+                  // Multiple info items, add a submenu for them
+                  [self insertItemWithTitle: infoString
+                        action: NULL
+                        keyEquivalent: @"" 
+                        atIndex: 0];
+                  infoItem = [self itemAtIndex: 0];
+                  infoMenu = [NSMenu new];
+                  [self setSubmenu: infoMenu forItem: infoItem];
+                  RELEASE(infoMenu);
+
+                  for (i = 0; i < k; i++)
+                    {
+                      NSMenuItem *anItem = [array objectAtIndex: i];
+                  
+                      [appMenu removeItem: anItem];
+                      [infoMenu addItem: anItem];
+                    }
+                }
+              else
+                {
+                  // No service menu, or it is the first item.
+                  // We still look for an info item.
+                  NSMenuItem *anItem = [array objectAtIndex: 0];
+                  NSString *title = [anItem title];
+
+                  // The menu may not be localized, so we have to 
+                  // check both the English and the local version.
+                  if ([title isEqual: @"Info"] ||
+                      [title isEqual: infoString])
+                    {
+                      [appMenu removeItem: anItem];
+                      [self insertItem: anItem atIndex: 0];
+                      k = 1;
+                    }
+                  else
+                    {
+                      k = 0;
+                    }
+                }
+
+              // Copy the remaining entries.
+              for (i = k; i < [array count]; i++)
+                {
+                  NSMenuItem *anItem = [array objectAtIndex: i];
+                  
+                  [appMenu removeItem: anItem];
+                  [self addItem: anItem];
+                }
+
+              [self removeItem: appItem];
+            }
+        }  
     }
 
+  // recurse over all submenus
   for (i = 0; i < [_items count]; i++)
     {
-      NSMenuItem	*anItem = [_items objectAtIndex: i];
-      NSMenu		*submenu = [anItem submenu];
+      NSMenuItem *anItem = [_items objectAtIndex: i];
+      NSMenu *submenu = [anItem submenu];
 
       if (submenu != nil)
-	{
-	  if ([submenu isTransient])
-	    {
-	      [submenu closeTransient];
-	    }
-	  [submenu close];
-	  [submenu _organizeMenu];
-	}
+        {
+          if ([submenu isTransient])
+            {
+              [submenu closeTransient];
+            }
+          [submenu close];
+          [submenu _organizeMenu];
+        }
     }
+
   [[self menuRepresentation] update];
   [self sizeToFit];
 }
@@ -395,45 +466,45 @@ static BOOL menuBarVisible = YES;
       NSDebugLLog (@"NSMenu", @"Synchronizing user defaults");
       key = [self _locationKey];
       if (key != nil)
-	{
-	  NSUserDefaults	*defaults;
-	  NSMutableDictionary	*menuLocations;
-	  NSString		*locString;
+        {
+          NSUserDefaults	*defaults;
+          NSMutableDictionary	*menuLocations;
+          NSString		*locString;
 
-	  defaults = [NSUserDefaults standardUserDefaults];
-	  menuLocations = [defaults objectForKey: NSMenuLocationsKey];
-	  if ([menuLocations isKindOfClass: [NSDictionary class]])
-	    menuLocations = [menuLocations mutableCopy];
-	  else
-	    menuLocations = nil;
+          defaults = [NSUserDefaults standardUserDefaults];
+          menuLocations = [defaults objectForKey: NSMenuLocationsKey];
+          if ([menuLocations isKindOfClass: [NSDictionary class]])
+            menuLocations = AUTORELEASE([menuLocations mutableCopy]);
+          else
+            menuLocations = nil;
 
-	  if ([_aWindow isVisible]
-	    && ([self isTornOff] || ([NSApp mainMenu] == self)))
-	    {
-	      if (menuLocations == nil)
-		{
-		  menuLocations = AUTORELEASE([[NSMutableDictionary alloc]
-		    initWithCapacity: 2]);
-		}
-	      locString = [[self window] stringWithSavedFrame];
-	      [menuLocations setObject: locString forKey: key];
-	    }
-	  else
-	    {
-	      [menuLocations removeObjectForKey: key];
-	    }
-
-	  if ([menuLocations count] > 0)
-	    {
-	      [defaults setObject: menuLocations
-			   forKey: NSMenuLocationsKey];
-	    }
-	  else
-	    {
-	      [defaults removeObjectForKey: NSMenuLocationsKey];
-	    }
-	  [defaults synchronize];
-	}
+          if ([_aWindow isVisible]
+              && ([self isTornOff] || ([NSApp mainMenu] == self)))
+            {
+              if (menuLocations == nil)
+                {
+                  menuLocations = AUTORELEASE([[NSMutableDictionary alloc]
+                                                  initWithCapacity: 2]);
+                }
+              locString = [[self window] stringWithSavedFrame];
+              [menuLocations setObject: locString forKey: key];
+            }
+          else
+            {
+              [menuLocations removeObjectForKey: key];
+            }
+          
+          if ([menuLocations count] > 0)
+            {
+              [defaults setObject: menuLocations
+                        forKey: NSMenuLocationsKey];
+            }
+          else
+            {
+              [defaults removeObjectForKey: NSMenuLocationsKey];
+            }
+          [defaults synchronize];
+        }
     }
 }
 
@@ -942,6 +1013,31 @@ static BOOL menuBarVisible = YES;
 
 - (void) update
 {
+  if (self == [NSApp mainMenu])
+    {
+      /* For microsoft style menus, we attempt to notice that we have
+       * a main window, and make sure that the menu appears in it.
+       */
+      if (NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", nil) == 
+        NSWindows95InterfaceStyle)
+	{
+	  NSWindow	*w = [NSApp mainWindow];
+
+	  if (w == nil)
+	    {
+	      /* Make sure we are set up properly and displayed.
+	       */
+	      if ([_aWindow isVisible] == NO)
+		{
+	          [self setMain: YES];
+		}
+	    }
+	  else if ([w menu] != self)
+	    {
+	      [w setMenu: self];
+	    }
+	}
+    }
   if (_delegate)
     {
       if ([_delegate respondsToSelector:@selector(menuNeedsUpdate:)])
@@ -975,7 +1071,7 @@ static BOOL menuBarVisible = YES;
               while (cont && i < num)
                 {
                   cont = [_delegate menu: self
-                                    updateItem: [self itemAtIndex: i]
+                                    updateItem: (NSMenuItem*)[self itemAtIndex: i]
                                     atIndex: i
                                     shouldCancel: NO];
                   i++;
@@ -1042,6 +1138,11 @@ static BOOL menuBarVisible = YES;
 	    {
 	      shouldBeEnabled = [validator validateMenuItem: item];
 	    }
+	  else if ([validator
+		     respondsToSelector: @selector(validateUserInterfaceItem:)])
+	    {
+	      shouldBeEnabled = [validator validateUserInterfaceItem: item];
+	    }
 	  else
 	    {
 	      shouldBeEnabled = YES;
@@ -1181,10 +1282,10 @@ static BOOL menuBarVisible = YES;
 
 - (void) setDelegate: (id)delegate
 {
-  _delegate=delegate;
+  _delegate = delegate;
 }
 
-- (float)menuBarHeight
+- (float) menuBarHeight
 {
   // FIXME
   return [NSMenuView menuBarHeight];
@@ -1203,14 +1304,35 @@ static BOOL menuBarVisible = YES;
       return;
     }
 
-  _menu.horizontal = [menuRep isHorizontal];
+  /* If we are replacing a menu representation with a new version,
+   * we should display it in the same view as the old representation.
+   * If we can't find a view for that, we display in the content view
+   * of our default window.
+   */
+  if ([_view superview] == nil)
+    {
+      contentView = [_aWindow contentView];
+    }
+  else
+    {
+      contentView = [_view superview];
+    }
 
   if (_view == menuRep)
     {
+      /* Hack ... if the representation was 'borrowed' for an in-window
+       * menu, we will still have it recorded as ours, but it won't be
+       * in our view hierarchy, so we have to re-add it.
+       */
+      if (contentView != [menuRep superview])
+	{
+          [contentView addSubview: menuRep];
+	}
       return;
     }
 
-  contentView = [_aWindow contentView];
+  _menu.horizontal = [menuRep isHorizontal];
+
   if (_view != nil)
     {
       // remove the old representation
@@ -1408,7 +1530,20 @@ static BOOL menuBarVisible = YES;
 
   if ([aDecoder allowsKeyedCoding])
     {
-      dAuto = YES;
+      //
+      // NSNoAutoenable is present when the "Autoenable" option is NOT checked.
+      // NO = Autoenable menus, YES = Don't auto enable menus.  We, therefore,
+      // have to invert the values of this flag in order to get the value of
+      // dAuto.
+      //
+      if ([aDecoder containsValueForKey: @"NSNoAutoenable"])
+        {
+	  dAuto = ([aDecoder decodeBoolForKey: @"NSNoAutoenable"] == NO) ? YES : NO;
+	}
+      else
+	{
+	  dAuto = NO;
+	}
       dTitle = [aDecoder decodeObjectForKey: @"NSTitle"];
       dItems = [aDecoder decodeObjectForKey: @"NSMenuItems"];
     }
@@ -1489,8 +1624,8 @@ static BOOL menuBarVisible = YES;
 - (void) _showTornOffMenuIfAny: (NSNotification*)notification
 {
   NSInterfaceStyle style = NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", nil);
-  if (style == NSMacintoshInterfaceStyle || 
-      style == NSWindows95InterfaceStyle)
+  if (style == NSMacintoshInterfaceStyle
+    || style == NSWindows95InterfaceStyle)
     {
       return;
     }
@@ -1644,7 +1779,6 @@ static BOOL menuBarVisible = YES;
 {
   NSMenu *sub = [self attachedMenu];
 
-
   if (_menu.transient)
     {
       NSDebugLLog (@"NSMenu", @"We should not close ordinary menu while transient version is still open");
@@ -1727,33 +1861,51 @@ static BOOL menuBarVisible = YES;
        */
       if (oldStyle != newStyle)
         {
-	  NSMenuView	*newRep;
-
-	  newRep = [[NSMenuView alloc] initWithFrame: NSZeroRect];
-	  if (newStyle == NSMacintoshInterfaceStyle ||
-	      newStyle == NSWindows95InterfaceStyle)
-	    {
-	      [newRep setHorizontal: YES];
-	    }
-	  else
-	    {
-	      [newRep setHorizontal: NO];
-	    }
-	  [newRep setInterfaceStyle: newStyle];
-	  [self setMenuRepresentation: newRep];
-	  [self _organizeMenu];
-	  RELEASE(newRep);
-	}
-
-      [[self window] setTitle: [[NSProcessInfo processInfo] processName]];
-      [[self window] setLevel: NSMainMenuWindowLevel];
-      [self _setGeometry];
-      [self sizeToFit];
+          NSMenuView *newRep;
 
+          newRep = [[NSMenuView alloc] initWithFrame: NSZeroRect];
+          if (newStyle == NSMacintoshInterfaceStyle
+              || newStyle == NSWindows95InterfaceStyle)
+            {
+              [newRep setHorizontal: YES];
+            }
+          else
+            {
+              [newRep setHorizontal: NO];
+            }
+          [newRep setInterfaceStyle: newStyle];
+          [self setMenuRepresentation: newRep];
+          [self _organizeMenu];
+          RELEASE(newRep);
+          if (newStyle == NSWindows95InterfaceStyle)
+            {
+              /* Put menu in the main window for microsoft style.
+               */
+              [[NSApp mainWindow] setMenu: self];
+            }
+          else if ([[NSApp mainWindow] menu] == self)
+            {
+              /* Remove the menu from the main window.
+               */
+              [[NSApp mainWindow] setMenu: nil];
+            }
+        }
+      
+      /* Adjust the menu window to suit the menu view unless the menu
+       * is being displayed in the application main window.
+       */
+      if ([[NSApp mainWindow] menu] != self)
+        {
+          [[self window] setTitle: [[NSProcessInfo processInfo] processName]];
+          [[self window] setLevel: NSMainMenuWindowLevel];
+          [self _setGeometry];
+          [self sizeToFit];
+        }
+      
       if ([NSApp isActive])
         {
-	  [self display];
-	}
+          [self display];
+        }
     }
   else 
     {
