@@ -106,15 +106,6 @@ static NSMapTable *viewInfo = 0;
 
 @implementation NSMenuView
 
-static NSRect
-_addLeftBorderOffsetToRect(NSRect aRect)
-{
-  aRect.origin.x--;
-  aRect.size.width++;
-
-  return aRect;
-}
-
 /*
  * Class methods.
  */
@@ -296,10 +287,14 @@ _addLeftBorderOffsetToRect(NSRect aRect)
 {
   if (flag == YES && _horizontal == NO)
     {
+      NSRect scRect = [[NSScreen mainScreen] frame];
       GSIArray a = NSZoneMalloc(NSDefaultMallocZone(), sizeof(GSIArray_t));
 
       GSIArrayInitWithZoneAndCapacity(a, NSDefaultMallocZone(), 8);
       NSMapInsert(viewInfo, self, a);
+
+      scRect.size.height = [NSMenuView menuBarHeight];
+      [self setFrameSize: scRect.size];
     }
   else if (flag == NO && _horizontal == YES)
     {
@@ -394,6 +389,7 @@ _addLeftBorderOffsetToRect(NSRect aRect)
   // Mark the new cell and the menu view as needing resizing.
   [cell setNeedsSizing: YES];
   [self setNeedsSizing: YES];
+  [self setNeedsDisplayForItemAtIndex: index];
 }
 
 - (NSMenuItemCell*) menuItemCellForItemAtIndex: (int)index
@@ -445,10 +441,10 @@ _addLeftBorderOffsetToRect(NSRect aRect)
   [aCell setEnabled: [[aCell menuItem] isEnabled]];
   // Mark the cell associated with the item as needing resizing.
   [aCell setNeedsSizing: YES];
-  [self setNeedsDisplayForItemAtIndex: index];
 
   // Mark the menu view as needing to be resized.
   [self setNeedsSizing: YES];
+  [self setNeedsDisplayForItemAtIndex: index];
 }
 
 - (void) itemAdded: (NSNotification*)notification
@@ -486,6 +482,7 @@ _addLeftBorderOffsetToRect(NSRect aRect)
 
   // Mark the menu view as needing to be resized.
   [self setNeedsSizing: YES];
+  [self setNeedsDisplay: YES];
 }
 
 - (void) itemRemoved: (NSNotification*)notification
@@ -506,6 +503,7 @@ _addLeftBorderOffsetToRect(NSRect aRect)
     }
   // Mark the menu view as needing to be resized.
   [self setNeedsSizing: YES];
+  [self setNeedsDisplay: YES];
 }
 
 /*
@@ -644,13 +642,16 @@ _addLeftBorderOffsetToRect(NSRect aRect)
       unsigned i;
       unsigned howMany = [_itemCells count];
       float currentX = 8;
-      NSRect scRect = [[NSScreen mainScreen] frame];
+//      NSRect scRect = [[NSScreen mainScreen] frame];
 
       GSIArrayRemoveAllItems(cellRects);
 
+/*
       scRect.size.height = [NSMenuView menuBarHeight];
       [self setFrameSize: scRect.size];
       _cellSize.height = scRect.size.height;
+*/
+      _cellSize.height = [NSMenuView menuBarHeight];
 
       for (i = 0; i < howMany; i++)
         {
@@ -947,7 +948,9 @@ _addLeftBorderOffsetToRect(NSRect aRect)
     {
       NSRect aRect = [self rectOfItemAtIndex: i];
       
-      aRect = _addLeftBorderOffsetToRect(aRect);
+      //NSLog(@"indexOfItemAtPoint called for %@ %@ %d %@", self, NSStringFromPoint(point), i, NSStringFromRect(aRect));
+      aRect.origin.x -= _leftBorderOffset;
+      aRect.size.width +=  _leftBorderOffset;
 
       if (NSMouseInRect(point, aRect, NO))
         return (int)i;
@@ -961,7 +964,8 @@ _addLeftBorderOffsetToRect(NSRect aRect)
   NSRect aRect;
 
   aRect = [self rectOfItemAtIndex: index];
-  aRect = _addLeftBorderOffsetToRect(aRect);
+  aRect.origin.x -= _leftBorderOffset;
+  aRect.size.width +=  _leftBorderOffset;
   [self setNeedsDisplayInRect: aRect];
 }
 
@@ -984,10 +988,9 @@ _addLeftBorderOffsetToRect(NSRect aRect)
                                  [aSubmenu menuRepresentation])
           == GSWindowMakerInterfaceStyle)
         {
-          NSRect aRect = [self rectOfItemAtIndex: 
-            [_attachedMenu indexOfItemWithSubmenu: aSubmenu]];
-          NSPoint subOrigin = [_window convertBaseToScreen: 
-            NSMakePoint(aRect.origin.x, aRect.origin.y)];
+          NSRect aRect =  [self convertRect: [self rectOfItemAtIndex: 
+            [_attachedMenu indexOfItemWithSubmenu: aSubmenu]] toView: nil];
+          NSPoint subOrigin = [_window convertBaseToScreen: aRect.origin];
 
           return NSMakePoint (NSMaxX(frame),
             subOrigin.y - NSHeight(submenuFrame) - 3 +
@@ -995,10 +998,10 @@ _addLeftBorderOffsetToRect(NSRect aRect)
         }
       else if ([self _rootIsHorizontal: 0] == YES)
         {
-          NSRect aRect = [self rectOfItemAtIndex:
-            [_attachedMenu indexOfItemWithSubmenu: aSubmenu]];
-          NSPoint subOrigin = [_window convertBaseToScreen:
-            NSMakePoint(aRect.origin.x, aRect.origin.y)];
+          NSRect aRect =  [self convertRect: [self rectOfItemAtIndex: 
+            [_attachedMenu indexOfItemWithSubmenu: aSubmenu]] toView: nil];
+          NSPoint subOrigin = [_window convertBaseToScreen: aRect.origin];
+
           // FIXME ... why is the offset +1 needed below? 
           return NSMakePoint (NSMaxX(frame),
             subOrigin.y - NSHeight(submenuFrame) + aRect.size.height + 1);
@@ -1011,11 +1014,9 @@ _addLeftBorderOffsetToRect(NSRect aRect)
     }
   else
     {
-      NSRect aRect = [self rectOfItemAtIndex: 
-                       [_attachedMenu indexOfItemWithSubmenu: aSubmenu]];
-      NSPoint subOrigin = [_window convertBaseToScreen: 
-                                    NSMakePoint(NSMinX(aRect),
-                                    NSMinY(aRect))];
+      NSRect aRect =  [self convertRect: [self rectOfItemAtIndex: 
+	[_attachedMenu indexOfItemWithSubmenu: aSubmenu]] toView: nil];
+      NSPoint subOrigin = [_window convertBaseToScreen: aRect.origin];
 
       return NSMakePoint(subOrigin.x, subOrigin.y - NSHeight(submenuFrame));
     }
@@ -1153,29 +1154,43 @@ _addLeftBorderOffsetToRect(NSRect aRect)
     }  
   
   // Update position, if needed, using the preferredEdge
-  if (edge == NSMinYEdge)
+  if ([_attachedMenu _ownedByPopUp])
     {
-/*
-      if ([_attachedMenu _ownedByPopUp] && 
-          ([[_attachedMenu _owningPopUp] pullsDown]))
-        {
-          screenFrame.origin.y -= screenRect.size.height;  
-        }
-*/
+      // Per Cocoa documentation, the selected cell should always show up
+      // over the button for popups.   For pull down menus, the preferred 
+      // edge is relevant.   This is also apparent from testing under Cocoa.
+      if ([[_attachedMenu _owningPopUp] pullsDown])
+	{
+	  // Handle the Y edge...
+	  /*
+	  if ([[[_attachedMenu _owningPopUp] controlView] isFlipped])
+	    {
+	      if (edge == NSMaxYEdge)
+		{
+		  screenFrame.origin.y += screenRect.size.height;  
+		}
+	    }
+	  else
+	    {
+	      if (edge == NSMinYEdge)
+		{
+		  screenFrame.origin.y += screenRect.size.height;  
+		}
+	    }
+	  */
+
+	  // Handle the X edge...
+	  if (edge == NSMaxXEdge)
+	    {
+	      screenFrame.origin.x += screenRect.size.width;
+	    }
+	  else if (edge == NSMinXEdge)
+	    {
+	      screenFrame.origin.x -= screenRect.size.width;
+	    }
+	}
     }
-  else if (edge == NSMaxYEdge)
-    {
-      screenFrame.origin.y += screenRect.size.height;  
-    }
-  else if (edge == NSMaxXEdge)
-    {
-      screenFrame.origin.x += screenRect.size.width;
-    }
-  else if (edge == NSMinXEdge)
-    {
-      screenFrame.origin.x -= screenRect.size.width;
-    }
-  
+
   // Get the frameRect
   r = [NSWindow frameRectForContentRect: screenFrame
                 styleMask: [_window styleMask]];
@@ -1277,6 +1292,52 @@ _addLeftBorderOffsetToRect(NSRect aRect)
 #define MOVE_THRESHOLD_DELTA 2.0
 #define DELAY_MULTIPLIER     10
 
+- (BOOL) _executeItemAtIndex: (int)indexOfActionToExecute
+	       removeSubmenu: (BOOL)subMenusNeedRemoving
+{
+  if (indexOfActionToExecute >= 0
+      && [_attachedMenu attachedMenu] != nil && [_attachedMenu attachedMenu] ==
+      [[_items_link objectAtIndex: indexOfActionToExecute] submenu])
+    {
+#if 1
+      if (NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", self)
+	  == NSMacintoshInterfaceStyle)
+        {
+          /*
+           * FIXME ... always remove submenus in mac mode ... this is not
+           * quite the way the mac behaves, but it's closer than the normal
+           * behavior.
+           */
+          subMenusNeedRemoving = YES;
+        }
+#endif
+      if (subMenusNeedRemoving)
+        {
+          [self detachSubmenu];
+        }
+      // Clicked on a submenu.
+      return NO;
+    }
+  
+  if (NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", self)
+      == NSMacintoshInterfaceStyle)
+    {
+      NSMenu *tmp = _attachedMenu;
+      
+      do
+        {
+          if ([tmp isEqual: [NSApp mainMenu]] == NO)
+            {
+              [tmp close];
+            }
+          tmp = [tmp supermenu];
+        }
+      while (tmp != nil);
+    }
+
+  return YES;
+}
+
 - (BOOL) trackWithEvent: (NSEvent*)event
 {
   unsigned eventMask = NSPeriodicMask;
@@ -1347,7 +1408,8 @@ _addLeftBorderOffsetToRect(NSRect aRect)
           int index;
 
           location = [_window mouseLocationOutsideOfEventStream];
-          index    = [self indexOfItemAtPoint: location];
+          index = [self indexOfItemAtPoint: 
+            [self convertPoint: location fromView: nil]];
 
           if (event == original)
             {
@@ -1536,75 +1598,40 @@ _addLeftBorderOffsetToRect(NSRect aRect)
   indexOfActionToExecute = _highlightedItemIndex;
 
   // remove transient menus. --------------------------------------------
-    {
-      NSMenu *currentMenu = _attachedMenu;
-
-      while (currentMenu && ![currentMenu isTransient])
-        {
-          currentMenu = [currentMenu attachedMenu];
-        }
-
-      while ([currentMenu isTransient] && [currentMenu supermenu])
-        {
-          currentMenu = [currentMenu supermenu];
-        }
-
-      [[currentMenu menuRepresentation] detachSubmenu];
-
-      if ([currentMenu isTransient])
-        {
-          [currentMenu closeTransient];
-        }
-    }
-
-  // ---------------------------------------------------------------------
+  {
+    NSMenu *currentMenu = _attachedMenu;
+    
+    while (currentMenu && ![currentMenu isTransient])
+      {
+	currentMenu = [currentMenu attachedMenu];
+      }
+    
+    while ([currentMenu isTransient] && [currentMenu supermenu])
+      {
+	currentMenu = [currentMenu supermenu];
+      }
+    
+    [[currentMenu menuRepresentation] detachSubmenu];
+    
+    if ([currentMenu isTransient])
+      {
+	[currentMenu closeTransient];
+      }
+  }
+  
   if (indexOfActionToExecute == -1)
     {
       return YES;
     }
 
-  if (indexOfActionToExecute >= 0
-    && [_attachedMenu attachedMenu] != nil && [_attachedMenu attachedMenu] ==
-    [[_items_link objectAtIndex: indexOfActionToExecute] submenu])
+  if([self _executeItemAtIndex: indexOfActionToExecute
+	   removeSubmenu: subMenusNeedRemoving] == NO)
     {
-#if 1
-      if (NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", self)
-        == NSMacintoshInterfaceStyle)
-        {
-          /*
-           * FIXME ... always remove submenus in mac mode ... this is not
-           * quite the way the mac behaves, but it's closer than the normal
-           * behavior.
-           */
-          subMenusNeedRemoving = YES;
-        }
-#endif
-      if (subMenusNeedRemoving)
-        {
-          [self detachSubmenu];
-        }
-      // Clicked on a submenu.
       return NO;
     }
 
-  if (NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", self)
-    == NSMacintoshInterfaceStyle)
-    {
-      NSMenu *tmp = _attachedMenu;
-
-      do
-        {
-          if ([tmp isEqual: [NSApp mainMenu]] == NO)
-            {
-              [tmp close];
-            }
-          tmp = [tmp supermenu];
-        }
-      while (tmp != nil);
-    }
-
   [_attachedMenu performActionForItemAtIndex: indexOfActionToExecute];
-
+  
   /*
    * Remove highlighting.
    * We first check if it still highlighted because it could be the
