@@ -814,7 +814,7 @@ static NSImage *unexpandable  = nil;
   /* Determine starting column as fast as possible */
   x_pos = NSMinX (aRect);
   i = 0;
-  while ((x_pos > _columnOrigins[i]) && (i < _numberOfColumns))
+  while ((i < _numberOfColumns) && (x_pos > _columnOrigins[i]))
     {
       i++;
     }
@@ -826,7 +826,7 @@ static NSImage *unexpandable  = nil;
   /* Determine ending column as fast as possible */
   x_pos = NSMaxX (aRect);
   // Nota Bene: we do *not* reset i
-  while ((x_pos > _columnOrigins[i]) && (i < _numberOfColumns))
+  while ((i < _numberOfColumns) && (x_pos > _columnOrigins[i]))
     {
       i++;
     }
@@ -1716,9 +1716,12 @@ static NSImage *unexpandable  = nil;
   id sitem = (startitem == nil) ? (id)[NSNull null] : (id)startitem;
   NSMutableArray *anarray = nil;
 
-  // Check to see if item is expandable before getting the number of
-  // items.
-  if([_dataSource outlineView: self isItemExpandable: startitem])
+  /* Check to see if item is expandable before getting the number of
+   * items. For macos compatibility the topmost item (startitem==nil)
+   * is always considered expandable and must not be checked.
+   */
+  if (startitem == nil
+    || [_dataSource outlineView: self isItemExpandable: startitem])
     {
       num = [_dataSource outlineView: self
 			 numberOfChildrenOfItem: startitem];
