@@ -220,15 +220,14 @@ static NSParagraphStyle	*defaultStyle = nil;
       int		i;
 
       for (i = 0; i < 12; i++)
-	{
-	  NSTextTab	*tab;
+        {
+          NSTextTab	*tab;
 
-	  /* FIXME: (i * 1) ? */
-	  tab = [[NSTextTab alloc] initWithType: NSLeftTabStopType
-				       location: (i * 1) * 28.0];
-	  [style->_tabStops addObject: tab];
-	  RELEASE(tab);
-	}
+          tab = [[NSTextTab alloc] initWithType: NSLeftTabStopType
+                                   location: (i + 1) * 28.0];
+          [style->_tabStops addObject: tab];
+          RELEASE(tab);
+        }
       defaultStyle = style;
     }
   return defaultStyle;
@@ -449,7 +448,8 @@ static NSParagraphStyle	*defaultStyle = nil;
       NSParagraphStyle	*c;
 
       c = (NSParagraphStyle*)NSCopyObject (self, 0, aZone);
-      c->_tabStops = [_tabStops mutableCopyWithZone: aZone];
+      c->_textBlocks = [_textBlocks mutableCopyWithZone: aZone];
+      c->_textLists = [_textLists mutableCopyWithZone: aZone];
       return c;
     }
 }
@@ -781,6 +781,11 @@ static NSParagraphStyle	*defaultStyle = nil;
   [_tabStops removeAllObjects];
   [_tabStops addObjectsFromArray: p->_tabStops];
 
+  if (p->_textBlocks)
+    [self setTextBlocks: p->_textBlocks];
+  if (p->_textLists)
+    [self setTextLists: p->_textLists];
+
   _alignment = p->_alignment;
   _firstLineHeadIndent = p->_firstLineHeadIndent;
   _headIndent = p->_headIndent;
@@ -808,7 +813,9 @@ static NSParagraphStyle	*defaultStyle = nil;
   c->isa = [NSParagraphStyle class];
   GSDebugAllocationAdd(c->isa, c);
   c->_tabStops = [_tabStops mutableCopyWithZone: aZone];
-  return c;
+  c->_textBlocks = [_textBlocks mutableCopyWithZone: aZone];
+  c->_textLists = [_textLists mutableCopyWithZone: aZone];
+   return c;
 }
 
 @end
