@@ -44,6 +44,10 @@
  * and so on.  */
 static NSImage *_pbc_image[5];
 
+@interface NSPopUpButtonCell (CocoaExtensions)
+- (void) _popUpItemAction: (id)sender;
+@end
+
 @implementation NSPopUpButtonCell
 + (void) initialize
 {
@@ -384,6 +388,9 @@ static NSImage *_pbc_image[5];
   // FIXME: The documentation is unclear what to set here.
   //[anItem setAction: [self action]];
   //[anItem setTarget: [self target]];
+  // Or
+  //[anItem setAction: @selector(_popUpItemAction:)];
+  //[anItem setTarget: self];
 
   // Select the new item if there isn't any selection.
   if (_selectedItem == nil)
@@ -828,7 +835,10 @@ static NSImage *_pbc_image[5];
       onScreen: [cvWin screen]
       preferredEdge: _pbcFlags.preferredEdge
       popUpSelectedItem: selectedItem];
-  
+
+  // Set to be above the main window
+  [cvWin addChildWindow: [mr window] ordered: NSWindowAbove];
+
   // Last, display the window
   [[mr window] orderFrontRegardless];
 
@@ -844,11 +854,17 @@ static NSImage *_pbc_image[5];
 - (void) dismissPopUp
 {
   NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
+  NSMenuView *mr = [_menu menuRepresentation];
+  NSWindow *mWin;
 
   [nc removeObserver: self
       name: NSMenuDidSendActionNotification
       object: _menu];
   [_menu close];
+
+  // remove from main window
+  mWin = [mr window];
+  [[mWin parentWindow] removeChildWindow: mWin];
 }
 
 /* Private method handles all cases after doing a selection from 
@@ -1019,7 +1035,7 @@ static NSImage *_pbc_image[5];
   s.width += 4; /* Right border to image (border included) */
 
   /* (vertical) border: */
-  s.height += 2 * (_sizeForBorderType (NSBezelBorder).height);
+  s.height += 2 * [[GSTheme theme] sizeForBorderType: NSBezelBorder].height;
 
   /* Spacing between border and inside: */
   s.height += 2 * 1;
@@ -1048,21 +1064,25 @@ static NSImage *_pbc_image[5];
   [super encodeWithCoder: aCoder];
   if ([aCoder allowsKeyedCoding])
     {
-      [aCoder encodeBool: [self altersStateOfSelectedItem] forKey: @"NSAltersState"];
-      [aCoder encodeBool: [self usesItemFromMenu] forKey: @"NSUsesItemFromMenu"];
-      [aCoder encodeInt: [self arrowPosition] forKey: @"NSArrowPosition"];
-      [aCoder encodeInt: [self preferredEdge] forKey: @"NSPreferredEdge"];
-      
+      [aCoder encodeBool: [self altersStateOfSelectedItem] 
+	      forKey: @"NSAltersState"];
+      [aCoder encodeBool: [self usesItemFromMenu] 
+	      forKey: @"NSUsesItemFromMenu"];
+      [aCoder encodeInt:  [self arrowPosition] 
+	      forKey: @"NSArrowPosition"];
+      [aCoder encodeInt:  [self preferredEdge] 
+	      forKey: @"NSPreferredEdge"];
+      [aCoder encodeInt:  [self indexOfSelectedItem] 
+	      forKey: @"NSSelectedIndex"];
+      [aCoder encodeBool: [self pullsDown]
+	      forKey: @"NSPullDown"];
+
       // encode the menu, if present.
       if (_menu != nil)
         {
-          [aCoder encodeObject: _menu forKey: @"NSMenu"];
+          [aCoder encodeObject: _menu 
+		  forKey: @"NSMenu"];
         }
-
-      if (_menuItem != nil)
-	{
-	  [aCoder encodeObject: _menuItem forKey: @"NSMenuItem"];
-	}
     }
   else
     {    
@@ -1107,7 +1127,7 @@ static NSImage *_pbc_image[5];
       if ([aDecoder containsValueForKey: @"NSArrowPosition"])
         {
           NSPopUpArrowPosition position = [aDecoder decodeIntForKey: 
-                                                        @"NSArrowPosition"];
+						      @"NSArrowPosition"];
           
           [self setArrowPosition: position];
         }
@@ -1117,10 +1137,23 @@ static NSImage *_pbc_image[5];
           
           [self setPreferredEdge: edge];
         }
+      if ([aDecoder containsValueForKey: @"NSSelectedIndex"])
+        {
+	  int selectedIdx = [aDecoder decodeIntForKey: 
+					@"NSSelectedIndex"];
+	  [self selectItem: [self itemAtIndex: selectedIdx]];
+	}
+      if ([aDecoder containsValueForKey: @"NSPullDown"])
+        {
+	  BOOL pullDown = [aDecoder decodeBoolForKey: @"NSPullDown"];
+	  [self setPullsDown: pullDown];
+	}
 
       menu = [aDecoder decodeObjectForKey: @"NSMenu"];
       [self setMenu: nil];
       [self setMenu: menu];
+      [self setAutoenablesItems: NO];
+      [self setEnabled: YES];
     }
   else
     {
@@ -1181,3 +1214,15 @@ static NSImage *_pbc_image[5];
 }
 
 @end
+
+@implementation NSPopUpButtonCell (CocoaExtensions)
+
+/*
+ * The selector for this method gets used by menu items in Apple NIB files.
+ */
+- (void) _popUpItemAction: (id)sender
+{
+  [NSApp sendAction: [self action] to: [self target] from: self];
+}
+
+@end
