@@ -227,7 +227,7 @@
 - (BOOL)trackMouse:(NSEvent *)theEvent adding:(BOOL)adding
 {
   NSView *client = [_rulerView clientView];
-  NSEvent *newEvent;
+  NSEvent *newEvent = nil;
   int eventMask = NSLeftMouseDraggedMask | NSLeftMouseUpMask;
   BOOL isFar = NO;
   BOOL askedCanRemove = NO;
@@ -241,6 +241,7 @@
   NSRect markersRect;
   float location;
   NSRect drawRect;
+  NSRect bounds = [_rulerView bounds];
   NSPoint drawPoint;
   BOOL returnValue = NO;
   NSWindow *window;
@@ -340,12 +341,22 @@
   /* loop processing events until mouse up */
   while (_isDragging)
     {
-      newEvent = [NSApp nextEventMatchingMask: eventMask
-                                    untilDate: [NSDate distantFuture]
-                                       inMode: NSEventTrackingRunLoopMode
-                                      dequeue: YES];
+      if(newEvent == nil) 
+	{
+	  newEvent = theEvent;
+	  previousMousePositionInWindow = NSMakePoint(0,0); 
+	}
+      else
+	{
+	  newEvent = [NSApp nextEventMatchingMask: eventMask
+			    untilDate: [NSDate distantFuture]
+			    inMode: NSEventTrackingRunLoopMode
+			    dequeue: YES];
+	}
+
       switch ([newEvent type]) 
-        {
+	{
+	  case NSLeftMouseDown:
 	  case NSLeftMouseDragged:
 	      /* take mouse position from outside of event stream
 	         and ignore event if in same position as previous event, 
@@ -364,7 +375,7 @@
 	      /* see if mouse is far from ruler area (to remove marker) */
               mousePositionInRuler = [_rulerView convertPoint: mousePositionInWindow
                                                      fromView: nil];
-	      isFar = !NSMouseInRect(mousePositionInRuler, markersRect, flipped);
+	      isFar = !NSMouseInRect(mousePositionInRuler, bounds, flipped);
 	      
 	      /* if it is the first time it's far from the ruler area,
 	         see if it can be removed */
