@@ -35,6 +35,7 @@
 #include <Foundation/NSNotification.h>
 
 #include "AppKit/NSColor.h"
+#include "AppKit/NSColorList.h"
 #include "AppKit/NSCell.h"
 #include "AppKit/NSClipView.h"
 #include "AppKit/NSEvent.h"
@@ -60,21 +61,43 @@
 typedef struct _scrollViewFlags 
 {
 #ifdef WORDS_BIGENDIAN
-  unsigned int __unused4:22;
+  unsigned int __unused6:14;
+  unsigned int __unused5:1;
   unsigned int autohidesScrollers:1;
-  unsigned int __unused1:3;
+  unsigned int __unused4:1;
+  unsigned int __unused3:1;
+  unsigned int __unused2:1;
+  unsigned int doesNotDrawBackground:1;
+  unsigned int __unused1:1;
+  unsigned int hasVRuler:1;
+  unsigned int hasHRuler:1;
+  unsigned int showRulers:1;
+  unsigned int oldRulerInstalled:1;
+  unsigned int nonDynamic:1;
   unsigned int hasHScroller:1;
   unsigned int hasVScroller:1;
-  unsigned int __unused0:2;
+  unsigned int hScrollerRequired:1;
+  unsigned int vScrollerRequired:1;
   NSBorderType border:2;
 #else
   NSBorderType border:2;
-  unsigned int __unused0:2;
+  unsigned int vScrollerRequired:1;
+  unsigned int hScrollerRequired:1;
   unsigned int hasVScroller:1;
   unsigned int hasHScroller:1;
-  unsigned int __unused1:3;
+  unsigned int nonDynamic:1;
+  unsigned int oldRulerInstalled:1;
+  unsigned int showRulers:1;
+  unsigned int hasHRuler:1;
+  unsigned int hasVRuler:1;
+  unsigned int __unused1:1;
+  unsigned int doesNotDrawBackground:1;
+  unsigned int __unused2:1;
+  unsigned int __unused3:1;
+  unsigned int __unused4:1;
   unsigned int autohidesScrollers:1;
-  unsigned int __unused4:22;
+  unsigned int __unused5:1;
+  unsigned int __unused6:14;
 #endif  
 } GSScrollViewFlags;
 
@@ -121,7 +144,7 @@ static float scrollerWidth;
                         borderType: (NSBorderType)borderType
 {
   NSSize size = frameSize;
-  NSSize border = _sizeForBorderType(borderType);
+  NSSize border = [[GSTheme theme] sizeForBorderType: borderType];
 
   /*
    * Substract 1 from the width and height of
@@ -149,7 +172,7 @@ static float scrollerWidth;
                         borderType: (NSBorderType)borderType
 {
   NSSize size = contentSize;
-  NSSize border = _sizeForBorderType(borderType);
+  NSSize border = [[GSTheme theme] sizeForBorderType: borderType];
 
   /*
    * Add 1 to the width and height for the line that separates the
@@ -737,7 +760,8 @@ static float scrollerWidth;
     }
 
   // FIXME: Should we just hide the scroll bar or remove it?
-  if ((_autohidesScrollers) && (documentFrame.size.height > clipViewBounds.size.height))
+  if ((_autohidesScrollers)
+    && (documentFrame.size.height > clipViewBounds.size.height))
     {
       [self setHasVerticalScroller: YES];        
     } 
@@ -774,7 +798,8 @@ static float scrollerWidth;
         }
     }
 
-  if ((_autohidesScrollers) && (documentFrame.size.width > clipViewBounds.size.width))
+  if ((_autohidesScrollers)
+    && (documentFrame.size.width > clipViewBounds.size.width))
     {
       [self setHasHorizontalScroller: YES];        
     } 
@@ -974,13 +999,14 @@ static float scrollerWidth;
 - (void) tile
 {
   NSRect headerRect, contentRect;
-  NSSize border = _sizeForBorderType(_borderType);
+  NSSize border = [[GSTheme theme] sizeForBorderType: _borderType];
   NSRectEdge bottomEdge, topEdge;
   float headerViewHeight = 0;
   NSRectEdge verticalScrollerEdge = NSMinXEdge;
   NSInterfaceStyle style;
 
   style = NSInterfaceStyleForKey(@"NSScrollViewInterfaceStyle", nil);
+
   if (style == NSMacintoshInterfaceStyle
     || style == NSWindows95InterfaceStyle)
     {
@@ -1001,6 +1027,12 @@ static float scrollerWidth;
 
   /* Prepare the contentRect by insetting the borders.  */
   contentRect = NSInsetRect(_bounds, border.width, border.height);
+  if (contentRect.size.width < 0 || contentRect.size.height < 0)
+    {
+      /* FIXME ... should we do something else when given
+       * too small a size to tile? */
+      return;
+    }
   
   [self _synchronizeHeaderAndCornerView];
   
@@ -1075,7 +1107,7 @@ static float scrollerWidth;
   /* Now place the corner view.  */
   if (_hasCornerView)
     {
-      NSPoint        p = headerRect.origin;
+      NSPoint p = headerRect.origin;
 
       if (verticalScrollerEdge == NSMaxXEdge)
         {
@@ -1113,6 +1145,20 @@ static float scrollerWidth;
 - (void) drawRect: (NSRect)rect
 {
   NSGraphicsContext *ctxt = GSCurrentContext();
+  GSTheme	*theme = [GSTheme theme];
+  NSColor	*color;
+  NSString	*name;
+
+  name = [theme nameForElement: self];
+  if (name == nil)
+    {
+      name = @"NSScrollView";
+    }
+  color = [theme colorNamed: name state: GSThemeNormalState cache: YES];
+  if (color == nil)
+    {
+      color = [NSColor controlDarkShadowColor];
+    }
   
   switch (_borderType)
     {
@@ -1120,20 +1166,20 @@ static float scrollerWidth;
         break;
 
       case NSLineBorder:
-        [[NSColor controlDarkShadowColor] set];
+        [color set];
         NSFrameRect(_bounds);
         break;
 
       case NSBezelBorder:
-        [[GSTheme theme] drawGrayBezel: _bounds withClip: rect];
+        [theme drawGrayBezel: _bounds withClip: rect];
         break;
 
       case NSGrooveBorder:
-        [[GSTheme theme] drawGroove: _bounds withClip: rect];
+        [theme drawGroove: _bounds withClip: rect];
         break;
     }
 
-  [[NSColor controlDarkShadowColor] set];
+  [color set];
   DPSsetlinewidth(ctxt, 1);
 
   if (_hasVertScroller)
@@ -1426,7 +1472,7 @@ static float scrollerWidth;
       scrollViewFlags.hasHScroller = _hasHorizScroller;
       scrollViewFlags.border = _borderType;
       scrollViewFlags.__unused4 = 0;
-      scrollViewFlags.__unused0 = 0;
+      scrollViewFlags.__unused1 = 0;
       memcpy((void *)&flags, (void *)&scrollViewFlags,sizeof(unsigned long));
 
       [aCoder encodeInt: flags forKey: @"NSsFlags"];
@@ -1479,7 +1525,12 @@ static float scrollerWidth;
       NSScroller *hScroller = [aDecoder decodeObjectForKey: @"NSHScroller"];
       NSScroller *vScroller = [aDecoder decodeObjectForKey: @"NSVScroller"];
       NSClipView *content = [aDecoder decodeObjectForKey: @"NSContentView"]; 
+      NSView *docView = [content documentView];
+      BOOL post_frame = [docView postsFrameChangedNotifications];
+      BOOL post_bound = [docView postsBoundsChangedNotifications];
 
+      [docView setPostsFrameChangedNotifications: NO];
+      [docView setPostsBoundsChangedNotifications: NO];
       _hLineScroll = 10;
       _hPageScroll = 10;
       _vLineScroll = 10;
@@ -1490,23 +1541,27 @@ static float scrollerWidth;
         {
           int flags = [aDecoder decodeInt32ForKey: @"NSsFlags"];
           GSScrollViewFlags scrollViewFlags;
-          // FIXME: Why not just use a cast?
+	  
+	  //
+	  // Do a memory copy here since the compiler errors out on casts from
+	  // scalar (int/float/long) types to structs.
+	  //
           memcpy((void *)&scrollViewFlags,(void *)&flags,sizeof(struct _scrollViewFlags));
 
           _hasVertScroller = scrollViewFlags.hasVScroller;
           _hasHorizScroller = scrollViewFlags.hasHScroller;
           _autohidesScrollers = scrollViewFlags.autohidesScrollers;
-          // _scrollsDynamically = (!scrollViewFlags.notDynamic);
-          // _rulersVisible = scrollViewFlags.rulersVisible;
-          // _hasHorizRuler = scrollViewFlags.hasHRuler;
-          // _hasVertRuler = scrollViewFlags.hasVRuler;
+          _scrollsDynamically = (!scrollViewFlags.nonDynamic);
+          _rulersVisible = scrollViewFlags.showRulers;
+          _hasHorizRuler = scrollViewFlags.hasHRuler;
+          _hasVertRuler = scrollViewFlags.hasVRuler;
           // [self setDrawsBackground: (!scrollViewFlags.doesNotDrawBack)];
           _borderType = scrollViewFlags.border;
         }
 
       if (content != nil)
         {
-          // FIXME: I think this is not needed as we re-tile anyway.
+          // Move the content view since it is not moved when we retile.
           NSRect frame = [content frame];
           float w = [vScroller frame].size.width;
 
@@ -1520,28 +1575,32 @@ static float scrollerWidth;
               frame.origin.x += w;
               [content setFrame: frame];
             }
-
-          [self setContentView: content];
         }
      
       if (hScroller != nil && _hasHorizScroller)
         {
           [self setHorizontalScroller: hScroller];
+	  [hScroller setHidden: NO];
         }
 
       if (vScroller != nil && _hasVertScroller)
         {
           [self setVerticalScroller: vScroller];
+	  [vScroller setHidden: NO];
         }
 
       if ([aDecoder containsValueForKey: @"NSHeaderClipView"])
         {
           _hasHeaderView = YES;
-          _hasCornerView = YES;          
           _headerClipView = [aDecoder decodeObjectForKey: @"NSHeaderClipView"];
         }
 
-      [self tile];
+      // [self tile];
+      // set the document view into the content.
+      [self addSubview: vScroller];
+      [self setContentView: content];
+      [docView setPostsFrameChangedNotifications: post_frame];
+      [docView setPostsBoundsChangedNotifications: post_bound];
     }
   else
     {
@@ -1595,6 +1654,12 @@ static float scrollerWidth;
       NSDebugLLog(@"NSScrollView", @"NSScrollView: finish decoding\n");
     }
 
+  [[NSNotificationCenter defaultCenter]
+    addObserver: self
+    selector: @selector(_themeDidActivate:)
+    name: GSThemeDidActivateNotification
+    object: nil];
+
   return self;
 }
 
