@@ -33,6 +33,7 @@
 
 #include <Foundation/NSDate.h>
 #include <Foundation/NSRunLoop.h>
+#include <Foundation/NSNotification.h>
 #include <Foundation/NSDebug.h>
 
 #include "AppKit/NSApplication.h"
@@ -45,6 +46,8 @@
 #include "AppKit/NSScrollView.h"
 #include "AppKit/NSWindow.h"
 
+#include "GNUstepGUI/GSTheme.h"
+
 /**<p>TODO Description</p>
  */
 @implementation NSScroller
@@ -54,16 +57,33 @@
  */
 
 /* button cells used by scroller instances to draw scroller buttons and knob. */
-static NSButtonCell* upCell = nil;
-static NSButtonCell* downCell = nil;
-static NSButtonCell* leftCell = nil;
-static NSButtonCell* rightCell = nil;
-static NSButtonCell* knobCell = nil;
+static NSButtonCell	*upCell = nil;
+static NSButtonCell	*downCell = nil;
+static NSButtonCell	*leftCell = nil;
+static NSButtonCell	*rightCell = nil;
+static NSCell	*horizontalKnobCell = nil;
+static NSCell	*verticalKnobCell = nil;
+static NSCell	*horizontalKnobSlotCell = nil;
+static NSCell	*verticalKnobSlotCell = nil;
+static float	scrollerWidth = 18.0;
 
-static const float scrollerWidth = 18;
 static const float buttonsOffset = 2; // buttonsWidth = sw - buttonsOffset
 
-static NSColor *scrollBarColor = nil;
++ (void) _themeWillDeactivate: (NSNotification*)n
+{
+  /* Clear cached information from the old theme ... will get info from
+   * the new theme as required.
+   */
+  scrollerWidth = 0.0;
+  upCell = nil;
+  downCell = nil;
+  leftCell = nil;
+  rightCell = nil;
+  horizontalKnobCell = nil;
+  verticalKnobCell = nil;
+  horizontalKnobSlotCell = nil;
+  verticalKnobSlotCell = nil;
+}
 
 /*
  * Class methods
@@ -73,7 +93,10 @@ static NSColor *scrollBarColor = nil;
   if (self == [NSScroller class])
     {
       [self setVersion: 1];
-      ASSIGN (scrollBarColor, [NSColor scrollBarColor]);
+      [[NSNotificationCenter defaultCenter] addObserver: self
+	selector: @selector(_themeWillDeactivate:)
+	name: GSThemeWillDeactivateNotification
+	object: nil];
     }
 }
 
@@ -83,6 +106,10 @@ static NSColor *scrollBarColor = nil;
  */
 + (float) scrollerWidth
 {
+  if (scrollerWidth == 0.0)
+    {
+      scrollerWidth = [[GSTheme theme] defaultScrollerWidth];  
+    }
   return scrollerWidth;
 }
 
@@ -254,7 +281,8 @@ static NSColor *scrollBarColor = nil;
       
       _hitPart = NSScrollerNoPart;
       
-      [aDecoder decodeValueOfObjCType: @encode(unsigned int) at: &_arrowsPosition];
+      [aDecoder decodeValueOfObjCType: @encode(unsigned int)
+				   at: &_arrowsPosition];
       [aDecoder decodeValueOfObjCType: @encode(BOOL) at: &_isEnabled];
       [aDecoder decodeValueOfObjCType: @encode(id) at: &_target];
       // Undo RETAIN by decoder
@@ -323,54 +351,43 @@ static NSColor *scrollBarColor = nil;
  */
 - (void) drawParts
 {
-  /*
-   * Create the class variable button cells if they do not yet exist.
-   */
-  if (knobCell)
+  GSTheme *theme;
+
+  if (upCell)
     return;
 
-  upCell = [NSButtonCell new];
-  [upCell setHighlightsBy: NSChangeBackgroundCellMask | NSContentsCellMask];
-  [upCell setImage: [NSImage imageNamed: @"common_ArrowUp"]];
-  [upCell setAlternateImage: [NSImage imageNamed: @"common_ArrowUpH"]];
-  [upCell setImagePosition: NSImageOnly];
-  [upCell setContinuous: YES];
-  [upCell sendActionOn: (NSLeftMouseDownMask | NSPeriodicMask)];
-  [upCell setPeriodicDelay: 0.3 interval: 0.03];
+  theme = [GSTheme theme];
+
+  upCell
+    = [theme cellForScrollerArrow: NSScrollerDecrementArrow horizontal:NO];
+  downCell
+    = [theme cellForScrollerArrow: NSScrollerIncrementArrow horizontal:NO];
+  leftCell
+    = [theme cellForScrollerArrow: NSScrollerDecrementArrow horizontal:YES];
+  rightCell
+    = [theme cellForScrollerArrow: NSScrollerIncrementArrow horizontal:YES];
+  verticalKnobCell = [theme cellForScrollerKnob: NO];
+  horizontalKnobCell = [theme cellForScrollerKnob: YES];
+  verticalKnobSlotCell = [theme cellForScrollerKnobSlot: NO];
+  horizontalKnobSlotCell = [theme cellForScrollerKnobSlot: YES];
 
-  downCell = [NSButtonCell new];
-  [downCell setHighlightsBy: NSChangeBackgroundCellMask | NSContentsCellMask];
-  [downCell setImage: [NSImage imageNamed: @"common_ArrowDown"]];
-  [downCell setAlternateImage: [NSImage imageNamed: @"common_ArrowDownH"]];
-  [downCell setImagePosition: NSImageOnly];
   [downCell setContinuous: YES];
   [downCell sendActionOn: (NSLeftMouseDownMask | NSPeriodicMask)];
   [downCell setPeriodicDelay: 0.3 interval: 0.03];
-
-  leftCell = [NSButtonCell new];
-  [leftCell setHighlightsBy: NSChangeBackgroundCellMask | NSContentsCellMask];
-  [leftCell setImage: [NSImage imageNamed: @"common_ArrowLeft"]];
-  [leftCell setAlternateImage: [NSImage imageNamed: @"common_ArrowLeftH"]];
-  [leftCell setImagePosition: NSImageOnly];
+           
   [leftCell setContinuous: YES];
   [leftCell sendActionOn: (NSLeftMouseDownMask | NSPeriodicMask)];
   [leftCell setPeriodicDelay: 0.3 interval: 0.03];
-
-  rightCell = [NSButtonCell new];
-  [rightCell setHighlightsBy: NSChangeBackgroundCellMask | NSContentsCellMask];
-  [rightCell setImage: [NSImage imageNamed: @"common_ArrowRight"]];
-  [rightCell setAlternateImage: [NSImage imageNamed: @"common_ArrowRightH"]];
-  [rightCell setImagePosition: NSImageOnly];
+           
   [rightCell setContinuous: YES];
   [rightCell sendActionOn: (NSLeftMouseDownMask | NSPeriodicMask)];
   [rightCell setPeriodicDelay: 0.3 interval: 0.03];
-
-  knobCell = [NSButtonCell new];
-  [knobCell setButtonType: NSMomentaryChangeButton];
-  [knobCell setImage: [NSImage imageNamed: @"common_Dimple"]];
-  [knobCell setImagePosition: NSImageOnly];
-}
-
+  
+  [upCell setContinuous: YES];
+  [upCell sendActionOn: (NSLeftMouseDownMask | NSPeriodicMask)];
+  [upCell setPeriodicDelay: 0.3 interval: 0.03];
+}         
+          
 - (void) _setTargetAndActionToCells
 {
   [upCell setTarget: _target];
@@ -385,8 +402,11 @@ static NSColor *scrollBarColor = nil;
   [rightCell setTarget: _target];
   [rightCell setAction: _action];
 
-  [knobCell setTarget: _target];
-  [knobCell setAction: _action];
+  [horizontalKnobCell setTarget: _target];
+  [horizontalKnobCell setAction: _action];
+  
+  [verticalKnobCell setTarget:_target];
+  [horizontalKnobCell setTarget:_target];
 }
 
 - (void) checkSpaceForParts
@@ -677,8 +697,33 @@ static NSColor *scrollBarColor = nil;
 					   fromView: nil]];
 	  if (floatValue != _floatValue)
 	    {
-	      [self setFloatValue: floatValue];
-	      [self sendAction: _action to: _target];
+	      NSInterfaceStyle interfaceStyle;
+
+	      interfaceStyle
+		= NSInterfaceStyleForKey(@"NSScrollerInterfaceStyle", self);
+
+              if (interfaceStyle == NSNextStepInterfaceStyle 
+	        || interfaceStyle == GSWindowMakerInterfaceStyle)
+		{
+		  /* NeXTstep style is to scroll to point.
+		   */
+	          [self setFloatValue: floatValue];
+	          [self sendAction: _action to: _target];
+		}
+	      else
+		{
+		  /* Windows style is to scroll by a page.
+		   */
+		  if (floatValue > _floatValue)
+		    {
+		      _hitPart = NSScrollerIncrementPage;
+		    }
+		  else
+		    {
+		      _hitPart = NSScrollerDecrementPage;
+		    }
+	          [self sendAction: _action to: _target];
+		}
 	    }
 	  [self trackKnob: theEvent];
 	  break;
@@ -801,7 +846,7 @@ static NSColor *scrollBarColor = nil;
 
        /* Get the next event, blocking if necessary. */
        theEvent = [NSApp nextEventMatchingMask: eventMask
-                     untilDate: nil /* No limit, block until we get an event. */
+                     untilDate: [NSDate distantFuture] /* No limit, block until we get an event. */
                      inMode: NSEventTrackingRunLoopMode
                      dequeue: YES];
        eventType = [theEvent type];
@@ -888,6 +933,11 @@ static NSColor *scrollBarColor = nil;
   static NSRect rectForPartDecrementLine;
   static NSRect rectForPartKnobSlot;
 
+  if (upCell == nil)
+    {
+      [self drawParts];
+      [self checkSpaceForParts];
+    }
   if (_cacheValid == NO)
     {
       rectForPartIncrementLine = [self rectForPart: NSScrollerIncrementLine];
@@ -930,6 +980,11 @@ static NSColor *scrollBarColor = nil;
 	(whichButton == NSScrollerIncrementArrow ? "increment" : "decrement"),
 	rect.origin.x, rect.origin.y);
 
+  if (upCell == nil)
+    {
+      [self drawParts];
+      [self checkSpaceForParts];
+    }
   switch (whichButton)
     {
       case NSScrollerDecrementArrow:
@@ -948,20 +1003,37 @@ static NSColor *scrollBarColor = nil;
  */
 - (void) drawKnob
 {
-  [knobCell drawWithFrame: [self rectForPart: NSScrollerKnob] inView: self];
+  if (upCell == nil)
+    {
+      [self drawParts];
+      [self checkSpaceForParts];
+    }
+  if (_isHorizontal)
+    [horizontalKnobCell drawWithFrame: [self rectForPart: NSScrollerKnob]
+			       inView: self];
+  else
+    [verticalKnobCell drawWithFrame: [self rectForPart: NSScrollerKnob]
+			     inView: self];
 }
 
 - (void) drawKnobSlot
 {
   static NSRect rect;
 
+  if (upCell == nil)
+    {
+      [self drawParts];
+      [self checkSpaceForParts];
+    }
   if (_cacheValid == NO)
     {
       rect = [self rectForPart: NSScrollerKnobSlot];
     }
 
-  [scrollBarColor set];
-  NSRectFill (rect);
+  if (_isHorizontal)
+    [horizontalKnobSlotCell drawWithFrame: rect inView: self];
+  else
+    [verticalKnobSlotCell drawWithFrame: rect inView: self];
 }
 
 /**<p>Highlights the button whose under the mouse. Does nothing if the mouse
@@ -991,11 +1063,35 @@ static NSColor *scrollBarColor = nil;
 - (NSRect) rectForPart: (NSScrollerPart)partCode
 {
   NSRect scrollerFrame = _frame;
-  float x = 1, y = 1;
+  float x, y;
   float width, height;
-  float buttonsWidth = ([isa scrollerWidth] - buttonsOffset);
-  float buttonsSize = 2 * buttonsWidth + 2;
+  float buttonsWidth;
+  float buttonsSize;  
   NSUsableScrollerParts usableParts;
+
+  NSInterfaceStyle interfaceStyle = NSInterfaceStyleForKey(@"NSScrollerInterfaceStyle",self);
+
+  if (upCell == nil)
+    {
+      [self drawParts];
+      [self checkSpaceForParts];
+    }
+  /* We use the button offset if we in the NeXTstep interface style. */
+  if (interfaceStyle == NSNextStepInterfaceStyle
+    || interfaceStyle == GSWindowMakerInterfaceStyle)
+    { 
+      buttonsWidth = ([isa scrollerWidth] - buttonsOffset);
+      x = y = 1.0;
+      buttonsSize = 2 * buttonsWidth + 2;
+    }
+  else
+    {
+      buttonsWidth = [isa scrollerWidth];
+      x = y = 1.0;
+      buttonsSize = 2 * buttonsWidth;
+    }
+
+
   /*
    * If the scroller is disabled then the scroller buttons and the
    * knob are not displayed at all.
@@ -1045,7 +1141,7 @@ static NSColor *scrollBarColor = nil;
 
 	  /* calc the slot Height */
 	  slotHeight = height - (_arrowsPosition == NSScrollerArrowsNone
-				 ?  0 : buttonsSize);
+	    ?  0 : buttonsSize);
 	  knobHeight = _knobProportion * slotHeight;
 	  knobHeight = (float)floor(knobHeight);
 	  if (knobHeight < buttonsWidth)
@@ -1057,11 +1153,22 @@ static NSColor *scrollBarColor = nil;
 
 
 	  /* calc actual position */
-	  y += knobPosition + ((_arrowsPosition == NSScrollerArrowsMaxEnd
-			       || _arrowsPosition == NSScrollerArrowsNone)
-			       ?  0 : buttonsSize);
+          if (interfaceStyle == NSNextStepInterfaceStyle
+	    || interfaceStyle == GSWindowMakerInterfaceStyle)
+	    {
+	      y += knobPosition + ((_arrowsPosition == NSScrollerArrowsMaxEnd
+		|| _arrowsPosition == NSScrollerArrowsNone)
+		?  0 : buttonsSize);
+	      width = buttonsWidth;
+	    }
+          else
+	    {
+	      y += knobPosition + ((_arrowsPosition == NSScrollerArrowsNone)
+		? 0 : buttonsWidth); 
+	      width = buttonsWidth ;
+	    }
+
 	  height = knobHeight;
-	  width = buttonsWidth;
 	  break;
 	}
 
@@ -1076,11 +1183,19 @@ static NSColor *scrollBarColor = nil;
 	    break;
 	  }
 	height -= buttonsSize;
-	if (_arrowsPosition == NSScrollerArrowsMinEnd)
+	if ( (interfaceStyle == NSNextStepInterfaceStyle || 
+              interfaceStyle == GSWindowMakerInterfaceStyle) 
+            && _arrowsPosition == NSScrollerArrowsMinEnd)
 	  {
 	    y += buttonsSize;
 	  }
-	break;
+        else if (interfaceStyle != NSNextStepInterfaceStyle
+	  && interfaceStyle != GSWindowMakerInterfaceStyle)
+          {
+            y += buttonsWidth;
+            width = buttonsWidth;
+          }
+        break;
 
       case NSScrollerDecrementLine:
       case NSScrollerDecrementPage:
@@ -1089,7 +1204,9 @@ static NSColor *scrollBarColor = nil;
 	  {
 	    return NSZeroRect;
 	  }
-	else if (_arrowsPosition == NSScrollerArrowsMaxEnd)
+	else if ((interfaceStyle == NSNextStepInterfaceStyle
+	  || interfaceStyle == GSWindowMakerInterfaceStyle)
+	  && _arrowsPosition == NSScrollerArrowsMaxEnd)
 	  {
 	    y += (height - buttonsSize + 1);
 	  }
@@ -1104,14 +1221,22 @@ static NSColor *scrollBarColor = nil;
 	  {
 	    return NSZeroRect;
 	  }
-	else if (_arrowsPosition == NSScrollerArrowsMaxEnd)
-	  {
-	    y += (height - buttonsWidth);
-	  }
-	else if (_arrowsPosition == NSScrollerArrowsMinEnd)
-	  {
-	    y += (buttonsWidth + 1);
-	  }
+        else if (interfaceStyle == NSNextStepInterfaceStyle 
+	  || interfaceStyle == GSWindowMakerInterfaceStyle)
+          {
+	    if (_arrowsPosition == NSScrollerArrowsMaxEnd)
+	      {
+	        y += (height - buttonsWidth);
+	      }
+	    else if (_arrowsPosition == NSScrollerArrowsMinEnd)
+	      {
+	        y += (buttonsWidth + 1);
+	      }
+          }
+        else 
+          {
+            y += (height - buttonsWidth);
+          }
 	height = buttonsWidth;
 	width = buttonsWidth;
 	break;
@@ -1133,7 +1258,7 @@ static NSColor *scrollBarColor = nil;
 + (float) scrollerWidthForControlSize: (NSControlSize)controlSize
 {
   // FIXME
-  return scrollerWidth;
+  return [self scrollerWidth];
 }
 
 - (void) setControlSize: (NSControlSize)controlSize
