@@ -65,6 +65,7 @@
 #include "AppKit/NSWorkspace.h"
 #include "AppKit/NSApplication.h"
 #include "AppKit/NSImage.h"
+#include "AppKit/NSPasteboard.h"
 #include "AppKit/NSView.h"
 #include "AppKit/NSPanel.h"
 #include "AppKit/NSWindow.h"
@@ -72,6 +73,12 @@
 #include "GNUstepGUI/GSServicesManager.h"
 #include "GNUstepGUI/GSDisplayServer.h"
 
+/* Informal protocol for method to ask an app to open a URL.
+ */
+@interface NSObject (OpenURL)
+- (BOOL) application: (NSApplication*)a openURL: (NSURL*)u;
+@end
+
 /* Private method to check that a process exists.
  */
 @interface NSProcessInfo (Private)
@@ -803,7 +810,47 @@ static NSString			*_rootPath = @"/";
     }
   else
     {
-      return NO;
+      NSDictionary	*map;
+      NSString		*appName;
+      NSPasteboard      *pb;
+
+      /* Look up an application to handle this URL scheme.
+       * We get a dictionary containing all apps for the scheme.
+       * FIXME ... we just use the last app in the dictionary,
+       * but we really ought to have some way of saying which we want.
+       */
+      map = [applications objectForKey: @"GSSchemesMap"];
+      map = [map objectForKey: [[url scheme] lowercaseString]];
+      appName = [[map allKeys] lastObject];
+      if (appName != nil)
+	{
+	  id		app;
+
+	  /* Now try to get the application to open the URL.
+	   */
+	  app = GSContactApplication(appName, nil, nil);
+	  if (app != nil)
+	    {
+	      NS_DURING
+		{
+	          [app application: NSApp openURL: url];
+		}
+	      NS_HANDLER
+		{
+		  NSWarnLog(@"Failed to contact '%@' to open file", appName);
+		  return NO;
+		}
+	      NS_ENDHANDLER
+	      [NSApp deactivate];
+	      return YES;
+	    }
+	}
+      /* No application found to open the URL.
+       * Try any OpenURL service available.
+       */
+      pb = [NSPasteboard pasteboardWithUniqueName];
+      [url writeToPasteboard: pb];
+      return NSPerformService(@"OpenURL", pb);
     }
 }
 
@@ -1274,7 +1321,7 @@ inFileViewerRootedAtPath: (NSString*)rootFullpath
    */
   if (path == nil)
     {
-      path = [NSTask launchPathForTool: @"make_services"];
+      path = [[NSTask launchPathForTool: @"make_services"] retain];
     }
   task = [NSTask launchedTaskWithLaunchPath: path
 				  arguments: nil];
